/**
  ******************************************************************************
  * @file           : ms60f3xxx_crc.h
  * @author         : sinomcu.com-SA
  * @brief          : Header file of CRC module.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#ifndef MS60F3XXX_CRC_H
#define MS60F3XXX_CRC_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "ms60f3xxx.h"

/* Private types -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private constants ---------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
/* Exported types ------------------------------------------------------------*/
/** @defgroup CRC_Exported_Types CRC Exported Types
  * @{
  */

#define CRC32_INIT_VALUE   0xFFFFFFFF
#define CRC16_INIT_VALUE   0xFFFF
#define CRC8_INIT_VALUE    0xFF
#define IN_OUT_FORMAT_MASK  0x18
#define WIDTH_MASK    0x06
#define CRC_ENABLE_MASK 0x01
#define CRC_RESET_MASK 0x20
#define CRC_DEFAULT_CRC_INITVALUE       (0xFFFFFFFFu) /*!< Default CRC computation initialization value */



/** @defgroup CRC_EC_Default_Polynomial_Value    Default CRC generating polynomial value
  * @brief    Normal representation of this polynomial value is
  *           X^32 + X^26 + X^23 + X^22 + X^16 + X^12 + X^11 + X^10 +X^8 + X^7 + X^5 + X^4 + X^2 + X + 1 .
  * @{
  */
#define CRC_DEFAULT_CRC32_POLY          (0x04C11DB7u) /*!< Default CRC generating polynomial value */

typedef enum /**< \brief (CR) Control Register */
{
    IN_AND_OUT_NORMAL = 0,
    IN_NORMAL_OUT_REVERSE = (1 << 3),
    IN_REVERSE_OUT_NORMAL = (1 << 4),
    IN_REVERSE_OUT_REVERSE = (3 << 3),
} CRC_INOUT_Enum;


typedef enum /**< \brief Non register operation, only for flags  */
{
    CRC_8 = 0,
    CRC_16 = 2,
    CRC_32 = 4,
} CRC_WIDTH_Enum;


typedef enum /**< \brief (CR) Control Register */
{
    CRC_DISABLE = 0,
    CRC_ENABLE	= 1
} CRC_ENABLE_Enum;

typedef enum /**< \brief Non register operation, only for flags  */
{
    OF_IF = 1,
    MISMATCH_IF = 2,
    BLOCK_IF = 4,
    SINGLE_IF = 8
} CRC_STATUS_Enum;

/**
  * @brief CRC Init Structure definition
  */
typedef struct
{
    uint32_t InitValue;                 /*!< Init value to initiate CRC computation. No need to specify it if DefaultInitValueUse                                          is set to DEFAULT_INIT_VALUE_ENABLE.   */
    uint32_t POLY;                      /*!  CRC polymonial   for example crc16 x16+x12 +x5+1   poly=0x1021                                                                                                       */
    uint32_t CRC_XOR;                   /*   CRC XOR value register      */
    CRC_INOUT_Enum inout_type;         /*IN_AND_OUT_NORMAL =0,
                                      	IN_NORMAL_OUT_REVERSE=(1<<3),
	                                      IN_REVERSE_OUT_NORMAL=(1<<4),
	                                      IN_REVERSE_OUT_REVERSE=(3<<3),  */
    CRC_WIDTH_Enum width_type;       /* crc8 ,crc16,crc32   */
    CRC_ENABLE_Enum enable_type;  /*CRC enable  CRC_DISABLE=0, CRC_ENABLE	=1*/

} CRC_Init_T;


ErrorStatus CRC_Init(CRC_Init_T *CrcInitConfig);
uint32_t CRC_Calculate( CRC_WIDTH_Enum type, uint8_t *pBuffer, uint32_t BufferLength) ;
#ifdef __cplusplus
}
#endif

#endif

/******************************** END OF FILE *********************************/
