/**
  ******************************************************************************
  * @file           : ms60f3xxx_usart.h
  * @author         : sinomcu.com-SA
  * @brief          : Header file of usart module.
  * @gcc            : toolchain version 5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with 
	* basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#ifndef MS60F3XXX_USART_H
#define MS60F3XXX_USART_H

#ifdef __cplusplus
extern "C" {
#endif
	
/* =========================================================================================================================== */
/* ================                                          USART                                           ================ */
/* =========================================================================================================================== */
#include "ms60f3xxx.h"

#define MIN_CD_VALUE_SPI 4

/* ISO7816 protocol type. */
#define ISO7816_T_0   0
#define ISO7816_T_1   1

/* -------- USART_CR : (USART Offset: 0x0000) Control Register -------- */
#define USART_CR_RSTRX (0x1u << 2) /**< \brief (USART_CR) Reset Receiver */
#define USART_CR_RSTTX (0x1u << 3) /**< \brief (USART_CR) Reset Transmitter */
#define USART_CR_RXEN (0x1u << 4) /**< \brief (USART_CR) Receiver Enable */
#define USART_CR_RXDIS (0x1u << 5) /**< \brief (USART_CR) Receiver Disable */
#define USART_CR_TXEN (0x1u << 6) /**< \brief (USART_CR) Transmitter Enable */
#define USART_CR_TXDIS (0x1u << 7) /**< \brief (USART_CR) Transmitter Disable */
#define USART_CR_RSTSTA (0x1u << 8) /**< \brief (USART_CR) Reset Status Bits */
#define USART_CR_STTBRK (0x1u << 9) /**< \brief (USART_CR) Start Break */
#define USART_CR_STPBRK (0x1u << 10) /**< \brief (USART_CR) Stop Break */
#define USART_CR_STTTO (0x1u << 11) /**< \brief (USART_CR) Clear TIMEOUT Flag and Start Time-out After Next Character Received */
#define USART_CR_SENDA (0x1u << 12) /**< \brief (USART_CR) Send Address */
#define USART_CR_RSTIT (0x1u << 13) /**< \brief (USART_CR) Reset Iterations */
#define USART_CR_RSTNACK (0x1u << 14) /**< \brief (USART_CR) Reset Non Acknowledge */
#define USART_CR_RETTO (0x1u << 15) /**< \brief (USART_CR) Start Time-out Immediately */
#define USART_CR_RTSEN (0x1u << 18) /**< \brief (USART_CR) Request to Send Pin Control */
#define USART_CR_RTSDIS (0x1u << 19) /**< \brief (USART_CR) Request to Send Pin Control */
#define USART_CR_LINABT (0x1u << 20) /**< \brief (USART_CR) Abort LIN Transmission */
#define USART_CR_LINWKUP (0x1u << 21) /**< \brief (USART_CR) Send LIN Wakeup Signal */

#define USART_CR_RCS (0x1u << 18) /**< \brief (USART_CR) Releases the Client Select Line NSS (RTS pin) */
#define USART_CR_FCS (0x1u << 19) /**< \brief (USART_CR) Forces the Client Select Line NSS (RTS pin) to 0*/

/* -------- USART_MR : (USART Offset: 0x0004) Mode Register -------- */
#define USART_MR_USART_MODE_Pos 0
#define USART_MR_USART_MODE_Msk (0xfu << USART_MR_USART_MODE_Pos) /**< \brief (USART_MR) USART Mode of Operation */
#define USART_MR_USART_MODE(value) ((USART_MR_USART_MODE_Msk & ((value) << USART_MR_USART_MODE_Pos)))
#define USART_MR_USART_MODE_NORMAL (0x0u << 0) /**< \brief (USART_MR) Normal mode */
#define USART_MR_USART_MODE_RS485 (0x1u << 0) /**< \brief (USART_MR) RS485 */
#define USART_MR_USART_MODE_HW_HANDSHAKING (0x2u << 0) /**< \brief (USART_MR) Hardware Handshaking */
#define USART_MR_USART_MODE_IS07816_T_0 (0x4u << 0) /**< \brief (USART_MR) IS07816 Protocol: T = 0 */
#define USART_MR_USART_MODE_IS07816_T_1 (0x6u << 0) /**< \brief (USART_MR) IS07816 Protocol: T = 1 */
#define USART_MR_USART_MODE_IRDA (0x8u << 0) /**< \brief (USART_MR) IrDA */
#define USART_MR_USART_MODE_LIN_MASTER (0xAu << 0) /**< \brief (USART_MR) LIN Master mode */
#define USART_MR_USART_MODE_LIN_SLAVE (0xBu << 0) /**< \brief (USART_MR) LIN Slave mode */
#define USART_MR_USART_MODE_SPI_MASTER (0xEu << 0) /**< \brief (USART_MR) SPI master mode (CLKO must be written to 1 and USCLKS = 0, 1 or 2) */
#define USART_MR_USART_MODE_SPI_SLAVE (0xFu << 0) /**< \brief (USART_MR) SPI Slave mode */
#define USART_MR_CHRL_Pos 6
#define USART_MR_CHRL_Msk (0x3u << USART_MR_CHRL_Pos) /**< \brief (USART_MR) Character Length */
#define USART_MR_CHRL(value) ((USART_MR_CHRL_Msk & ((value) << USART_MR_CHRL_Pos)))
#define USART_MR_CHRL_5_BIT (0x0u << 6) /**< \brief (USART_MR) Character length is 5 bits */
#define USART_MR_CHRL_6_BIT (0x1u << 6) /**< \brief (USART_MR) Character length is 6 bits */
#define USART_MR_CHRL_7_BIT (0x2u << 6) /**< \brief (USART_MR) Character length is 7 bits */
#define USART_MR_CHRL_8_BIT (0x3u << 6) /**< \brief (USART_MR) Character length is 8 bits */
#define USART_MR_SYNC (0x1u << 8) /**< \brief (USART_MR) Synchronous Mode Select */
#define USART_MR_PAR_Pos 9
#define USART_MR_PAR_Msk (0x7u << USART_MR_PAR_Pos) /**< \brief (USART_MR) Parity Type */
#define USART_MR_PAR(value) ((USART_MR_PAR_Msk & ((value) << USART_MR_PAR_Pos)))
#define USART_MR_PAR_EVEN (0x0u << 9) /**< \brief (USART_MR) Even parity */
#define USART_MR_PAR_ODD (0x1u << 9) /**< \brief (USART_MR) Odd parity */
#define USART_MR_PAR_SPACE (0x2u << 9) /**< \brief (USART_MR) Parity forced to 0 (Space) */
#define USART_MR_PAR_MARK (0x3u << 9) /**< \brief (USART_MR) Parity forced to 1 (Mark) */
#define USART_MR_PAR_NO (0x4u << 9) /**< \brief (USART_MR) No parity */
#define USART_MR_PAR_MULTIDROP (0x6u << 9) /**< \brief (USART_MR) Multidrop mode */
#define USART_MR_NBSTOP_Pos 12
#define USART_MR_NBSTOP_Msk (0x3u << USART_MR_NBSTOP_Pos) /**< \brief (USART_MR) Number of Stop Bits */
#define USART_MR_NBSTOP(value) ((USART_MR_NBSTOP_Msk & ((value) << USART_MR_NBSTOP_Pos)))
#define USART_MR_NBSTOP_1_BIT (0x0u << 12) /**< \brief (USART_MR) 1 stop bit */
#define USART_MR_NBSTOP_1_5_BIT (0x1u << 12) /**< \brief (USART_MR) 1.5 stop bit (SYNC = 0) or reserved (SYNC = 1) */
#define USART_MR_NBSTOP_2_BIT (0x2u << 12) /**< \brief (USART_MR) 2 stop bits */
#define USART_MR_CHMODE_Pos 14
#define USART_MR_CHMODE_Msk (0x3u << USART_MR_CHMODE_Pos) /**< \brief (USART_MR) Channel Mode */
#define USART_MR_CHMODE(value) ((USART_MR_CHMODE_Msk & ((value) << USART_MR_CHMODE_Pos)))
#define USART_MR_CHMODE_NORMAL (0x0u << 14) /**< \brief (USART_MR) Normal mode */
#define USART_MR_CHMODE_AUTOMATIC (0x1u << 14) /**< \brief (USART_MR) Automatic Echo. Receiver input is connected to the TXD pin. */
#define USART_MR_CHMODE_LOCAL_LOOPBACK (0x2u << 14) /**< \brief (USART_MR) Local Loopback. Transmitter output is connected to the Receiver Input. */
#define USART_MR_CHMODE_REMOTE_LOOPBACK (0x3u << 14) /**< \brief (USART_MR) Remote Loopback. RXD pin is internally connected to the TXD pin. */
#define USART_MR_MSBF (0x1u << 16) /**< \brief (USART_MR) Bit Order */
#define USART_MR_MODE9 (0x1u << 17) /**< \brief (USART_MR) 9-bit Character Length */
#define USART_MR_CLKO (0x1u << 18) /**< \brief (USART_MR) Clock Output Select */
#define USART_MR_OVER (0x1u << 19) /**< \brief (USART_MR) Oversampling Mode */
#define USART_MR_INACK (0x1u << 20) /**< \brief (USART_MR) Inhibit Non Acknowledge */
#define USART_MR_DSNACK (0x1u << 21) /**< \brief (USART_MR) Disable Successive NACK */
#define USART_MR_VAR_SYNC (0x1u << 22) /**< \brief (USART_MR) Variable Synchronization of Command/Data Sync Start Frame Delimiter */
#define USART_MR_INVDATA (0x1u << 23) /**< \brief (USART_MR) Inverted Data */
#define USART_MR_MAX_ITERATION_Pos 24
#define USART_MR_MAX_ITERATION_Msk (0x7u << USART_MR_MAX_ITERATION_Pos) /**< \brief (USART_MR) Maximum Number of Automatic Iteration */
#define USART_MR_MAX_ITERATION(value) ((USART_MR_MAX_ITERATION_Msk & ((value) << USART_MR_MAX_ITERATION_Pos)))
#define USART_MR_FILTER (0x1u << 28) /**< \brief (USART_MR) Receive Line Filter */
#define USART_MR_MAN (0x1u << 29) /**< \brief (USART_MR) Manchester Encoder/Decoder Enable */
#define USART_MR_MODSYNC (0x1u << 30) /**< \brief (USART_MR) Manchester Synchronization Mode */
#define USART_MR_ONEBIT (0x1u << 31) /**< \brief (USART_MR) Start Frame Delimiter Selector */
#define USART_MR_CPHA (0x1u << 8) /**< \brief (USART_MR) SPI Clock Phase */
#define USART_MR_CPOL (0x1u << 16) /**< \brief (USART_MR) SPI Clock Polarity */
#define USART_MR_WRDBT (0x1u << 20) /**< \brief (USART_MR) Wait Read Data Before Transfer */

/* -------- USART_IER : (USART Offset: 0x0008) Interrupt Enable Register -------- */
#define USART_IER_RXRDY (0x1u << 0) /**< \brief (USART_IER) RXRDY Interrupt Enable */
#define USART_IER_TXRDY (0x1u << 1) /**< \brief (USART_IER) TXRDY Interrupt Enable */
#define USART_IER_RXBRK (0x1u << 2) /**< \brief (USART_IER) Receiver Break Interrupt Enable */
#define USART_IER_ENDRX (0x1u << 3) /**< \brief (USART_IER) End of Receive Buffer Interrupt Enable*/
#define USART_IER_ENDTX (0x1u << 4) /**< \brief (USART_IER) End of Transmit Buffer Interrupt Enable*/
#define USART_IER_OVRE (0x1u << 5) /**< \brief (USART_IER) Overrun Error Interrupt Enable */
#define USART_IER_FRAME (0x1u << 6) /**< \brief (USART_IER) Framing Error Interrupt Enable */
#define USART_IER_PARE (0x1u << 7) /**< \brief (USART_IER) Parity Error Interrupt Enable */
#define USART_IER_TIMEOUT (0x1u << 8) /**< \brief (USART_IER) Time-out Interrupt Enable */
#define USART_IER_TXEMPTY (0x1u << 9) /**< \brief (USART_IER) TXEMPTY Interrupt Enable */
#define USART_IER_ITER (0x1u << 10) /**< \brief (USART_IER) Max number of Repetitions Reached Interrupt Enable */
#define USART_IER_TXBUFE (0x1u << 11) /**< \brief (USART_IER) Transmit Buffer Empty Interrupt Enable  */
#define USART_IER_RXBUFF (0x1u << 12) /**< \brief (USART_IER)  Receive Buffer Full Interrupt Enable */
#define USART_IER_NACK (0x1u << 13) /**< \brief (USART_IER) Non Acknowledge Interrupt Enable */
#define USART_IER_CTSIC (0x1u << 19) /**< \brief (USART_IER) Clear to Send Input Change Interrupt Enable */
#define USART_IER_MANE (0x1u << 24) /**< \brief (USART_IER) Manchester Error Interrupt Enable */

#define USART_IER_UNRE (0x1u << 10) /**< \brief (USART_IER) SPI Underrun Error Interrupt Enable */

#define USART_IER_LINBK (0x1u << 13) /**< \brief (USART_IER) LIN Break Sent or LIN Break Received Interrupt Enable */
#define USART_IER_LINID (0x1u << 14) /**< \brief (USART_IER) LIN Identifier Sent or LIN Identifier Received Interrupt Enable */
#define USART_IER_LINTC (0x1u << 15) /**< \brief (USART_IER) LIN Transfer Completed Interrupt Enable */
#define USART_IER_LINBE (0x1u << 25) /**< \brief (USART_IER) LIN Bus Error Interrupt Enable */
#define USART_IER_LINISFE (0x1u << 26) /**< \brief (USART_IER) LIN Inconsistent Synch Field Error Interrupt Enable */
#define USART_IER_LINIPE (0x1u << 27) /**< \brief (USART_IER) LIN Identifier Parity Interrupt Enable */
#define USART_IER_LINCE (0x1u << 28) /**< \brief (USART_IER) LIN Checksum Error Interrupt Enable */
#define USART_IER_LINSNRE (0x1u << 29) /**< \brief (USART_IER) LIN Slave Not Responding Error Interrupt Enable */
#define USART_IER_LINSTE (0x1u << 30) /**< \brief (USART_IER) LIN Synch Tolerance Error Interrupt Enable */
#define USART_IER_LINHTE (0x1u << 31) /**< \brief (USART_IER) LIN Header Timeout Error Interrupt Enable */

/* -------- USART_IDR : (USART Offset: 0x000C) Interrupt Disable Register -------- */
#define USART_IDR_RXRDY (0x1u << 0) /**< \brief (USART_IDR) RXRDY Interrupt Disable */
#define USART_IDR_TXRDY (0x1u << 1) /**< \brief (USART_IDR) TXRDY Interrupt Disable */
#define USART_IDR_RXBRK (0x1u << 2) /**< \brief (USART_IDR) Receiver Break Interrupt Disable */
#define USART_IDR_ENDRX (0x1u << 3) /**< \brief (USART_IDR) End of Receive Buffer Interrupt Disable*/
#define USART_IDR_ENDTX (0x1u << 4) /**< \brief (USART_IDR) End of Transmit Buffer Interrupt Disable*/
#define USART_IDR_OVRE (0x1u << 5) /**< \brief (USART_IDR) Overrun Error Interrupt Enable */
#define USART_IDR_FRAME (0x1u << 6) /**< \brief (USART_IDR) Framing Error Interrupt Disable */
#define USART_IDR_PARE (0x1u << 7) /**< \brief (USART_IDR) Parity Error Interrupt Disable */
#define USART_IDR_TIMEOUT (0x1u << 8) /**< \brief (USART_IDR) Time-out Interrupt Disable */
#define USART_IDR_TXEMPTY (0x1u << 9) /**< \brief (USART_IDR) TXEMPTY Interrupt Disable */
#define USART_IDR_ITER (0x1u << 10) /**< \brief (USART_IDR) Max Number of Repetitions Reached Interrupt Disable */
#define USART_IDR_TXBUFE (0x1u << 11) /**< \brief (USART_IDR) Transmit Buffer Empty Interrupt Disable */
#define USART_IDR_RXBUFF (0x1u << 12) /**< \brief (USART_IDR)  Receive Buffer Full Interrupt Disable */
#define USART_IDR_NACK (0x1u << 13) /**< \brief (USART_IDR) Non Acknowledge Interrupt Disable */
#define USART_IDR_CTSIC (0x1u << 19) /**< \brief (USART_IDR) Clear to Send Input Change Interrupt Disable */
#define USART_IDR_MANE (0x1u << 24) /**< \brief (USART_IDR) Manchester Error Interrupt Disable */

#define USART_IDR_UNRE (0x1u << 10) /**< \brief (USART_IDR) SPI Underrun Error Interrupt Disable */

#define USART_IDR_LINBK (0x1u << 13) /**< \brief (USART_IDR) LIN Break Sent or LIN Break Received Interrupt Disable */
#define USART_IDR_LINID (0x1u << 14) /**< \brief (USART_IDR) LIN Identifier Sent or LIN Identifier Received Interrupt Disable */
#define USART_IDR_LINTC (0x1u << 15) /**< \brief (USART_IDR) LIN Transfer Completed Interrupt Disable */
#define USART_IDR_LINBE (0x1u << 25) /**< \brief (USART_IDR) LIN Bus Error Interrupt Disable */
#define USART_IDR_LINISFE (0x1u << 26) /**< \brief (USART_IDR) LIN Inconsistent Synch Field Error Interrupt Disable */
#define USART_IDR_LINIPE (0x1u << 27) /**< \brief (USART_IDR) LIN Identifier Parity Interrupt Disable */
#define USART_IDR_LINCE (0x1u << 28) /**< \brief (USART_IDR) LIN Checksum Error Interrupt Disable */
#define USART_IDR_LINSNRE (0x1u << 29) /**< \brief (USART_IDR) LIN Slave Not Responding Error Interrupt Disable */
#define USART_IDR_LINSTE (0x1u << 30) /**< \brief (USART_IDR) LIN Synch Tolerance Error Interrupt Disable */
#define USART_IDR_LINHTE (0x1u << 31) /**< \brief (USART_IDR) LIN Header Timeout Error Interrupt Disable */

/* -------- USART_IMR : (USART Offset: 0x0010) Interrupt Mask Register -------- */
#define USART_IMR_RXRDY (0x1u << 0) /**< \brief (USART_IMR) RXRDY Interrupt Mask */
#define USART_IMR_TXRDY (0x1u << 1) /**< \brief (USART_IMR) TXRDY Interrupt Mask */
#define USART_IMR_RXBRK (0x1u << 2) /**< \brief (USART_IMR) Receiver Break Interrupt Mask */
#define USART_IMR_ENDRX (0x1u << 3) /**< \brief (USART_IMR) End of Receive Buffer Interrupt Mask*/
#define USART_IMR_ENDTX (0x1u << 4) /**< \brief (USART_IMR) End of Transmit Buffer Interrupt Maske*/
#define USART_IMR_OVRE (0x1u << 5) /**< \brief (USART_IMR) Overrun Error Interrupt Mask */
#define USART_IMR_FRAME (0x1u << 6) /**< \brief (USART_IMR) Framing Error Interrupt Mask */
#define USART_IMR_PARE (0x1u << 7) /**< \brief (USART_IMR) Parity Error Interrupt Mask */
#define USART_IMR_TIMEOUT (0x1u << 8) /**< \brief (USART_IMR) Time-out Interrupt Mask */
#define USART_IMR_TXEMPTY (0x1u << 9) /**< \brief (USART_IMR) TXEMPTY Interrupt Mask */
#define USART_IMR_ITER (0x1u << 10) /**< \brief (USART_IMR) Max Number of Repetitions Reached Interrupt Mask */
#define USART_IMR_TXBUFE (0x1u << 11) /**< \brief (USART_IMR) Transmit Buffer Empty Interrupt Mask */
#define USART_IMR_RXBUFF (0x1u << 12) /**< \brief (USART_IMR)  Receive Buffer Full Interrupt Mask */
#define USART_IMR_NACK (0x1u << 13) /**< \brief (USART_IMR) Non Acknowledge Interrupt Mask */
#define USART_IMR_CTSIC (0x1u << 19) /**< \brief (USART_IMR) Clear to Send Input Change Interrupt Mask */
#define USART_IMR_MANE (0x1u << 24) /**< \brief (USART_IMR) Manchester Error Interrupt Mask */

#define USART_IMR_UNRE (0x1u << 10) /**< \brief (USART_IMR) SPI Underrun Error Interrupt Mask */

#define USART_IMR_LINBK (0x1u << 13) /**< \brief (USART_IMR) LIN Break Sent or LIN Break Received Interrupt Mask */
#define USART_IMR_LINID (0x1u << 14) /**< \brief (USART_IMR) LIN Identifier Sent or LIN Identifier Received Interrupt Mask */
#define USART_IMR_LINTC (0x1u << 15) /**< \brief (USART_IMR) LIN Transfer Completed Interrupt Mask */
#define USART_IMR_LINBE (0x1u << 25) /**< \brief (USART_IMR) LIN Bus Error Interrupt Mask */
#define USART_IMR_LINISFE (0x1u << 26) /**< \brief (USART_IMR) LIN Inconsistent Synch Field Error Interrupt Mask */
#define USART_IMR_LINIPE (0x1u << 27) /**< \brief (USART_IMR) LIN Identifier Parity Interrupt Mask */
#define USART_IMR_LINCE (0x1u << 28) /**< \brief (USART_IMR) LIN Checksum Error Interrupt Mask */
#define USART_IMR_LINSNRE (0x1u << 29) /**< \brief (USART_IMR) LIN Slave Not Responding Error Interrupt Mask */
#define USART_IMR_LINSTE (0x1u << 30) /**< \brief (USART_IMR) LIN Synch Tolerance Error Interrupt Mask */
#define USART_IMR_LINHTE (0x1u << 31) /**< \brief (USART_IMR) LIN Header Timeout Error Interrupt Mask */

/* -------- USART_SR : (USART Offset: 0x0014) Channel Status Register -------- */
#define USART_SR_RXRDY (0x1u << 0) /**< \brief (USART_SR) Receiver Ready (cleared by reading USART_RHR) */
#define USART_SR_TXRDY (0x1u << 1) /**< \brief (USART_SR) Transmitter Ready (cleared by writing USART_THR) */
#define USART_SR_RXBRK (0x1u << 2) /**< \brief (USART_SR) Break Received/End of Break (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_ENDRX (0x1u << 3) /**< \brief (USART_SR) End of Receive Buffer Interrupt*/
#define USART_SR_ENDTX (0x1u << 4) /**< \brief (USART_SR) End of Transmit Buffer Interrupt*/
#define USART_SR_OVRE (0x1u << 5) /**< \brief (USART_SR) Overrun Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_FRAME (0x1u << 6) /**< \brief (USART_SR) Framing Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_PARE (0x1u << 7) /**< \brief (USART_SR) Parity Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_TIMEOUT (0x1u << 8) /**< \brief (USART_SR) Receiver Time-out (cleared by writing a one to bit USART_CR.STTTO) */
#define USART_SR_TXEMPTY (0x1u << 9) /**< \brief (USART_SR) Transmitter Empty (cleared by writing USART_THR) */
#define USART_SR_ITER (0x1u << 10) /**< \brief (USART_SR) Max Number of Repetitions Reached (cleared by writing a one to bit USART_CR.RSTIT) */
#define USART_SR_TXBUFE (0x1u << 11) /**< \brief (USART_SR) Transmit Buffer Empty*/
#define USART_SR_RXBUFF (0x1u << 12) /**< \brief (USART_SR)  Receive Buffer Full */
#define USART_SR_NACK (0x1u << 13) /**< \brief (USART_SR) Non Acknowledge Interrupt (cleared by writing a one to bit USART_CR.RSTNACK) */
#define USART_SR_CTSIC (0x1u << 19) /**< \brief (USART_SR) Clear to Send Input Change Flag (cleared on read) */
#define USART_SR_CTS (0x1u << 23) /**< \brief (USART_SR) Image of CTS Input */
#define USART_SR_MANERR (0x1u << 24) /**< \brief (USART_SR) Manchester Error (cleared by writing a one to the bit USART_CR.RSTSTA) */

#define USART_SR_UNRE (0x1u << 10) /**< \brief (USART_SR) Underrun Error (cleared by writing a one to bit USART_CR.RSTSTA) */

#define USART_SR_LINBK (0x1u << 13) /**< \brief (USART_SR) LIN Break Sent or LIN Break Received (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINID (0x1u << 14) /**< \brief (USART_SR) LIN Identifier Sent or LIN Identifier Received (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINTC (0x1u << 15) /**< \brief (USART_SR) LIN Transfer Completed (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINBLS (0x1u << 23) /**< \brief (USART_SR) LIN Bus Line Status */
#define USART_SR_LINBE (0x1u << 25) /**< \brief (USART_SR) LIN Bit Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINISFE (0x1u << 26) /**< \brief (USART_SR) LIN Inconsistent Synch Field Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINIPE (0x1u << 27) /**< \brief (USART_SR) LIN Identifier Parity Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINCE (0x1u << 28) /**< \brief (USART_SR) LIN Checksum Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINSNRE (0x1u << 29) /**< \brief (USART_SR) LIN Slave Not Responding Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINSTE (0x1u << 30) /**< \brief (USART_SR) LIN Synch Tolerance Error (cleared by writing a one to bit USART_CR.RSTSTA) */
#define USART_SR_LINHTE (0x1u << 31) /**< \brief (USART_SR) LIN Header Timeout Error (cleared by writing a one to bit USART_CR.RSTSTA) */

/* -------- USART_RHR : (USART Offset: 0x0018) Receive Holding Register -------- */
#define USART_RHR_RXCHR_Pos 0
#define USART_RHR_RXCHR_Msk (0x1FFu << USART_RHR_RXCHR_Pos) /**< \brief (USART_RHR) Received Character */
#define USART_RHR_RXSYNH (0x1u << 15) /**< \brief (USART_RHR) Received Sync */
/* -------- USART_THR : (USART Offset: 0x001C) Transmit Holding Register -------- */
#define USART_THR_TXCHR_Pos 0
#define USART_THR_TXCHR_Msk (0x1FFu << USART_THR_TXCHR_Pos) /**< \brief (USART_THR) Character to be Transmitted */
#define USART_THR_TXCHR(value) ((USART_THR_TXCHR_Msk & ((value) << USART_THR_TXCHR_Pos)))
#define USART_THR_TXSYNH (0x1u << 15) /**< \brief (USART_THR) Sync Field to be Transmitted */
/* -------- USART_BRGR : (USART Offset: 0x0020) Baud Rate Generator Register -------- */
#define USART_BRGR_CD_Pos 0
#define USART_BRGR_CD_Msk (0xFFFFu << USART_BRGR_CD_Pos) /**< \brief (USART_BRGR) Clock Divider */
#define USART_BRGR_CD(value) ((USART_BRGR_CD_Msk & ((value) << USART_BRGR_CD_Pos)))
#define USART_BRGR_FP_Pos 16
#define USART_BRGR_FP_Msk (0x7u << USART_BRGR_FP_Pos) /**< \brief (USART_BRGR) Fractional Part */
#define USART_BRGR_FP(value) ((USART_BRGR_FP_Msk & ((value) << USART_BRGR_FP_Pos)))
/* -------- USART_RTOR : (USART Offset: 0x0024) Receiver Time-out Register -------- */
#define USART_RTOR_TO_Pos 0
#define USART_RTOR_TO_Msk (0x1FFFFu << USART_RTOR_TO_Pos) /**< \brief (USART_RTOR) Time-out Value */
#define USART_RTOR_TO(value) ((USART_RTOR_TO_Msk & ((value) << USART_RTOR_TO_Pos)))
/* -------- USART_TTGR : (USART Offset: 0x0028) Transmitter Timeguard Register -------- */
#define USART_TTGR_TG_Pos 0
#define USART_TTGR_TG_Msk (0xFFu << USART_TTGR_TG_Pos) /**< \brief (USART_TTGR) Timeguard Value */
#define USART_TTGR_TG(value) ((USART_TTGR_TG_Msk & ((value) << USART_TTGR_TG_Pos)))
/* -------- USART_FIDI : (USART Offset: 0x0040) FI DI Ratio Register -------- */
#define USART_FIDI_FI_DI_RATIO_Pos 0
#define USART_FIDI_FI_DI_RATIO_Msk (0xFFFFu << USART_FIDI_FI_DI_RATIO_Pos) /**< \brief (USART_FIDI) FI Over DI Ratio Value */
#define USART_FIDI_FI_DI_RATIO(value) ((USART_FIDI_FI_DI_RATIO_Msk & ((value) << USART_FIDI_FI_DI_RATIO_Pos)))
/* -------- USART_NER : (USART Offset: 0x0044) Number of Errors Register -------- */
#define USART_NER_NB_ERRORS_Pos 0
#define USART_NER_NB_ERRORS_Msk (0xFFu << USART_NER_NB_ERRORS_Pos) /**< \brief (USART_NER) Number of Errors */
/* -------- USART_IF : (USART Offset: 0x004C) IrDA Filter Register -------- */
#define USART_IF_IRDA_FILTER_Pos 0
#define USART_IF_IRDA_FILTER_Msk (0xFFu << USART_IF_IRDA_FILTER_Pos) /**< \brief (USART_IF) IrDA Filter */
#define USART_IF_IRDA_FILTER(value) ((USART_IF_IRDA_FILTER_Msk & ((value) << USART_IF_IRDA_FILTER_Pos)))
/* -------- USART_MAN : (USART Offset: 0x0050) Manchester Configuration Register -------- */
#define USART_MAN_TX_PL_Pos 0
#define USART_MAN_TX_PL_Msk (0xFu << USART_MAN_TX_PL_Pos) /**< \brief (USART_MAN) Transmitter Preamble Length */
#define USART_MAN_TX_PL(value) ((USART_MAN_TX_PL_Msk & ((value) << USART_MAN_TX_PL_Pos)))
#define USART_MAN_TX_PP_Pos 8
#define USART_MAN_TX_PP_Msk (0x3u << USART_MAN_TX_PP_Pos) /**< \brief (USART_MAN) Transmitter Preamble Pattern */
#define USART_MAN_TX_PP(value) ((USART_MAN_TX_PP_Msk & ((value) << USART_MAN_TX_PP_Pos)))
#define USART_MAN_TX_PP_ALL_ONE (0x0u << 8) /**< \brief (USART_MAN) The preamble is composed of '1's */
#define USART_MAN_TX_PP_ALL_ZERO (0x1u << 8) /**< \brief (USART_MAN) The preamble is composed of '0's */
#define USART_MAN_TX_PP_ZERO_ONE (0x2u << 8) /**< \brief (USART_MAN) The preamble is composed of '01's */
#define USART_MAN_TX_PP_ONE_ZERO (0x3u << 8) /**< \brief (USART_MAN) The preamble is composed of '10's */
#define USART_MAN_TX_MPOL (0x1u << 12) /**< \brief (USART_MAN) Transmitter Manchester Polarity */
#define USART_MAN_RX_PL_Pos 16
#define USART_MAN_RX_PL_Msk (0xFu << USART_MAN_RX_PL_Pos) /**< \brief (USART_MAN) Receiver Preamble Length */
#define USART_MAN_RX_PL(value) ((USART_MAN_RX_PL_Msk & ((value) << USART_MAN_RX_PL_Pos)))
#define USART_MAN_RX_PP_Pos 24
#define USART_MAN_RX_PP_Msk (0x3u << USART_MAN_RX_PP_Pos) /**< \brief (USART_MAN) Receiver Preamble Pattern detected */
#define USART_MAN_RX_PP(value) ((USART_MAN_RX_PP_Msk & ((value) << USART_MAN_RX_PP_Pos)))
#define   USART_MAN_RX_PP_ALL_ONE (0x0u << 24) /**< \brief (USART_MAN) The preamble is composed of '1's */
#define   USART_MAN_RX_PP_ALL_ZERO (0x1u << 24) /**< \brief (USART_MAN) The preamble is composed of '0's */
#define   USART_MAN_RX_PP_ZERO_ONE (0x2u << 24) /**< \brief (USART_MAN) The preamble is composed of '01's */
#define   USART_MAN_RX_PP_ONE_ZERO (0x3u << 24) /**< \brief (USART_MAN) The preamble is composed of '10's */
#define USART_MAN_RX_MPOL (0x1u << 28) /**< \brief (USART_MAN) Receiver Manchester Polarity */
#define USART_MAN_ONE (0x1u << 29) /**< \brief (USART_MAN) Must Be Set to 1 */
#define USART_MAN_DRIFT (0x1u << 30) /**< \brief (USART_MAN) Drift Compensation */
/* -------- USART_LINMR : (USART Offset: 0x0054) LIN Mode Register -------- */
#define USART_LINMR_NACT_Pos 0
#define USART_LINMR_NACT_Msk (0x3u << USART_LINMR_NACT_Pos) /**< \brief (USART_LINMR) LIN Node Action */
#define USART_LINMR_NACT(value) ((USART_LINMR_NACT_Msk & ((value) << USART_LINMR_NACT_Pos)))
#define USART_LINMR_NACT_PUBLISH (0x0u << 0) /**< \brief (USART_LINMR) The USART transmits the response. */
#define USART_LINMR_NACT_SUBSCRIBE (0x1u << 0) /**< \brief (USART_LINMR) The USART receives the response. */
#define USART_LINMR_NACT_IGNORE (0x2u << 0) /**< \brief (USART_LINMR) The USART does not transmit and does not receive the response. */
#define USART_LINMR_PARDIS (0x1u << 2) /**< \brief (USART_LINMR) Parity Disable */
#define USART_LINMR_CHKDIS (0x1u << 3) /**< \brief (USART_LINMR) Checksum Disable */
#define USART_LINMR_CHKTYP (0x1u << 4) /**< \brief (USART_LINMR) Checksum Type */
#define USART_LINMR_DLM (0x1u << 5) /**< \brief (USART_LINMR) Data Length Mode */
#define USART_LINMR_DLM_DLC 0x0u /**< \brief (USART_LINMR) Data Length  DLC register. */
#define USART_LINMR_DLM_IDCHR (0x1u << 5) /**< \brief (USART_LINMR) Data Length  LINIR.IDCHR register. */
#define USART_LINMR_FSDIS (0x1u << 6) /**< \brief (USART_LINMR) Frame Slot Mode Disable */
#define USART_LINMR_WKUPTYP (0x1u << 7) /**< \brief (USART_LINMR) Wakeup Signal Type */
#define USART_LINMR_DLC_Pos 8
#define USART_LINMR_DLC_Msk (0xFFu << USART_LINMR_DLC_Pos) /**< \brief (USART_LINMR) Data Length Control */
#define USART_LINMR_DLC(value) ((USART_LINMR_DLC_Msk & ((value) << USART_LINMR_DLC_Pos)))
#define USART_LINMR_SYNCDIS (0x1u << 17) /**< \brief (USART_LINMR) Synchronization Disable */
/* -------- USART_LINIR : (USART OFFset: 0x0058) LIN Identifier Register -------- */
#define USART_LINIR_IDCHR_Pos 0
#define USART_LINIR_IDCHR_Msk (0xFFu << USART_LINIR_IDCHR_Pos) /**< \brief (USART_LINIR) Identifier Character */
#define USART_LINIR_IDCHR(value) ((USART_LINIR_IDCHR_Msk & ((value) << USART_LINIR_IDCHR_Pos)))
/* -------- USART_LINBRR : (USART Offset: 0x005C) LIN Baud Rate Register -------- */
#define USART_LINBRR_LINCD_Pos 0
#define USART_LINBRR_LINCD_Msk (0xFFFFu << USART_LINBRR_LINCD_Pos) /**< \brief (USART_LINBRR) Clock Divider after Synchronization */
#define USART_LINBRR_LINFP_Pos 16
#define USART_LINBRR_LINFP_Msk (0x7u << USART_LINBRR_LINFP_Pos) /**< \brief (USART_LINBRR) Fractional Part after Synchronization */

/* -------- USART_WPMR : (USART Offset: 0x00E4) Write Protection Mode Register -------- */
#define USART_WPMR_WPEN (0x1u << 0) /**< \brief (USART_WPMR) Write Protection Enable */
#define USART_WPMR_WPKEY_Pos 8
#define USART_WPMR_WPKEY_Msk (0xFFFFFFu << USART_WPMR_WPKEY_Pos) /**< \brief (USART_WPMR) Write Protection Key */
#define USART_WPMR_WPKEY(value) ((USART_WPMR_WPKEY_Msk & ((value) << USART_WPMR_WPKEY_Pos)))
#define USART_WPMR_WPKEY_PASSWD (0x555341u << 8) /**< \brief (USART_WPMR) Writing any other value in this field aborts the write operation of the WPEN bit. Always reads as 0. */
/* -------- USART_WPSR : (USART Offset: 0x00E8) Write Protection Status Register -------- */
#define USART_WPSR_WPVS (0x1u << 0) /**< \brief (USART_WPSR) Write Protection Violation Status */
#define USART_WPSR_WPVSRC_Pos 8
#define USART_WPSR_WPVSRC_Msk (0xFFFFu << USART_WPSR_WPVSRC_Pos) /**< \brief (USART_WPSR) Write Protection Violation Source */

/* -------- USART_RPR : (USART Offset: 0x100) Receive Pointer Register -------- */
#define USART_RPR_RXPTR_Pos 0
#define USART_RPR_RXPTR_Msk (0xFFFFFFFFu << USART_RPR_RXPTR_Pos) /**< \brief (USART_RPR) Receive Pointer Register */
#define USART_RPR_RXPTR(value) ((USART_RPR_RXPTR_Msk & ((value) << USART_RPR_RXPTR_Pos)))
/* -------- USART_RCR : (USART Offset: 0x104) Receive Counter Register -------- */
#define USART_RCR_RXCTR_Pos 0
#define USART_RCR_RXCTR_Msk (0xFFFFu << USART_RCR_RXCTR_Pos) /**< \brief (USART_RCR) Receive Counter Register */
#define USART_RCR_RXCTR(value) ((USART_RCR_RXCTR_Msk & ((value) << USART_RCR_RXCTR_Pos)))
/* -------- USART_TPR : (USART Offset: 0x108) Transmit Pointer Register -------- */
#define USART_TPR_TXPTR_Pos 0
#define USART_TPR_TXPTR_Msk (0xFFFFFFFFu << USART_TPR_TXPTR_Pos) /**< \brief (USART_TPR) Transmit Counter Register */
#define USART_TPR_TXPTR(value) ((USART_TPR_TXPTR_Msk & ((value) << USART_TPR_TXPTR_Pos)))
/* -------- USART_TCR : (USART Offset: 0x10C) Transmit Counter Register -------- */
#define USART_TCR_TXCTR_Pos 0
#define USART_TCR_TXCTR_Msk (0xFFFFu << USART_TCR_TXCTR_Pos) /**< \brief (USART_TCR) Transmit Counter Register */
#define USART_TCR_TXCTR(value) ((USART_TCR_TXCTR_Msk & ((value) << USART_TCR_TXCTR_Pos)))
/* -------- USART_RNPR : (USART Offset: 0x110) Receive Next Pointer Register -------- */
#define USART_RNPR_RXNPTR_Pos 0
#define USART_RNPR_RXNPTR_Msk (0xFFFFFFFFu << USART_RNPR_RXNPTR_Pos) /**< \brief (USART_RNPR) Receive Next Pointer */
#define USART_RNPR_RXNPTR(value) ((USART_RNPR_RXNPTR_Msk & ((value) << USART_RNPR_RXNPTR_Pos)))
/* -------- USART_RNCR : (USART Offset: 0x114) Receive Next Counter Register -------- */
#define USART_RNCR_RXNCTR_Pos 0
#define USART_RNCR_RXNCTR_Msk (0xFFFFu << USART_RNCR_RXNCTR_Pos) /**< \brief (USART_RNCR) Receive Next Counter */
#define USART_RNCR_RXNCTR(value) ((USART_RNCR_RXNCTR_Msk & ((value) << USART_RNCR_RXNCTR_Pos)))
/* -------- USART_TNPR : (USART Offset: 0x118) Transmit Next Pointer Register -------- */
#define USART_TNPR_TXNPTR_Pos 0
#define USART_TNPR_TXNPTR_Msk (0xFFFFFFFFu << USART_TNPR_TXNPTR_Pos) /**< \brief (USART_TNPR) Transmit Next Pointer */
#define USART_TNPR_TXNPTR(value) ((USART_TNPR_TXNPTR_Msk & ((value) << USART_TNPR_TXNPTR_Pos)))
/* -------- USART_TNCR : (USART Offset: 0x11C) Transmit Next Counter Register -------- */
#define USART_TNCR_TXNCTR_Pos 0
#define USART_TNCR_TXNCTR_Msk (0xFFFFu << USART_TNCR_TXNCTR_Pos) /**< \brief (USART_TNCR) Transmit Counter Next */
#define USART_TNCR_TXNCTR(value) ((USART_TNCR_TXNCTR_Msk & ((value) << USART_TNCR_TXNCTR_Pos)))
/* -------- USART_PTCR : (USART Offset: 0x120) Transfer Control Register -------- */
#define USART_PTCR_RXTEN (0x1u << 0) /**< \brief (USART_PTCR) Receiver Transfer Enable */
#define USART_PTCR_RXTDIS (0x1u << 1) /**< \brief (USART_PTCR) Receiver Transfer Disable */
#define USART_PTCR_TXTEN (0x1u << 8) /**< \brief (USART_PTCR) Transmitter Transfer Enable */
#define USART_PTCR_TXTDIS (0x1u << 9) /**< \brief (USART_PTCR) Transmitter Transfer Disable */
/* -------- USART_PTSR : (USART Offset: 0x124) Transfer Status Register -------- */
#define USART_PTSR_RXTEN (0x1u << 0) /**< \brief (USART_PTSR) Receiver Transfer Enable */
#define USART_PTSR_TXTEN (0x1u << 8) /**< \brief (USART_PTSR) Transmitter Transfer Enable */

typedef enum /**< \brief (USART_MR.MODE) USART Mode of Operation */
{
    NORMAL_MODE            = 0u,
    RS485_MODE             = 1u,
    HW_HANDSHAKING_MODE    = 2u,
    MODEM_MODE             = 3u,
    IS07816_T_0_MODE       = 4u,
    IS07816_T_1_MODE       = 6u,
    IRDA_MODE              = 8u,
    LIN_MASTER_MODE        = 0x0Au,
    LIN_SLAVE_MODE         = 0x0Bu,
    SPI_MASTER_MODE        = 0x0Eu,
    SPI_SLAVE_MODE         = 0x0Fu
} USART_MODE_Enum;

typedef enum /**< \brief (USART_MR.CPHA/CPOL)  SPI Clock Phase and SPI Clock Polarity */
{
    SPI_MODE_0 = USART_MR_CPHA,
    SPI_MODE_1 = 0,
    SPI_MODE_2 = USART_MR_CPHA | USART_MR_CPOL,
    SPI_MODE_3 = USART_MR_CPOL,
} USART_SPI_MODE_Enum;
typedef enum /**< \brief (USART_MR.CHRL) Character Length. */
{
    USART_DATA_5_BIT = USART_MR_CHRL_5_BIT,
    USART_DATA_6_BIT = USART_MR_CHRL_6_BIT,
    USART_DATA_7_BIT = USART_MR_CHRL_7_BIT,
    USART_DATA_8_BIT = USART_MR_CHRL_8_BIT,
} USART_DataLen_Enum;
typedef enum /**< \brief MANCHEST or Normal encode. */
{
    Normal_Encode      = 0,
    Manchester_Encode  = (s32)0xA0000000UL,
} USART_TransEncode_Enum;
typedef enum /**< \brief (USART_MR.SYNC) Synchronous Mode Select. */ 
{
    Asynchronous = 0,
    Synchronous  = (0x1u << 8),
} USART_SYNCMODE_Enum;

typedef enum /**< \brief (USART_MR.PAR) Parity Type. */ 
{
    USART_PARITY_EVEN      = 0,              /* Even parity */
    USART_PARITY_ODD       = (0x1u << 9),    /* Odd parity */
    USART_PARITY_SPACE     = (0x2u << 9),    /* Parity forced to 0 (Space) */
    USART_PARITY_MARK      = (0x3u << 9),    /* Parity forced to 1 (Mark) */
    USART_PARITY_NO        = (0x4u << 9),    /* No parity */
    USART_PARITY_MULTIDROP = (0x6u << 9),    /* Multidrop mode */
} USART_Parity_Enum;

typedef enum /**< \brief (USART_MR.CHMODE) Channel Mode. */ 
{
    USART_NORMAL          = 0,               /* Normal Mode */
    USART_AUTOMATIC       = (0x1u << 14),    /* Automatic Echo,Receiver input is connected to the TXD pin. */
    USART_LOCAL_LOOPBACK  = (0x2u << 14),    /* Local Loopback, //Transmitter output is connected to the Receiver Input. */
    USART_REMOTE_LOOPBACK = (0x3u << 14)     /* Remote Loopback. RXD pin is internally connected to the TXD pin */
} USART_CHMODE_Enum;

typedef enum /**< \brief (USART_MR.MSBF)  Bit Order. */
{
    LSB_FIRST = 0,
    MSB_FIRST = (0x1u << 16),
} USART_BitOrder_Enum;

typedef enum /**< \brief (USART_MR.NBSTOP)  Number of Stop Bits. */ 
{
    USART_STOPBITS_1   = 0,            /* 1 stop bit */
    USART_STOPBITS_1_5 = (1u << 12),   /* 1.5 stop bit (SYNC = 0) or reserved (SYNC = 1) */
    USART_STOPBITS_2   = (2u << 12),   /* 2 stop bit */
} USART_StopBIT_Enum;

typedef enum /**< \brief (USART_MR.OVER)  Oversampling Mode. */ 
{
    USART_OVERSAMPLING_16 = 0,            
    USART_OVERSAMPLING_8  = (1u << 19),
} USART_OverSampe_Enum;

typedef struct
{
    uint32_t             BaudRate;            /*!< This member configures the USART communication baud rate.
                                                The baud rate is computed using the following formula:
                                                - IntegerDivider = ((PCLKx) / (16 * (USART_InitStruct->USART_BaudRate)                                            
	                                              - FractionalDivider = ((IntegerDivider - ((uint32_t) IntegerDivider)) * 16) + 0.5 */
    uint32_t             IrDA_Filter;
    uint16_t             TimeOut;

    USART_StopBIT_Enum   StopBits;            /*!< Specifies the number of stop bits transmitted.
                                               This parameter can be a value of @ref USART_Stop_Bits */
    USART_OverSampe_Enum OverSampling;
    USART_Parity_Enum    Parity;              /*!< Specifies the parity mode.   This parameter can be a value of @ref USART_Parity*/
    USART_MODE_Enum      Mode;
    USART_DataLen_Enum   WordLength;          /*!< Specifies the number of data bits transmitted or received in a frame.
                                               This parameter can be a value of @ref USART_Word_Length */
    USART_TransEncode_Enum TransEncode;           /*Manchest or Normal Encode*/
	  uint8_t TimeGuard;
} USART_Init_T;

/* Input parameters when initializing RS232 and similar modes. */
typedef struct
{
    /* Set baud rate of the USART (unused in slave modes). */
    uint32_t baudrate;
    uint32_t irda_filter;
    /*
     * Number of bits, which should be one of the following: USART_MR_CHRL_5_BIT,
     * USART_MR_CHRL_6_BIT, USART_MR_CHRL_7_BIT, USART_MR_CHRL_8_BIT or
     * USART_MR_MODE9.
     */
    USART_DataLen_Enum USART_WordLength;

    /*
     * Parity type, which should be one of the following: USART_MR_PAR_EVEN,
     * USART_MR_PAR_ODD, USART_MR_PAR_SPACE, USART_MR_PAR_MARK, USART_MR_PAR_NO
     * or USART_MR_PAR_MULTIDROP.
     */
    USART_Parity_Enum USART_Parity;

    /*
     * Number of stop bits between two characters: USART_MR_NBSTOP_1_BIT,
     * USART_MR_NBSTOP_1_5_BIT, USART_MR_NBSTOP_2_BIT.
     * \note USART_MR_NBSTOP_1_5_BIT is supported in asynchronous modes only.
     */
    USART_StopBIT_Enum USART_StopBits;

    /*
     * Run the channel in test mode, which should be one of following:
     * USART_MR_CHMODE_NORMAL, USART_MR_CHMODE_AUTOMATIC,
     * USART_MR_CHMODE_LOCAL_LOOPBACK, USART_MR_CHMODE_REMOTE_LOOPBACK.
     */
    USART_MODE_Enum   USART_Mode;

    /* Filter of IrDA mode, useless in other modes. */

}  USART_SimilarModeInit_T;

/* Input parameters when initializing ISO7816 mode. */
typedef struct
{
    /* Set the frequency of the ISO7816 clock. */
    uint32_t iso7816_hz;

    /*
     * The number of ISO7816 clock ticks in every bit period (1 to 2047,
     * 0 = disable clock). Baudrate rate = iso7816_hz / fidi_ratio.
     */
    uint32_t fidi_ratio;

    /*
     * How to calculate the parity bit: USART_MR_PAR_EVEN for normal mode or
     * USART_MR_PAR_ODD for inverse mode.
     */
    uint32_t parity_type;

    /*
     * Inhibit Non Acknowledge:
     *   - 0: the NACK is generated;
     *   - 1: the NACK is not generated.
     *
     * \note This bit will be used only in ISO7816 mode, protocol T = 0
     * receiver.
     */
    uint32_t inhibit_nack;

    /*
     * Disable successive NACKs.
     *  - 0: NACK is sent on the ISO line as soon as a parity error occurs
     * in the received character. Successive parity errors are counted up to
     * the value in the max_iterations field. These parity errors generate
     * a NACK on the ISO line. As soon as this value is reached, no additional
     * NACK is sent on the ISO line. The ITERATION flag is asserted.
     */
    uint32_t dis_suc_nack;

    /* Max number of repetitions (0 to 7). */
    uint32_t max_iterations;

    /*
     * Bit order in transmitted characters:
     *   - 0: LSB first;
     *   - 1: MSB first.
     */
    uint32_t bit_order;

    /*
     * Which protocol is used:
     *   - 0: T = 0;
     *   - 1: T = 1.
     */
    uint32_t protocol_type;
} USART_ISO7816_OPT_T;

/* Input parameters when initializing SPI mode. */
typedef struct
{
    /* Set the frequency of the SPI clock (unused in slave mode). */
    uint32_t baudrate;
    /*
    	 * Run the channel in test mode, which should be one of following:
    	 * USART_MR_CHMODE_NORMAL, USART_MR_CHMODE_AUTOMATIC,
    	 * USART_MR_CHMODE_LOCAL_LOOPBACK, USART_MR_CHMODE_REMOTE_LOOPBACK.
    	 */
    /*
    * Which SPI mode to use, which should be one of the following:
    * SPI_MODE_0, SPI_MODE_1, SPI_MODE_2, SPI_MODE_3.
    */
    USART_SPI_MODE_Enum spi_mode;
    USART_CHMODE_Enum channel_mode;
    USART_MODE_Enum mode;
    /*
     * Number of bits, which should be one of the following: USART_MR_CHRL_5_BIT,
     * USART_MR_CHRL_6_BIT, USART_MR_CHRL_7_BIT, USART_MR_CHRL_8_BIT or
     * USART_MR_MODE9.
     */
    USART_DataLen_Enum char_length;

} USART_SPI_T;

void USART_EnableTx(USART_Type *p_uart);
void USART_DisableTx(USART_Type *p_uart);
void USART_ResetTx(USART_Type *p_uart);
void USART_EnableRx(USART_Type *p_uart);
void USART_DisableRx(USART_Type *p_uart);
void USART_ResetRx(USART_Type *p_uart);
void USART_Enable(USART_Type *p_uart);
void USART_Disable(USART_Type *p_uart);
void USART_Reset(USART_Type *p_uart);
void USART_Enable_IT(USART_Type *p_uart, uint32_t ul_sources);
void USART_Disable_IT(USART_Type *p_uart, uint32_t ul_sources);
uint32_t USART_GetInterruptMask(USART_Type *p_uart);
uint32_t USART_GetStatus(USART_Type *p_uart);
void USART_ResetStatus(USART_Type *p_uart);
uint32_t USART_IsTxEmpty(USART_Type *p_uart);
ErrorStatus USART_Write(USART_Type *p_uart, const uint8_t uc_data);
ErrorStatus USART_Read(USART_Type *p_uart, uint8_t *p_uc_data);
void	USART_Send(USART_Type *p_uart, uint8_t data);
uint8_t USART_SPI_SendData(USART_Type *p_uart, uint8_t data);
void USART_SPI_SendBuffer(USART_Type *p_uart, uint8_t *data, uint8_t len);
void RS485_SendBuffer(USART_Type *p_uart, uint8_t *data, uint32_t len);
void RS485_SetTimeGuard(USART_Type *p_uart, uint8_t time_guard);
void USART_SetRxTimeout(USART_Type *p_usart, uint32_t timeout);
void USART_StartTxBreak(USART_Type *p_usart);
void USART_StopTxBreak(USART_Type *p_usart);
ErrorStatus USART_SendAddress(USART_Type *p_usart, uint32_t ul_addr);
ErrorStatus USART_SetAsyncBaudrate(USART_Type *p_usart, uint32_t baudrate, uint32_t ul_bus_clk);
static ErrorStatus USART_SetSyncMasterBaudrate(USART_Type *p_usart, uint32_t baudrate, uint32_t ul_bus_clk);
static void USART_SetSyncSlaveBaudrate(USART_Type *p_usart);
static ErrorStatus USART_Set_SPI_MasterBaudrate(USART_Type *p_usart, uint32_t baudrate, uint32_t ul_bus_clk);
void USART_DisableWriteProtect(USART_Type *p_usart);
void USART_RTS_High(USART_Type *p_usart);
ErrorStatus USART_InitRS232(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk);
ErrorStatus USART_InitHwHandshaking(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk);
ErrorStatus USART_InitSyncMaster(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk);
ErrorStatus USART_InitSyncSlave(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt);
ErrorStatus USART_InitIrda(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk);
static ErrorStatus USART_SetIso7816_Clock(USART_Type *p_usart, uint32_t clock, uint32_t ul_bus_clk);
ErrorStatus USART_InitIso7816(USART_Type *p_usart, USART_ISO7816_OPT_T *p_usart_opt, uint32_t ul_bus_clk);
void USART_ResetIterations(USART_Type *p_usart);
void USART_ResetNack(USART_Type *p_usart);
ErrorStatus USART_SPI_MasterInit(USART_Type *p_usart, USART_SPI_T *p_usart_spi, uint32_t ul_bus_clk);
ErrorStatus USART_SPI_SlaveInit(USART_Type *p_usart, USART_SPI_T *p_usart_spi);
ErrorStatus  USART_InitLinMaster(USART_Type *p_usart, uint32_t ul_baudrate, uint32_t ul_bus_clk);
ErrorStatus USART_InitLinSlave(USART_Type *p_usart, uint32_t ul_baudrate, uint32_t ul_bus_clk);
void USART_LinAbortTx(USART_Type *p_usart);
void USART_LinSendWakeupSignal(USART_Type *p_usart);
void USART_LinSetNodeAction(USART_Type *p_usart, uint8_t uc_action);
void USART_LinDisableParity(USART_Type *p_usart);
void USART_LinEnableParity(USART_Type *p_usart);
void USART_LinDisableChecksum(USART_Type *p_usart);
void USART_LinEnableChecksum(USART_Type *p_usart);
void USART_LinSetChecksumType(USART_Type *p_usart, uint8_t uc_type);
void USART_LinSetDataLenMode(USART_Type *p_usart, uint8_t uc_mode);
void USART_LinDisableFrameSlot(USART_Type *p_usart);
void USART_LinEnableFrameSlot(USART_Type *p_usart);
void USART_LinSetWakeupSignalType(USART_Type *p_usart, uint8_t uc_type);
void USART_LinSetResponseDataLen(USART_Type *p_usart, uint8_t uc_len);
void USART_LinSetTxIdentifier(USART_Type *p_usart, uint8_t uc_id);
uint8_t USART_LinReadIdentifier(USART_Type *p_usart);
uint8_t USART_LinGetDataLength(USART_Type *usart);
uint8_t USART_LinIdentifierSendComplete(USART_Type *usart);
uint8_t USART_LinIdentifierReceptionComplete(USART_Type *usart);
void USART_DMA_Receive(USART_Type *p_uart, uint8_t *data, uint32_t len);
void USART_DMA_Send(USART_Type *p_uart, uint8_t *data, uint32_t len);
void USART_Init(USART_Type* p_usart, USART_Init_T *p_usart_init);

#ifdef __cplusplus
}
#endif

#endif /* MS60F3XXX_USART_H */


