/**
  ******************************************************************************
  * @file           : ms60f3xxx_adc.c
  * @author         : sinomcu.com-SA
  * @brief          : ADC module driver.
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_adc.h"
#include "ms60f3xxx_scu.h"
/**
  * @brief  Deinitializes the ADCx peripheral registers to their default reset values.
  * @param  None
  * @retval None
  */
void ADC_DeInit(void)
{
    /* Enable ADC reset state */
    SCU_PeriphClockCmd(SCU_ADC_CLKEN, ENABLE);
    /* Release ADC from reset state */
    SCU_PeriphClockCmd(SCU_ADC_CLKEN, DISABLE);
}

/**
  * @brief  ADC configuration VREF output settings
  * @param  vref: Select VREF output type that contains the configuration information for VREF_OUT_ENUM
  * @param  ref_sel: Select ADC internal reference source VREF or AVDD
  * @retval None
  */
void	ADC_VREF_OUT(ADC_VREF_OUT_Enum vref, ADC_REF_SEL_Enum ref_sel)
{
    uint16_t temp;

    if(!ref_sel)
    {
        ADC->CR3 = 0;
        temp = ADC->TPS_CR;
        temp &= ~ADC_VREF_OUT_2v4;
        temp |= vref;
        ADC->TPS_CR = temp;
    }
    else
    {
        ADC->CR3 = ref_sel;
        ADC->TPS_CR |= vref;
    }
}

/**
  * @brief  Initializes the ADCx peripheral according to the specified parameters
  *         in the ADC_InitStruct.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_InitStruct: pointer to an ADC_InitTypeDef structure that contains
  *         the configuration information for the specified ADC peripheral.
  * @retval None
  */
void ADC_Init(ADC_Type* ADCx, ADC_Init_T* ADC_InitConfig)
{

    /*---------------------------- ADCx CR1 Configuration -----------------*/
    ADCx->CR1 = 0;
    ADCx->CR1 |= ADC_InitConfig->ADC_Resolution | ADC_InitConfig->ADC_Mode | ADC_InitConfig->ADC_ScanConvMode;

    /*---------------------------- ADCx CR2 Configuration -----------------*/
    ADCx->CR2 = 0;
    ADCx->CR2 |= ADC_InitConfig->ADC_ContinuousConvMode
                 | ADC_InitConfig->ADC_DMAAccessMode
                 | ADC_InitConfig->ADC_DataAlign;
    ADCx->CCR &= ~(0xFul << 16);
    ADCx->CCR |= (ADC_InitConfig->ADC_Prescaler << 16) | ADC_InitConfig->ADC_CLK_SEL;
    ADCx->CCR &= ~(0x10000Ful << 16);
    ADCx->CCR |= (ADC_InitConfig->ADC_Prescaler << 16) | ADC_InitConfig->ADC_CLK_SEL;
}

/**
  * @brief  Number of Regular Channel Selection
  * @param  ADC_NbrOfChannel:the ADC_NbrOfChannel that
						Number of Regular Channel Selection according to the specified parameters
  *         in the ADC_SQL_ENUM.
  * @retval None
  */
void ADC_RegularNbrOfChannel(ADC_SQL_Enum  ADC_NbrOfChannel)
{
    ADC->SQR1 &= ~(7ul << 21);
    ADC->SQR1 |= ADC_NbrOfChannel;
}

/**
  * @brief  Number of Injected Channel Selection
  * @param  ADC_NbrOfChannel:the ADC_NbrOfChannel that
						Number of Injected Channel Selection according to the specified parameters
  *         in the ADC_JSQL_ENUM.
  * @retval None
  */
void ADC_InjectedNbrOfChannel(ADC_JSQL_Enum  ADC_NbrOfChannel)
{
    ADC->JSQR &= ~(1 << 20);
    ADC->JSQR |= ADC_NbrOfChannel;
}

/**
  * @brief  Enables or disables the specified ADC peripheral.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  NewState: new state of the ADCx peripheral.
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_Cmd(ADC_Type* ADCx, FunctionalState NewState)
{
    uint16_t i;

    if (NewState != DISABLE)
    {
        /* Set the ADON bit to wake up the ADC from power down mode */
        ADCx->CR2 |= ADC_ADON;

        for(i = 0; i < 0xFFF0; i++)
            __NOP();

        ADCx->ENR = 1;
    }
    else
    {
        /* Disable the selected ADC peripheral */
        ADCx->CR2 &= ~ADC_ADON;
    }
}

/**
  * @brief  Enables or disables the specified ADC DMA request.
  * @param  ADCx: where x can be 1 or 3 to select the ADC peripheral.
  * @param  NewState: new state of the selected ADC DMA transfer.
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_DMACmd(ADC_Type* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    if (NewState != DISABLE)
    {
        /* Enable the selected ADC DMA request */
        ADCx->CR2 |= (1UL << 4);
    }
    else
    {
        /* Disable the selected ADC DMA request */
        ADCx->CR2 &= ~(1UL << 4);
    }
}

/**
  * @brief  Enables the specified ADC interrupts.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_IT: specifies the ADC interrupt sources to be enabled.
  *   This parameter can be any combination of the following values:
  *     @arg EOC: End of conversion interrupt mask
  *     @arg AWD: Analog watchdog interrupt mask
  *     @arg JEOC: End of injected conversion interrupt mask
  *     @arg OVR: Last conversion data was overwritten interrupt mask
  *     @arg ENDRX current transmission quantity end set interrupt mask
  * @retval None
  */
void ADC_Enable_IT(ADC_Type* ADCx, ADC_INTERRUPT_Enum ADC_IT)
{
    uint8_t itmask = 0;
    /* Get the ADC IT index */
    itmask = (uint8_t)ADC_IT;

   
        /* Enable the selected ADC interrupts */
        ADCx->IER = itmask;

}
/**
  * @brief   disables the specified ADC interrupts.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_IT: specifies the ADC interrupt sources to be disabled.
  *   This parameter can be any combination of the following values:
  *     @arg EOC: End of conversion interrupt mask
  *     @arg AWD: Analog watchdog interrupt mask
  *     @arg JEOC: End of injected conversion interrupt mask
  *     @arg OVR: Last conversion data was overwritten interrupt mask
  *     @arg ENDRX current transmission quantity end set interrupt mask
  * @retval None
  */
void ADC_Disable_IT(ADC_Type* ADCx, ADC_INTERRUPT_Enum ADC_IT)
{
    uint8_t itmask = 0;
    /* Get the ADC IT index */
    itmask = (uint8_t)ADC_IT;

        /* Disable the selected ADC interrupts */
        ADCx->IDR = itmask;
   
}
/**
  * @brief  Enables or disables the selected ADC software start conversion .
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  NewState: new state of the selected ADC software start conversion.
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_SoftwareStartConvCmd(ADC_Type* ADCx, FunctionalState NewState)
{
    if (NewState != DISABLE)
    {
        /* Enable the selected ADC conversion on external event and start the selected
           ADC conversion */
        ADCx->CR2 |= CR2_EXTTRIG_SWSTART_Set;
    }
    else
    {
        /* Disable the selected ADC conversion on external event and stop the selected
           ADC conversion */
        ADCx->CR2 &= ~CR2_EXTTRIG_SWSTART_Set;
    }
}

/**
  * @brief  Select ADC External Trigger Conversion Type .
  * @param  ADC_ExternalTrigConv:External Trigger Conversion Type Selection according to the specified parameters
  *         in the ADC TRIGER_MODE_ENUM.
  * @retval None
  */
void ADC_ExternalTrigConv(ADC_TRIGER_MODE_Enum  ADC_ExternalTrigConv)
{
    ADC->CR2 &= ~ADC_ExternalTrigConv;
    ADC->CR2 |= ADC_ExternalTrigConv;
}

/**
  * @brief  Gets the selected ADC Software start conversion Status.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @retval The new state of ADC software start conversion (SET or RESET).
  */
FlagStatus ADC_GetSoftwareStartConvStatus(ADC_Type* ADCx)
{
    FlagStatus bitstatus = RESET;
    /* Check the parameters */

    /* Check the status of SWSTART bit */
    if ((ADCx->CR2 & CR2_SWSTART_Set) != (uint32_t)RESET)
    {
        /* SWSTART bit is set */
        bitstatus = SET;
    }
    else
    {
        /* SWSTART bit is reset */
        bitstatus = RESET;
    }

    /* Return the SWSTART bit status */
    return  bitstatus;
}

/**
  * @brief  Configures the discontinuous mode for the selected ADC regular
  *         group channel.
  * @param  UM: Select Configures the discontinuous group channel according to the specified parameters
  *         in the ADC_DISNUMX_ENUM.
  * @param  Channel_Sum: Select  the discontinuous group channel number sum according to the specified parameters
  *         in the ADC_DISNUMX_ENUM.
  * @retval None
  */
void ADC_DISCNUMConfig(ADC_DISNUMX_Enum  UM, ADC_DISNUM_Enum Channel_Sum)
{
    switch(UM)
    {
        case ADC_DISCNUM1:
            ADC->CCR &= ~0x03ul;
            ADC->CCR |= Channel_Sum;
            break;

        case ADC_DISCNUM2:
            ADC->CCR &= ~(0x03ul << 2);
            ADC->CCR |= (Channel_Sum << 2);
            break;

        case ADC_DISCNUM3:
            ADC->CCR &= ~(0x03ul << 4);
            ADC->CCR |= (Channel_Sum << 4);
            break;

        case ADC_DISCNUM4:
            ADC->CCR &= ~(0x03ul << 6);
            ADC->CCR |= (Channel_Sum << 6);
            break;
    }
}

/**
  * @brief  Enables or disables the discontinuous mode on regular group
  *         channel for the specified ADC
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  NewState: new state of the selected ADC discontinuous mode
  *         on regular group channel.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_DiscModeCmd(ADC_Type* ADCx, FunctionalState NewState)
{
    /* Check the parameters */
    if (NewState != DISABLE)
    {
        ADCx->CR1 &= (~ADC_DISCON_ENALBE);
        ADCx->CR1 |= ADC_DISCON_ENALBE;
        /* Enable the selected ADC regular discontinuous mode */
    }
    else
    {
        /* Disable the selected ADC regular discontinuous mode */
        ADCx->CR1 &= ~ADC_DISCON_ENALBE;
    }
}

/**
  * @brief  Configures for the selected ADC regular Odd channel its corresponding
  *         rank in the sequencer and its sample time.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_Channel: the ADC channel to configure.
  *   This parameter can be one of the following values:
  *     @arg ADC_00: ADC Channel0 selected
  *     @arg ADC_01: ADC Channel1 selected
  *     @arg ADC_02: ADC Channel2 selected
  *     @arg ADC_03: ADC Channel3 selected
  *     @arg ADC_04: ADC Channel4 selected
  *     @arg ADC_05: ADC Channel5 selected
  *     @arg ADC_06: ADC Channel6 selected
  *     @arg ADC_07: ADC Channel7 selected
  *     @arg ADC_08: ADC Channel8 selected
  *     @arg ADC_09: ADC Channel9 selected
  *     @arg ADC_0A: ADC Channel10 selected
  *     @arg ADC_0B: ADC Channel11 selected
  *     @arg ADC_0C: ADC Channel12 selected
  *     @arg ADC_0D: ADC Channel13 selected
  *     @arg ADC_0E: ADC Channel14 selected
  *     @arg ADC_0F: ADC Channel15 selected
  * @param  Rank: The rank in the regular group sequencer. This parameter must be between 1 to 8.
  * @param  ADC_SampleTime: The sample time value to be set for the selected channel.
  *   This parameter can be one of the following values:
  *     @arg CLK_2: Sample time equal to 2 cycles
  *     @arg CLK_4: Sample time equal to 4 cycles
  *     @arg CLK_6: Sample time equal to 6 cycles
  *     @arg CLK_8: Sample time equal to 8 cycles
  *     @arg CLK_16: Sample time equal to 16 cycles
  *     @arg CLK_32: Sample time equal to 32 cycles
  *     @arg CLK_64: Sample time equal to 64 cycles
  *     @arg CLK_128: Sample time equal to 128 cycles
  * @retval None
  */
void ADC_RegularOddChannelConfig(ADC_Type* ADCx, ADC_ODD_CHANNEL_Enum channel, uint8_t rank, ADC_SMP_Enum ADC_SampleTime)
{
    uint32_t rsq, sampt;

    /* configure ADC regular sequence */
    if(rank < 3U)
    {
        rsq = ADCx->SQR3;
        rsq &=  ~((uint32_t)(0X1F << (10U * rank)));
        rsq |= ((uint32_t)channel << (10U * rank));
        ADCx->SQR3 = rsq;
    }
    else if(rank < 6U)
    {
        rsq =  ADCx->SQR2;
        rsq &= ~((uint32_t)(0X1F << (10U * (rank - 3U))));
        rsq |= ((uint32_t)channel << (10U * (rank - 3U)));
        ADCx->SQR2 = rsq;
    }
    else if(rank < 8U)
    {
        rsq =  ADCx->SQR1;
        rsq &= ~((uint32_t)(0X1F << (10U * (rank - 6U))));
        rsq |= ((uint32_t)channel << (10U * (rank - 6U)));
        ADCx->SQR1 = rsq;
    }
    else
    {
        /* illegal parameters */
    }

    /* configure ADC sampling time */
    sampt = ADCx->SMPR1;
    sampt &= ~((uint32_t)(0X7 << (3U * rank)));
    sampt |= (uint32_t)(ADC_SampleTime << (3U * rank));
    ADCx->SMPR1 = sampt;
}
/**
  * @brief  Configures for the selected ADC regular Even channel its corresponding
  *         rank in the sequencer and its sample time.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_Channel: the ADC channel to configure.
  *   This parameter can be one of the following values:
  *     @arg ADC_10: ADC Channel0 selected
  *     @arg ADC_11: ADC Channel1 selected
  *     @arg ADC_12: ADC Channel2 selected
  *     @arg ADC_13: ADC Channel3 selected
  *     @arg ADC_14: ADC Channel4 selected
  *     @arg ADC_15: ADC Channel5 selected
  *     @arg ADC_16: ADC Channel6 selected
  *     @arg ADC_17: ADC Channel7 selected
  *     @arg ADC_18: ADC Channel8 selected
  *     @arg ADC_19: ADC Channel9 selected
  *     @arg ADC_1A: ADC Channel10 selected
  *     @arg ADC_1B: ADC Channel11 selected
  *     @arg ADC_1C: ADC Channel12 selected
  *     @arg ADC_1D: ADC Channel13 selected
  *     @arg ADC_1E: ADC Channel14 selected
  *     @arg ADC_1F: ADC Channel15 selected
  * @param  Rank: The rank in the regular group sequencer. This parameter must be between 1 to 8.
  * @param  ADC_SampleTime: The sample time value to be set for the selected channel.
  *   This parameter can be one of the following values:
  *     @arg CLK_2: Sample time equal to 2 cycles
  *     @arg CLK_4: Sample time equal to 4 cycles
  *     @arg CLK_6: Sample time equal to 6 cycles
  *     @arg CLK_8: Sample time equal to 8 cycles
  *     @arg CLK_16: Sample time equal to 16 cycles
  *     @arg CLK_32: Sample time equal to 32 cycles
  *     @arg CLK_64: Sample time equal to 64 cycles
  *     @arg CLK_128: Sample time equal to 128 cycles
  * @retval None
  */
void ADC_RegularEvenChannelConfig(ADC_Type* ADCx,  ADC_EVEN_CHANNEL_Enum channel, uint8_t rank, ADC_SMP_Enum sample_time)
{
    uint32_t rsq, sampt;

    /* configure ADC regular sequence */
    if(rank < 3U)
    {
        rsq = ADCx->SQR3;
        rsq &=  ~((uint32_t)(0X1F << (5U * (rank*2+1))));
        rsq |= ((uint32_t)channel << (5U * (rank*2+1)));
        ADCx->SQR3 = rsq;
    }
    else if(rank < 6U)
    {
        rsq =  ADCx->SQR2;
        rsq &= ~((uint32_t)(0X1F << (5U * ((rank-3)*2+1))));
        rsq |= ((uint32_t)channel <<(5U * ((rank-3)*2+1)));
        ADCx->SQR2 = rsq;
    }
    else if(rank < 8U)
    {
        rsq =  ADCx->SQR1;
        rsq &= ~((uint32_t)(0X1F << (5U * ((rank-6)*2+1))));
        rsq |= ((uint32_t)channel <<(5U * ((rank-6)*2+1)));
        ADCx->SQR1 = rsq;
    }
    else
    {
        /* illegal parameters */
    }

    /* configure ADC sampling time */
    sampt = ADCx->SMPR1;
    sampt &= ~((uint32_t)(0X7 << (3U * ((rank )))));
    sampt |= (uint32_t)(sample_time << (3U * (rank )));
    ADCx->SMPR1 = sampt;
}

/**
  * @brief  Returns the last ADCx conversion result data for regular channel.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
	* @param  DRX: ADC Regular Group Data Register where DRX can select in the ADC_DR_ENUM.
  * @retval The Data conversion value.
  */
uint16_t ADC_GetConversionValue(ADC_Type* ADCx, ADC_DR_Enum DRX)
{
    /* Return the selected ADC conversion value */
    return  *(&ADCx->DR1 + DRX * 4);
}

/**
  * @brief  Enables or disables the selected ADC automatic injected group
  *         conversion after regular one.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  NewState: new state of the selected ADC auto injected conversion
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_AutoInjectedConvCmd(ADC_Type* ADCx, FunctionalState NewState)
{
    /* Check the parameters */

    if (NewState != DISABLE)
    {
        /* Enable the selected ADC automatic injected group conversion */
        ADCx->CR1 |= ADC_JAUTO_ENALBE;
    }
    else
    {
        /* Disable the selected ADC automatic injected group conversion */
        ADCx->CR1 &= ADC_JAUTO_DISABLE;
    }
}

/**
  * @brief  Gets the selected ADC Software start injected conversion Status.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @retval The new state of ADC software start injected conversion (SET or RESET).
  */
FlagStatus ADC_GetSoftwareStartInjectedConvCmdStatus(ADC_Type* ADCx)
{
    FlagStatus bitstatus = RESET;

    /* Check the status of JSWSTART bit */
    if ((ADCx->CR2 & (1 << 14)) != (uint32_t)RESET)
    {
        /* JSWSTART bit is set */
        bitstatus = SET;
    }
    else
    {
        /* JSWSTART bit is reset */
        bitstatus = RESET;
    }

    /* Return the JSWSTART bit status */
    return  bitstatus;
}

/**
  * @brief  Configures for the selected ADC injected Odd channel its corresponding
  *         rank in the sequencer and its sample time.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_Channel: the ADC channel to configure.
  *   This parameter can be one of the following values:
  *     @arg ADC_00: ADC Channel0 selected
  *     @arg ADC_01: ADC Channel1 selected
  *     @arg ADC_02: ADC Channel2 selected
  *     @arg ADC_03: ADC Channel3 selected
  * @param  Rank: The rank in the injected group sequencer. This parameter must be between 1 and 4.
  * @param  ADC_SampleTime: The sample time value to be set for the selected channel.
  *   This parameter can be one of the following values:
  *     @arg CLK_2: Sample time equal to 2 cycles
  *     @arg CLK_4: Sample time equal to 4 cycles
  *     @arg CLK_6: Sample time equal to 6 cycles
  *     @arg CLK_8: Sample time equal to 8 cycles
  *     @arg CLK_16: Sample time equal to 16 cycles
  *     @arg CLK_32: Sample time equal to 32 cycles
  *     @arg CLK_64: Sample time equal to 64 cycles
  *     @arg CLK_128: Sample time equal to 128 cycles
  * @retval None
  */
void ADC_InjectedOddChannelConfig(ADC_Type* ADCx,  ADC_ODD_CHANNEL_Enum channel, uint8_t rank, ADC_SMP_Enum sample_time)
{
    uint32_t rsq, sampt;
    rsq = ADCx->JSQR;
    rsq &=  ~((uint32_t)(0X1F << (5U * rank)));
    rsq |= ((uint32_t)channel << (5U * rank));
    ADCx->JSQR = rsq;

    /* configure ADC sampling time */
    if(rank < 2U)
    {
        sampt = ADCx->SMPR1;
        sampt &= ~((uint32_t)(0X7 << 24));
        sampt |= (uint32_t)(sample_time << 24);
        ADCx->SMPR1 = sampt;
    }
    else if (rank < 4U)
    {
        /* configure ADC sampling time */
        sampt = ADCx->SMPR1;
        sampt &= ~((uint32_t)(0X7 << 27));
        sampt |= (uint32_t)(sample_time << 27);
        ADCx->SMPR1 = sampt;
    }
    else
    {
        /* illegal parameters */
    }
}

/**
  * @brief  Configures for the selected ADC injected Even channel its corresponding
  *         rank in the sequencer and its sample time.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_Channel: the ADC channel to configure.
  *   This parameter can be one of the following values:
  *     @arg ADC_10: ADC Channel0 selected
  *     @arg ADC_11: ADC Channel1 selected
  *     @arg ADC_12: ADC Channel2 selected
  *     @arg ADC_13: ADC Channel3 selected
  * @param  Rank: The rank in the injected group sequencer. This parameter must be between 1 and 4.
  * @param  ADC_SampleTime: The sample time value to be set for the selected channel.
  *   This parameter can be one of the following values:
  *     @arg CLK_2: Sample time equal to 2 cycles
  *     @arg CLK_4: Sample time equal to 4 cycles
  *     @arg CLK_6: Sample time equal to 6 cycles
  *     @arg CLK_8: Sample time equal to 8 cycles
  *     @arg CLK_16: Sample time equal to 16 cycles
  *     @arg CLK_32: Sample time equal to 32 cycles
  *     @arg CLK_64: Sample time equal to 64 cycles
  *     @arg CLK_128: Sample time equal to 128 cycles
  * @retval None
  */
void ADC_InjectedEvenChannelConfig(ADC_Type* ADCx,  ADC_EVEN_CHANNEL_Enum channel, uint8_t rank, ADC_SMP_Enum sample_time)
{
    uint32_t rsq, sampt;
    rsq = ADCx->JSQR;
    rsq &=  ~((uint32_t)(0X1F << (5U * rank)));
    rsq |= ((uint32_t)channel << (5U * rank));
    ADCx->JSQR = rsq;

    if(rank < 2U)
    {
        sampt = ADCx->SMPR1;
        sampt &= ~((uint32_t)(0X7 << 24));
        sampt |= (uint32_t)(sample_time << 24);
        ADCx->SMPR1 = sampt;
    }
    else if (rank < 4U)
    {
        /* configure ADC sampling time */
        sampt = ADCx->SMPR1;
        sampt &= ~((uint32_t)(0X7 << 27));
        sampt |= (uint32_t)(sample_time << 27);
        ADCx->SMPR1 = sampt;
    }
    else
    {
        /* illegal parameters */
    }
}

/**
  * @brief  Configures the sequencer length for injected channels
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  Length: The sequencer length where Length can select in the ADC_JSQL_ENUM..
  *   This parameter must be a number between 2 or 4.
  * @retval None
  */
void ADC_InjectedSequencerLengthConfig(ADC_Type* ADCx, ADC_JSQL_Enum Length)
{
    uint32_t tmpreg1 = 0;
    /* Get the old register value */
    tmpreg1 = ADCx->JSQR;
    /* Clear the old injected sequnence lenght JL bits */
    tmpreg1 &= (1UL << 20);
    /* Set the injected sequnence lenght JL bits */
    tmpreg1 |= Length;
    /* Store the new register value */
    ADCx->JSQR = tmpreg1;
}

/**
  * @brief  Returns the ADC injected channel conversion result
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  JDRX: ADC injected Group Data Register where JDRX can select in the ADC_JDR_ENUM.
  *   This parameter can be one of the following values:
  *     @arg JDR1: Injected Channel1 Data Register
  *     @arg JDR2: Injected Channel2 Data Register
  *     @arg JDR3: Injected Channel3 Data Register
  *     @arg JDR4: Injected Channel4 Data Register
  * @retval The Data conversion value.
  */
uint16_t ADC_GetInjectedConversionValue(ADC_Type* ADCx,  ADC_JDR_Enum JDRX)
{
    /* Returns the selected injected channel conversion data value */
    return  *(&ADCx->JDR1 + JDRX * 4);
}

/**
  * @brief  Enables or disables the analog watchdog on single/all regular
  *         or injected channels
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_AnalogWatchdog: the ADC analog watchdog configuration.
  * @retval None
  */
void ADC_AnalogWatchdogCmd(ADC_function_Enum ADC_AnalogWatchdog, ADC_AWD_CONFIG_Enum AWD_CONFIG, ADC_AWD_CHANNEL_ENUM AWD_CHANNEL)
{
    uint32_t tmpreg = 0;
    /* Get the old register value */
    tmpreg = ADC->CR1;
    /* Set the analog watchdog enable mode */
    tmpreg |= ADC_AnalogWatchdog | AWD_CONFIG | AWD_CHANNEL;
    /* Store the new register value */
    ADC->CR1 = tmpreg;
}

/**
  * @brief  Configures the high and low thresholds of the analog watchdog.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  HighThreshold: the ADC analog watchdog High threshold value.
  *   This parameter must be a 12bit value.
  * @param  LowThreshold: the ADC analog watchdog Low threshold value.
  *   This parameter must be a 12bit value.
  * @retval None
  */
void ADC_AnalogWatchdogThresholdsConfig(ADC_Type* ADCx, uint16_t HighThreshold,uint16_t LowThreshold)
{
    /* Set the ADCx high threshold */
    ADCx->HTR = HighThreshold;
    /* Set the ADCx low threshold */
    ADCx->LTR = LowThreshold;
}

/**
  * @brief  Checks whether the specified ADC flag is set or not.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_FLAG: specifies the flag to check.
  *   This parameter can be one of the following values:
  *     @arg EOC: End of conversion flag
  *     @arg AWD: Analog watchdog flag
  *     @arg JEOC: End of injected conversion flag
  *     @arg OVR: Last conversion data was overwritten flag
  *     @arg ENDRX:current transmission quantity end set flag
  * @retval The new state of ADC_FLAG (SET or RESET).
  */
FlagStatus ADC_GetFlagStatus(ADC_Type* ADCx, ADC_SR_Enum ADC_FLAG)
{
    FlagStatus bitstatus = RESET;

    /* Check the status of the specified ADC flag */
    if ((ADCx->SR & ADC_FLAG) != (uint8_t)RESET)
    {
        /* ADC_FLAG is set */
        bitstatus = SET;
    }
    else
    {
        /* ADC_FLAG is reset */
        bitstatus = RESET;
    }

    /* Return the ADC_FLAG status */
    return  bitstatus;
}

/**
  * @brief  Clears the ADCx's pending flags.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_FLAG: specifies the flag to clear.
  *   This parameter can be any combination of the following values:
  *     @arg CLEAR_SR_AWD: Analog watchdog flag
  *     @arg CLEAR_SR_EOC: End of conversion flag
  *     @arg CLEAR_SR_JEOC: End of injected group conversion flag
  *     @arg CLEAR_SR_OVR: Last conversion data was overwritten flag
  *     @arg CLEAR_SR_ENDRX:current transmission quantity end set flag
  * @retval None
  */
void ADC_ClearFlag(ADC_Type* ADCx, ADC_CLEAR_SR_Enum ADC_FLAG)
{
    /* Clear the selected ADC flags */
    ADCx->SR &= ~(uint32_t)ADC_FLAG;
}

/**
  * @brief  Checks whether the specified ADC interrupt has occurred or not.
  * @param  ADCx: where x can be 1, 2 or 3 to select the ADC peripheral.
  * @param  ADC_IT: specifies the ADC interrupt source to check.
  *   This parameter can be one of the following values:
  *     @arg EOC: End of conversion flag
  *     @arg AWD: Analog watchdog flag
  *     @arg JEOC: End of injected conversion flag
  *     @arg OVR: Last conversion data was overwritten flag
  *     @arg ENDRX:current transmission quantity end set flag
  * @retval The new state of ADC_IT (SET or RESET).
  */
uint8_t ADC_GetITStatus(ADC_Type* ADCx, ADC_INTERRUPT_Enum ADC_IT)
{
    uint8_t bitstatus = 0;
    bitstatus = ADCx->IMR & ADC_IT;
    return  bitstatus;
}

/**
  * @brief  Configure DMA transmission length and address.
  * @param  len: DMA transmission length.
  * @param  buffer:DMA transmission address pointer.
  * @retval None
  */
void  ADC_DMA_Configure(uint8_t len, uint16_t *buffer)
{
    ADC->PTCR = ADC_PTCR_DIS;
    ADC->RPR = (u32)buffer;
    ADC->RCR = len;
    ADC->PTCR = ADC_PTCR_EN;
}

/**
  * @brief  ADC obtains internal temperature output
  * @param  result: ADC collects temperature channel conversion results
  * @param  vreftype:Configure ADC VREF reference source options
  * @retval None
  */
int16_t ADC_Get_Temperature(unsigned short result, ADC_VREF_Enum vreftype)
{
    int32_t temp = 0;

    switch((unsigned char)vreftype)
    {
        case ADC_VREF1V8:
            temp = (int32_t)(1537 - result);
            temp *= 1000;
            temp /= 4550;
            break;

        case ADC_VREF2V4:
            temp = (int32_t)(1153 - result);
            temp *= 1000;
            temp /= 3414;
            break;

        case ADC_VREFAVDD:
            temp = (int32_t)(839 - result);
            temp *= 1000;
            temp /= 2482;
            break;

        default:
            break;
    }

    return (int16_t)temp;
}


