/**
  ******************************************************************************
  * @file           : ms60f3xxx_COMP.c
  * @author         : sinomcu.com-SA
  * @brief          : COMP module driver.
  * @gcc            : toolchain version 5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with 
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
          
#include "ms60f3xxx_comp.h"
#pragma pack(1)  
/**
  * @brief  Initializes the COMP peripheral according to the specified parameters
  *         in the COMP_Init_T.
  * @param  COMP_InitStruct: pointer to an COMP_Init_T structure that contains
  *         the configuration information for the specified COMP peripheral.
  * @retval None
  */
void COMP_Init(COMP_Init_T* COMP_InitStruct)
{
    unsigned int value;
    value = COMP_InitStruct->NFISL +
            COMP_InitStruct->PFISL +
            COMP_InitStruct->HYST +
            COMP_InitStruct->POLAR +
            COMP_InitStruct->FILT_N +
            COMP_InitStruct->VEL_CTL;
    if (COMP_InitStruct->COMPSelect == COMP0)
    {
        if (COMP_InitStruct->COMP_en == COMP_ENABLE)
        {
            COMP->CR0 = 0;
            COMP->CR0 |= value +
                         COMP_InitStruct->COMP0_N +
                         COMP_InitStruct->COMP0_P +
                         COMP_InitStruct->COMP_en;
        }
    }
    else
    {
    }
    if(COMP_InitStruct->COMPSelect == COMP1)
    {
        if (COMP_InitStruct->COMP_en == COMP_ENABLE)
        {
            COMP->CR1 = 0;
            COMP->CR1 |= value +
                         COMP_InitStruct->COMP1_N +
                         COMP_InitStruct->COMP1_P +
                         COMP_InitStruct->COMP_en;
        }
    }
    else
    {
    }
}

/**
  * @brief  Set comp internal DAC output configuration.
  * @param  VALUE: Set the data to be converted for internal DAC.
  * @param  comp: Select the internal DAC to set according to the specified parameters
  *         in the COMP_DAC_Enum..
  * @param  VREF_output: VREF with DAC selected according to the specified parameters
  *         in the VREF_output.
  *
  * @retval None
  */
void COMP_DAC_INPUT(uint8_t  VALUE, COMP_DAC_Enum  comp, VREF_output VREF_OUT)
{
    if(comp == COMP_DAC0)
    {
        COMP->DAC_CR |= VREF_OUT + ((uint16_t)VALUE << 8);
    }
    else
    {
        COMP->DAC_CR |= ((uint32_t)VREF_OUT << 16) + ((uint32_t)VALUE << 24);
    }
}

/**
  * @brief  Enables the specified COMP interrupt request.
  * @param  p_comp: Set comp trigger interrupt source where p_comp can select in the COMPX_elem .
  * @retval None
  */
void COMP_Enable_IT(COMPx_elem  p_comp)
{
    
        COMP->IER &= ~p_comp;
        COMP->IER |= p_comp;

}

/**
  * @brief  Disables the specified COMP interrupt request.
  * @param  p_comp: Set comp trigger interrupt source where p_comp can select in the COMPX_elem .
  * @retval None
  */
void COMP_Disable_IT(COMPx_elem  p_comp)
{

        COMP->IDR &= ~p_comp;
        COMP->IDR |= p_comp;
    
}

/**
  * @brief  clear the specified COMP interrupt flag.
  * @param  p_comp: Select the interrupt source to clear where p_comp can select in the COMPX_elem .
  * @retval None
  */
void  COMP_IFR_Clear_flag(COMPx_elem  p_comp)
{
    COMP->IFR &= ~p_comp;
}

/**
  * @brief  COMP Output Data Selection Configuration.
  * @param  COMPSelect: Select COMPX output where COMPSelect can select in the COMPX_elem .
  * @retval None
  */
booltyped  COMP_DataOut(COMP_Enum COMPSelect)
{
    booltyped DataOut = falsefp;
    if (COMPSelect == COMP0)
    {
        DataOut = (booltyped)COMP->CR0_b.OUT;
    }
    else if(COMPSelect == COMP1)
    {
        DataOut = (booltyped)COMP->CR1_b.OUT;
    }
    return DataOut;
}

/**
  * @brief  Initializes the COMP_LOOPCR peripheral according to the specified parameters
  *         in the COMP_LOOPCR_T.
  * @param  COMP_InitStruct: pointer to an COMP_LOOPCR_T structure that contains
  *         the configuration information for the specified COMP_LOOPCR peripheral.
  * @retval None
  */
void  COMP_LOOPCR_Init(COMP_LOOPCR_T *COMP_InitStruct)
{
    COMP->LOOPCR0 |= COMP_InitStruct->COMP0_NSEL0 +
                     COMP_InitStruct->COMP0_PSEL0 +
                     (COMP_InitStruct->COMP0_NSEL1 << 8) +
                     (COMP_InitStruct->COMP0_PSEL1 << 8) +
                     (COMP_InitStruct->COMP0_NSEL2 << 16) +
                     (COMP_InitStruct->COMP0_PSEL2 << 16) +
                     COMP_InitStruct->GAP;


    COMP->LOOPCR1 |= COMP_InitStruct->COMP1_NSEL0 +
                     COMP_InitStruct->COMP1_PSEL0 +
                     (COMP_InitStruct->COMP1_NSEL1 << 8) +
                     (COMP_InitStruct->COMP1_PSEL1 << 8) +
                     (COMP_InitStruct->COMP1_NSEL2 << 16) +
                     (COMP_InitStruct->COMP1_PSEL2 << 16) +
                     COMP_InitStruct->GAP;

}

/**
  * @brief  Enables or disables the selected COMPx_LOOP.
  * @param  COMP_en: COMP_en can select in the COMP_LOOP_Enum .
  * @param  NewState: new state of the selected COMP_en
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void  COMP_LOOPCRCmd(COMP_LOOP_Enum  COMP_en, FunctionalState NewState)
{
    if (NewState != DISABLE)
    {
        if (COMP_en == COMP0_LOOP_EN)
        {
            COMP->LOOPCR0 |= COMP_en;
        }
        if (COMP_en == COMP1_LOOP_EN)
        {
            COMP->LOOPCR1 |= COMP_en;
        }
    }
    else
    {
        /* illegal parameters */
    }
}

/**
  * @brief  COMPLOOP Output Data Selection Configuration.
  * @param  COMPX:  Select COMP_LOOP output where COMPX can select in the COMP_Enum .
  * @param  output: Select COMP_LOOP output Data register .
  * @retval None
  */
booltyped  COMP_LOOPoutput(COMP_Enum COMPX, COMPx_output output)
{
    booltyped result = falsefp;
    if (COMPX == COMP0)
    {
        result = (booltyped)(COMP->LOOPDAT0 & output);
    }
    if (COMPX == COMP1)
    {
        result = (booltyped)(COMP->LOOPDAT1 & output);
    }
    return result ;
}

