/**
  ******************************************************************************
  * @file           : ms60f3xxx_cordic.c
  * @author         : sinomcu.com-SA
  * @brief          : CORDIC module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx.h"
#include "ms60f3xxx_cordic.h"
#include "ms60f3xxx_scu.h"

#define K1 0x26DD3B
#define K3  0x4D47A1

/** \brief  CORDIC moduel Init  include enable cordic clock.
 *  \param  void
 *  \return void
 */

unsigned char cordic_errorflag;
void CORDIC_Init(void)
{

    SCU_PeriphClockCmd(SCU_CORDIC_CLKEN, ENABLE);
    CORDIC->CR = ALL_UNIT_EN; /*enable all caculate unit*/
//    CORDIC_Enable_IT(CORDIC_ERR_IE);
}

/**  @brief  CORDIC enable interrupt
  *  @param  uint8_t cordic_int
  *  @typedef enum
  *{
  * CORDIC_DONE_IE =0x01,
  * CORDIC_ERR_IE  =0x02,
  * DIV_DONE_IE =0x04,
  * CORDIC1_ERR_IE  =0x08,
  *}CORDIC_INT_TYPE;
  *  @return void
  */
void CORDIC_Enable_IT(uint8_t cordic_int)
{
    CORDIC->IE = cordic_int;
    NVIC_EnableIRQ(CORDIC_IRQn);
}

/**  @brief  CORDIC disable interrupt
  *  @param  uint8_t cordic_int
  *  @typedef enum
  *{
  * CORDIC_DONE_IE =0x01,
  * CORDIC_ERR_IE  =0x02,
  * DIV_DONE_IE =0x04,
  * CORDIC1_ERR_IE  =0x08,
  *}CORDIC_INT_TYPE;
  *  @return None
  */
void CORDIC_Disable_IT(uint8_t cordic_int)
{

    CORDIC->ID = cordic_int;
    NVIC_EnableIRQ(CORDIC_IRQn);
}

/**  @brief  CORDIC get interrupt flag
  *  @param  void
  *  @return interrupt
  *typedef enum
  *{
  *CORDIC_DONE_IF =0x01,
  *CORDIC_ERR_IF  =0x02,
  *DIV_DONE_IF =0x04,
  *CORDIC1_ERR_IF  =0x08,
  *}CORDIC_INT_TYPE;
  *return value may be one or more combination
  */
uint16_t CORDIC_GetIf(void)
{
    return CORDIC->STA;
}

/** @brief CORDIC DIVIDED function.
  * @param int32_t dividend.
  * @param int32_t divisor. (divisor!=0)
  * @condition  divisor!=0, abs(dividend/divisor)<2
  * @return dividend/divisor.
  */
int32_t CORDIC_DivFun(int32_t *dividend, int32_t *divisor)
{
    CORDIC->DX1 = *dividend;
    CORDIC->DY1 = *divisor;
    CORDIC->MODE1 = CORDIC_START;

    while(!(CORDIC->STA & DIV_DONEIF))
    {
        if(cordic_errorflag == 1)
        {
            break;
        }
    }

    *dividend = CORDIC->DX1;
    *divisor = CORDIC->DY1;

    if(cordic_errorflag == 1)
    {
        cordic_errorflag = 0;
        return 0xFFFF;
    }

    return 0;

}

/**  @brief CORDIC MODUEL sin and cos function.
  *  @param LINE_UNIT_ENUM unit
  *  @typedef enum
  *  {
  *      UNIT3 =3,
  *     UNIT4 =4,
  *     UNIT5 =5
  *  }LINE_UNIT_ENUM;
  *  select one as unit parameter
  *  @param int24_t rad.   [-7f0000,7f0000]  is equal to [-pai,pai] // if(rad.signal) rad.value1=-(0xFFFFFF-rad.value1+1 )/0x400000  else  rad.value/0x400000    pi/2  0x400000
  *  @param int24_t *sint. 1/K 1.646760258121065  1/1.646760258121065 1/K 0x26dd3B
  *  @param int24_t *cost.   x26dd3B; 1/K 0x26dd3B
  *  @condition  divisor!=0, abs(dividend/divisor)<2
  *  @return 0.   *cost=cos(rad)/MPS,*sint=sin(rad)/MPS  k1=1.646760258121065 MPS=1
  */

int32_t CORDIC_SinCosFun(int24_t *rad, int24_t *sint, int24_t *cost)
{
    CORDIC->DX0 = sint->value*K1;
    CORDIC->DY0 = cost->value*K1;
    CORDIC->DZ0 = rad->value;
    CORDIC->MODE0 = SINCOS_MODE;


    while(!CORDIC->STA)
    {}
    CORDIC->STA = 0;
    cost->value = CORDIC->DX0;
    sint->value = CORDIC->DY0;

    if(cordic_errorflag == 1)
    {
        cordic_errorflag = 0;
        return 0xFFFF;
    }

    return 0;
}


/**  @brief CORDIC MODUEL SQRT(x2+y2) function.
  *  @param LINE_UNIT_ENUM unit
  *  @typedef enum
  *  {
  *     UNIT0 =0,
  *     UNIT1 =1,
  *     UNIT2 =2
  *  }NLINE_UNIT_ENUM;
  *	 select one as unit parameter
  *  @param int32_t *x.
  *  @param int32_t y.
  *  @param int32_t *rad.
  *  @condition
  *  @return 0.   *x=sqrt(x2+y2),*rad=*rad+atan(y/x)*(2 Power 23)
  */

uint32_t CORDIC_SqrtFun(uint24_t *x, uint24_t *y, uint24_t *rad)
{

    CORDIC->DZ0 = rad->value;
    CORDIC->DX0 = x->value;
    CORDIC->DY0 = y->value;
    CORDIC->MODE0 = SQRT_MODE;

    while(!(CORDIC->STA & CORDIC_DONE0IF))
    {
        if(cordic_errorflag == 1)
        {
            break;
        }
    }

    CORDIC->STA = 0;
    x->value = CORDIC->DX0;
    x->value=x->value*3;
		x->value=x->value/5;
		
    rad->value = CORDIC->DZ0;

    if(cordic_errorflag == 1)
    {
        cordic_errorflag = 0;
        return 0xFFFF;
    }

    return 0;
}


/** @brief CORDIC MODUEL HSIN HCOS function.
  *  @param NLINE_UNIT_ENUM unit
  *  @typedef enum
  *  {
  *     UNIT0 =0,
  *     UNIT1 =1,
  *     UNIT2 =2
  *  }NLINE_UNIT_ENUM;
  *	 select one as unit parameter
  *  @param int24_t *hcos.
  *  @param int24_t *hsin
  *  @param int24_t  rad.
  *  @condition  |*rad|<1.118
  *  @return 0.   *hcos=K3*cosh(rad)*(2 power 23),*hsin=K3*sinh(rad)*(2 power 23)  K3=0.828159360960215
  */
int32_t CODIC_HsincosFun(int24_t *hcos, int24_t *hsin, int24_t *rad)
{
    CORDIC->DZ0 = rad->value;
    CORDIC->DX0 = hcos->value*K3;
    CORDIC->DY0 = hsin->value*K3;
    CORDIC->MODE0 = HSINCOS_MODE;

    while(!(CORDIC->STA & CORDIC_DONE0IF))
    {
        if(cordic_errorflag == 1)
        {
            break;
        }
    }

    hcos->value = CORDIC->DX0;
    hsin->value = CORDIC->DY0;

    if(cordic_errorflag == 1)
    {
        cordic_errorflag = 0;
        return 0xFFFF;
    }

    return 0;
}


/**  @brief CODIC_HSQRT_atanh function.
  *  @param NLINE_UNIT_ENUM unit
  *  @typedef enum
  *  {
  *      UNIT0 =0,
  *      UNIT1 =1,
  *      UNIT2 =2
  *  }NLINE_UNIT_ENUM;
  *	 select one as unit parameter
  *  @param int32_t *x.
  *  @param int32_t *y
  *  @param int32_t *rad.
  *  @condition *x>0, |y|<=|x| |arctanh((*y)/(*x))|<1.118
  *  @return 0.   *hcos=K3*sqrt(x2-y2),*hsin=*rad+atanh(y/x)*(2 power 23)/3.1415926  K3=0.828159360960215
  */
int32_t CODIC_HsqrtFun(int24_t *x, int24_t *y, int24_t *rad)
{

    CORDIC->DZ0 = rad->value;
    CORDIC->DX0 = x->value;
    CORDIC->DY0 = y->value;
    CORDIC->MODE0 = HSQRT_MODE;

    while(!(CORDIC->STA & CORDIC_DONE0IF))
    {
        if(cordic_errorflag == 1)
        {
            break;
        }
    }

    x->value = CORDIC->DX0;
    y->value = CORDIC->DZ0;

    if(cordic_errorflag == 1)
    {
        cordic_errorflag = 0;
        return 0xFFFF;
    }

    return 0;

}
