/**
  ******************************************************************************
  * @file           : ms60f3xxx_crc.c
  * @author         : sinomcu.com-SA
  * @brief          : CRC module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_crc.h"
#include "ms60f3xxx.h"
#include "ms60f3xxx_scu.h"

/**
  * @brief  Initialize the CRC according to the specified
  *         parameters in the CRC_Init_T
  * @param  CrcInitConfig  pointer to a CRC_Init_T structure that contains
  *                the configuration information for the specified CRC module.
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_Init(CRC_Init_T *CrcInitConfig)
{
    /* Enable clock */
    SCU_PeriphClockCmd(SCU_CRC_CLKEN, ENABLE);

    /* Set CRC initial value has been picked up by user */

    CRC->CR = CrcInitConfig->enable_type;
    CRC->CR += CrcInitConfig->inout_type + CrcInitConfig->width_type;

    CRC->CRC_POLY = CrcInitConfig->POLY;
    CRC->INIT = CrcInitConfig->InitValue;
    CRC->XOR = CrcInitConfig->CRC_XOR;
    return SUCCESS;
}
/**
  * @brief  setting the CRC according to the specified
  *         parameters init_value
  * @param  init_value  CRC caculate init value
  *
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_SetInitValue(uint32_t init_value)
{
    CRC->INIT = init_value;

    if(CRC->DOUT == init_value)
        return SUCCESS;
    else
        return ERROR;
}

/**
  * @brief  setting the CRC according to the specified
  *         parameters poly_value
  * @param  poly_value  CRC caculate init value
  *
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */

ErrorStatus CRC_SetPolyValue(uint32_t poly_value)
{
    CRC->CRC_POLY = poly_value;
    return SUCCESS;
}

/**
  * @brief  setting the CRC according to the specified
  *         parameters XOR_value
  * @param  xor_value  CRC caculate XOR value
  *
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_SetXorValue(uint32_t xor_value)
{
    CRC->CRC_POLY = xor_value;
    return SUCCESS;
}

/**
  * @brief  setting the CRC according to the specified
  *         parameters in and out format
  * @param  inout_format  CRC DATIN format and CRC DOUT formate
  *             IN_AND_OUT_NORMAL =0,
  *             IN_NORMAL_OUT_REVERSE=(1<<3),
  *             IN_REVERSE_OUT_NORMAL=(1<<4),
  *             IN_REVERSE_OUT_REVERSE=(3<<3),
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_SetInOutFormat(CRC_INOUT_Enum inout_format)
{

    CRC->CR &= (~IN_OUT_FORMAT_MASK);
    CRC->CR |= inout_format;
    return SUCCESS;
}


/**
  * @brief  setting the CRC according to the specified
  *         parameters WIDTH_value
  * @param  width  CRC caculate width value
  *          CRC8
  *          CRC16
  *          CRC32
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_SetWidthValue(CRC_WIDTH_Enum width)
{

    CRC->CR &= (~WIDTH_MASK);
    CRC->CR |= width;
    return SUCCESS;
}

/**
  * @brief  setting the CRC enable or disable
  *
  * @param  enable   CRC caculate width value
             CRC_DISABLE=0,
             CRC_ENABLE	=1
  *
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC registers are initialized
  *          - ERROR: not applicable
  */
ErrorStatus CRC_Enable(CRC_ENABLE_Enum enable)
{

    CRC->CR &= (~CRC_ENABLE_MASK);
    CRC->CR |= enable;
    return SUCCESS;
}

/**
  * @brief  CRC_Reset
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: CRC RESET
  *          - ERROR: CRC not RESET
  */
ErrorStatus CRC_Reset(void)
{

    CRC->CR |= CRC_RESET_MASK;
    return SUCCESS;
}

/**
  *  @brief  Caculate CRC result
  *  @param parameters  WIDTH_ENUM_TYPE type     CRC width
  *                      CRC8
  *                      CRC16
  *                      CRC32
  *  @param parameters   uint8_t *pBuffer        CRC input data array
  *  @param parameters   uint32_t BufferLength    input data array len
  *  @retval  CRC result value 
  */
uint32_t CRC_Calculate(CRC_WIDTH_Enum type, uint8_t *pBuffer, uint32_t BufferLength)
{
    uint32_t index;          /* CRC input data buffer index */
    uint32_t crc_value;
    uint8_t i = 0;
    CRC_Reset();

    switch(type)
    {
        case CRC_8:
            for(index = 0; index < BufferLength; index++)
            {

                CRC->DIN = pBuffer[index];
            }

            crc_value = CRC->DOUT & 0xFF;
            break;

        case CRC_16:
            for(index = 0; index < BufferLength / 2; index++)
            {

                i = 2;

                CRC->DIN = pBuffer[i * index + 1] + ((uint16_t)pBuffer[i * index] << 8);
            }

            crc_value = CRC->DOUT & 0xFFFF;
            break;

        case CRC_32:
        {

            i = 4;

            for(index = 0; index < BufferLength / 4; index++)
            {

                CRC->DIN = pBuffer[i * index + 3] + (uint32_t)(pBuffer[(i * index + 2)] << 8) + (uint32_t)(pBuffer[(i * index + 1)] << 16) + ((uint32_t)pBuffer[(i * index)] << 24);

            }

            crc_value = CRC->DOUT;
        }

        default:
            break;

    }

    return crc_value;
}

/******************************** END OF FILE *********************************/
