/**
  ******************************************************************************
  * @file           : ms60f3xxx_flash.c
  * @author         : sinomcu.com-SA
  * @brief          : FLASH module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */

#include "ms60f3xxx_flash.h"
#include "ms60f3xxx.h"
#include "ms60f3xxx_wwdg.h"
#include "ms60f3xxx_crc.h"

/**
  * @brief  FLASH_Remove_Read_Protect,flash removes read protection.
  * @param  None
  * @retval None
  */
void FLASH_Remove_Read_Protect(void)
{
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(PRVT_NVR_EN) = FLASH_PRVT_WRITE_EN_KEY;
    MEM_MAP(FLS_MANU_NVR_EN) = FLASH_MANU_NVR_EN_KEY;
    FLASH->IFR = 0;
    FLASH->MR = FLASH_SECTOR_ERASE_MODE;
    *(unsigned char *)NVR12_ADDRESS = 0;

    while(!(FLASH->IFR & 0x02));

    FLASH->IFR = 0;
}

/**
  * @brief  FLASH_Read_Protect,flash read protection.
  * @param  None
  * @retval None
  */
void FLASH_Read_Protect(void)
{

    unsigned char temp;
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(PRVT_NVR_EN) = FLASH_PRVT_WRITE_EN_KEY;
    FLASH->MR = FLASH_PROGRAM_MODE;
    temp = READ_PROTECT_ADDRESS_CONTENT();

    if(temp & 0x01)
    {
        temp &= 0xfe;

        if(FLASH_Program(READ_PROTECT_ADDRESS, &temp, 1) != FLASH_COMPLETE)
        {
            return;
        }

    }
}

/**
  * @brief  Flash_CrcCalculate,flash crc calculation.
  * @param  uint32_t start,start address.
  * @param  uint32_t end,end address.
  * @retval CRC Calculate result
  */
uint32_t Flash_CrcCalculate(uint32_t start, uint32_t end)
{
#define CRC_START_CAL 1
    uint32_t crc;
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    FLASH->ICRCAR = start;
    FLASH->ECRCAR = end;
    FLASH->CRCCR_b.START = CRC_START_CAL;

    while(!(FLASH->IFR & FLASH_IR_CRCEND));

    crc = FLASH->CRCVR;
    return crc;
}

/**
  * @brief  FLASH_PROGRAM_Unlock,Unlock flash programming restrictions.
  * @param  None
  * @retval None
  */
void FLASH_PROGRAM_Unlock(void)
{
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(MAIN_CODE_EN) =  FLASH_CODE_WRITE_EN_KEY;

    FLASH->MR = FLASH_PROGRAM_MODE;
}

/**
  * @brief  FLASH_PROGRAM_Lock,Lock flash programming restrictions.
  * @param  None
  * @retval None
  */
void FLASH_PROGRAM_Lock(void)
{
    FLASH->MR = FLASH_LOCK_KEY;
    MEM_MAP(CODE_CFG_EN) =  FLASH_UNLOCK_KEY;
}


/**
  * @brief  FLASH_NVR_PROGRAM_Unlock,Unlock FLASH NVR PROGRAM restrictions.
  * @param  None
  * @retval None
  */
void FLASH_NVR_PROGRAM_Unlock(void)
{
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(PRVT_NVR_EN) =  FLASH_PRVT_WRITE_EN_KEY;
    FLASH->MR = FLASH_PROGRAM_MODE;
}
/**
  * @brief  FLASH_NVR_PROGRAM_Lock,Lock FLASH NVR PROGRAM restrictions.
  * @param  None
  * @retval None
  */
void FLASH_NVR_PROGRAM_Lock(void)
{
    MEM_MAP(PRVT_NVR_EN) =  FLASH_LOCK_KEY;
    FLASH->MR = FLASH_LOCK_KEY;
    MEM_MAP(FLS_CFG_EN) = FLASH_LOCK_KEY;
}

/**
  * @brief  FLASH_NVR_ERASE_Unlock,Unlock FLASH NVR ERASE restrictions.
  * @param  None
  * @retval None
  */
void FLASH_NVR_ERASE_Unlock(void)
{
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(CODE_CFG_EN) =  FLASH_CODE_WRITE_EN_KEY;
    MEM_MAP(PRVT_NVR_EN) =  FLASH_PRVT_WRITE_EN_KEY;
    FLASH->MR = FLASH_SECTOR_ERASE_MODE;
}

/**
  * @brief  FLASH_ERASE_SECTOR_Unlock,Unlock FLASH ERASE SECTOR restrictions.
  * @param  None
  * @retval None
  */
void FLASH_ERASE_SECTOR_Unlock(void)
{
    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(CODE_CFG_EN) =  FLASH_CODE_WRITE_EN_KEY;
    FLASH->MR = FLASH_SECTOR_ERASE_MODE;
}

/**
  * @brief  FLASH_ERASE_SECTOR_Lock,Lock FLASH ERASE SECTOR restrictions.
  * @param  None
  * @retval None
  */
void FLASH_ERASE_SECTOR_Lock(void)
{
    FLASH->MR = FLASH_LOCK_KEY;
    MEM_MAP(CODE_CFG_EN) =  FLASH_LOCK_KEY;
    MEM_MAP(FLS_CFG_EN) = FLASH_LOCK_KEY;
}

/**
  * @brief  NVR_EraseSector,NVR Erase Sector.
  * @param  uint32_t Sector_Address:The address of sector
  * @retval FLASH_Status FLASH_OPERRATION_MODE
  */
FLASH_Status NVR_EraseSector(uint32_t Sector_Address)
{

    FLASH_Status status = FLASH_COMPLETE;
    unsigned char *paddr;
    paddr = (unsigned char *)Sector_Address;
    FLASH_NVR_ERASE_Unlock();
    *paddr = 0;
    status = FLASH_WaitForBitStatusDone(FLASH_FLAG_SECTOR_ERASE_END, 0xFFFF);
    FLASH_ClearFlag(FLASH_FLAG_SECTOR_ERASE_END);

    return status;
}

/**
  * @brief  FLASH_EraseChip,FLASH Erase Chip.
  * @param  None
  * @retval FLASH_Status FLASH_OPERRATION_MODE
  */
FLASH_Status FLASH_EraseChip(void)
{
    FLASH_Status status = FLASH_COMPLETE;

    MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;
    MEM_MAP(CODE_CFG_EN) =  FLASH_CODE_WRITE_EN_KEY;
    FLASH->MR = FLASH_CHIP_ERASE_MODE;
    FLASH->IFR = 0;
    MEM_MAP(0x01000000) = 0;

    /* Wait for the operation to be completed */
    status = FLASH_WaitForBitStatusDone(FLASH_FLAG_CHIP_ERASE_END, 0xFFFF);
    FLASH_ClearFlag(FLASH_FLAG_CHIP_ERASE_END);
    return status;
}

/**
  * @brief  FLASH_EraseSector,FLASH Erase Sector.
  * @param  uint32_t Sector_Address: the address of sector
  * @retval FLASH_Status FLASH_OPERRATION_MODE
  */
FLASH_Status FLASH_EraseSector(uint32_t Sector_Address)
{
    FLASH_Status status = FLASH_COMPLETE;

    if(Sector_Address < 0x20000)
        FLASH_ERASE_SECTOR_Unlock();
    else if((Sector_Address >= NVR0_ADDRESS) && (Sector_Address < NVR10_ADDRESS + 0x200))
    {
    }
    else if((Sector_Address >= NVR13_ADDRESS) && (Sector_Address < NVR13_ADDRESS + 0x200))
    {
        FLASH_NVR_ERASE_Unlock();
    }
    else if((Sector_Address >= NVR15_ADDRESS) && (Sector_Address < NVR15_ADDRESS + 0x200))
    {
    }
    else if(Sector_Address < 0x01020000 && Sector_Address >= 0x01000000)
    {
        FLASH_ERASE_SECTOR_Unlock();
    }
    else
        return FLSH_ADDR_ERRPR;

    *(__IO uint32_t *)(Sector_Address) = 0;

    /* Wait for the operation to be completed */
    status = FLASH_WaitForBitStatusDone(FLASH_FLAG_SECTOR_ERASE_END, 0xFFFF);
    FLASH_ClearFlag(FLASH_FLAG_SECTOR_ERASE_END);


    return status;
}

/**
  * @brief  FLASH_GetFlagStatus,FLASH Get Flag Status.
  * @param  uint32_t FLASH_FLAG:clear flag can be one of the FLASH_Status(Enum)
  typedef enum
  {
    FLASH_COMPLETE = 1,
    FLASH_BUSY,
    FLASH_ADDR_OVERFLOW,
    FLASH_ERROR_RD_ADDR,
    FLASH_ERROR_PG_ADDR,
    FLASH_ERROR_ERASE_ADDR,
    FLASH_PROGRAM_END,
    FLASH_CHIP_ERASE_END,
    FLASH_SECTOR_ERASE_END,
    FLASH_TIMEOUT,
    FLASH_ERASE_CHECK_ERROR,
    FLASH_PRG_CHECK_ERROR,
    FLSH_ADDR_ERRPR
  } FLASH_Status;
  * @retval FlagStatus:SET or RESET
  */
FlagStatus FLASH_GetFlagStatus(uint32_t FLASH_FLAG)
{
    FlagStatus bitstatus = RESET;

    /* Check the parameters FLASH_FLAG*/

    if(IS_FLASH_GET_FLAG(FLASH_FLAG) == falsefp)
    {
        while(!IS_FLASH_GET_FLAG(FLASH_FLAG))
        {

            __nop();/*Pendding Here.*/
        }
    }

    if( (FLASH->IFR & FLASH_FLAG) != (uint32_t)RESET )
    {
        bitstatus = SET;     /*Bit done occurs.*/
    }
    else
    {
        bitstatus = RESET;
    }

    /* Return the new state of FLASH_FLAG (SET or RESET) */
    return bitstatus;
}



/**
  * @brief  Clears the FLASH's pending flags.
  * @param  uint32_t FLASH_FLAG:clear flag can be one of the FLASH_Status(Enum)
  typedef enum
  {
    FLASH_COMPLETE = 1,
    FLASH_BUSY,
    FLASH_ADDR_OVERFLOW,
    FLASH_ERROR_RD_ADDR,
    FLASH_ERROR_PG_ADDR,
    FLASH_ERROR_ERASE_ADDR,
    FLASH_PROGRAM_END,
    FLASH_CHIP_ERASE_END,
    FLASH_SECTOR_ERASE_END,
    FLASH_TIMEOUT,
    FLASH_ERASE_CHECK_ERROR,
    FLASH_PRG_CHECK_ERROR,
    FLSH_ADDR_ERRPR
  } FLASH_Status;
  * @retval None
  */
void FLASH_ClearFlag(uint32_t FLASH_FLAG)
{
    /* Check the parameters */
    if(IS_FLASH_CLEAR_FLAG(FLASH_FLAG) == falsefp)
    {
        while(truefp)
        {
            __nop();
        }
    }

    FLASH->IFR &= (~((uint32_t)FLASH_FLAG));
}

/**
  * @brief  Waits for a FLASH operation status or a TIMEOUT to occur.
  * @param  uint32_t FLASH_FLAG:clear flag can be one of the FLASH_Status(Enum)
  typedef enum
  {
    FLASH_COMPLETE = 1,
    FLASH_BUSY,
    FLASH_ADDR_OVERFLOW,
    FLASH_ERROR_RD_ADDR,
    FLASH_ERROR_PG_ADDR,
    FLASH_ERROR_ERASE_ADDR,
    FLASH_PROGRAM_END,
    FLASH_CHIP_ERASE_END,
    FLASH_SECTOR_ERASE_END,
    FLASH_TIMEOUT,
    FLASH_ERASE_CHECK_ERROR,
    FLASH_PRG_CHECK_ERROR,
    FLSH_ADDR_ERRPR
  } FLASH_Status;
  * @param  Timeout: FLASH programming Timeout
  * @retval FLASH Status: The returned value can be:
  *         FLASH_BUSY = 1,
  *         FLASH_COMPLETE,
  *         FLASH_ADDR_OVERFLOW,
  *         FLASH_ERROR_RD_ADDR,
  *         FLASH_ERROR_PG_ADDR,
  *         FLASH_ERROR_ERASE_ADDR,
  *         FLASH_PROGRAM_END,
  *         FLASH_CHIP_ERASE_END,
  *         FLASH_SECTOR_ERASE_END,
  *         FLASH_TIMEOUT.
  */
FLASH_Status FLASH_WaitForBitStatusDone(uint32_t FLASH_FLAG, uint32_t Timeout)
{
    FLASH_Status status = FLASH_COMPLETE;

    /* Wait for a FLASH operation to complete or a TIMEOUT to occur */


    while((FLASH_GetFlagStatus(FLASH_FLAG) == RESET) && (Timeout != 0x00))
    {
        Timeout--;
    }

    if(Timeout == 0x00 )
    {
        status = FLASH_TIMEOUT;
    }

    /* Return the operation status */
    return status;
}

/**
  * @brief  FLASH_Program,FLASH Program.
  * @param  uint32_t Address:The address of the Flash
  * @param  uint8_t *buffData pointer
  * @param  uint32_t len:the len of data
  * @retval FlagStatus:
  typedef enum
  {
    FLASH_COMPLETE = 1,
    FLASH_BUSY,
    FLASH_ADDR_OVERFLOW,
    FLASH_ERROR_RD_ADDR,
    FLASH_ERROR_PG_ADDR,
    FLASH_ERROR_ERASE_ADDR,
    FLASH_PROGRAM_END,
    FLASH_CHIP_ERASE_END,
    FLASH_SECTOR_ERASE_END,
    FLASH_TIMEOUT,
    FLASH_ERASE_CHECK_ERROR,
    FLASH_PRG_CHECK_ERROR,
    FLSH_ADDR_ERRPR
  } FLASH_Status;
	return result : one or more of thre enums.
  */
FLASH_Status FLASH_Program(uint32_t Address, uint8_t *buff, uint32_t len)
{
    uint8_t *paddr;

    uint32_t i = 0;
    FLASH_Status status = FLASH_COMPLETE;

    if(Address < 0x20000)
        FLASH_PROGRAM_Unlock();
    else if(Address >= 0x01000000 && Address < 0x01020000)
    {
        FLASH_PROGRAM_Unlock();
    }
    else
    {
        FLASH_PROGRAM_Unlock();
    }

    FLASH->IFR = 0;
    paddr = (unsigned char *)Address;

    for(i = 0; i < len; i++)
    {

        paddr[i] = buff[i];

        while(!(FLASH->IFR & FLASH_FLAG_PROGRAM_END))
            ;

        FLASH->IFR = 0;
    }

    return status;
}




