/**
  ******************************************************************************
  * @file           : ms60f3xxx_gpio.c
  * @author         : sinomcu.com-SA
  * @brief          : GPIO module driver.
  * @gcc            : toolchain version   5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with 
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_gpio.h"
#include "ms60f3xxx.h"

/**
  *@brief: GPIO_Init
  *@note: A single GPIO can be configured as float input, pull up input, pull down input, push pull output, open drain output, multiplexing function, analog function, You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx  GPIO selection, can be configured to GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@param: mode GPIO mode selection, can be configured as GPIO_MODE_INPUT_FLOAT, GPIO_MODE_INPUT_PU, etc.
  *@retval: None
  */
void GPIO_Init(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin, GPIO_Mode_Enum mode)
{
    GPIOx->MODER &= ~((uint32_t)(0x3) << (GPIO_Pin * 2));
    GPIOx->PU &= ~((uint32_t)(0x1) << GPIO_Pin);
    GPIOx->PD &= ~((uint32_t)(0x1) << GPIO_Pin);
    GPIOx->OD &= ~((uint32_t)(0x1) << GPIO_Pin);
    GPIOx->IE &= ~((uint32_t)(0x1) << GPIO_Pin);
    GPIOx->SR &= ~((uint32_t)(0x1) << GPIO_Pin);

    switch(mode)
    {
        case GPIO_MODE_INPUT_FLOAT:
            GPIOx->IE |= (uint32_t)0x1 << GPIO_Pin;
            GPIOx->PU &= ~((uint32_t)0x1 << GPIO_Pin);
            GPIOx->PD &= ~((uint32_t)0x1 << GPIO_Pin);
            break;

        case GPIO_MODE_INPUT_PU:
            GPIOx->PU |= (uint32_t)0x1 << GPIO_Pin;
            GPIOx->IE |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_INPUT_PD:
            GPIOx->PD |= (uint32_t)0x1 << GPIO_Pin;
            GPIOx->IE |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_OUTPUT_PP:
            GPIOx->MODER |= (uint32_t)0x1 << (GPIO_Pin * 2);
            break;

        case GPIO_MODE_OUTPUT_OD:
            GPIOx->MODER |= (uint32_t)0x1 << (GPIO_Pin * 2);
            GPIOx->OD |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_OUTPUT_OD_PU:
            GPIOx->PU |= (uint32_t)0x1 << GPIO_Pin;
            GPIOx->MODER |= (uint32_t)0x1 << (GPIO_Pin * 2);
            GPIOx->OD |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_OUTPUT_OD_PD:
            GPIOx->PD |= (uint32_t)0x1 << GPIO_Pin;
            GPIOx->MODER |= (uint32_t)0x1 << (GPIO_Pin * 2);
            GPIOx->OD |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_AF:
            GPIOx->MODER |= (uint32_t)0x2 << (GPIO_Pin * 2);
            GPIOx->IE |= (uint32_t)0x1 << GPIO_Pin;
            break;

        case GPIO_MODE_ANALOG:
            GPIOx->MODER |= (uint32_t)0x3 << (GPIO_Pin * 2);
            GPIOx->PU &= ~((uint32_t)0x1 << GPIO_Pin);
            GPIOx->PD &= ~((uint32_t)0x1 << GPIO_Pin);
            break;

        default :
            break;
    }
}

/**
  *@brief: GPIO_PinRemapConfig
  *@note: GPIO Multiplexing Function Selection,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@param: Reuse selection, can be configured as GPIO_ALT_SYSTEM, GPIO_ALT_SPI0, etc.
  *@retval: None
  */
void GPIO_PinRemapConfig(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin, GPIO_ALT_Enum GPIO_AFReg)
{
    GPIOx->MODER &= ~(((uint32_t)0x3) << (GPIO_Pin * 2));
    GPIOx->MODER |= (uint32_t)0x2 << (GPIO_Pin * 2);

    if(GPIO_Pin > GPIO_PIN11)
    {
        GPIOx->AFR3 &= ~((uint32_t)(0xff) << ((GPIO_Pin - 12) * 8));
        GPIOx->AFR3 |= (uint32_t)GPIO_AFReg << ((GPIO_Pin - 12) * 8);
    }
    else if(GPIO_Pin > GPIO_PIN7)
    {
        GPIOx->AFR2 &= ~((uint32_t)(0xff) << ((GPIO_Pin - 8) * 8));
        GPIOx->AFR2 |= (uint32_t)GPIO_AFReg << ((GPIO_Pin - 8) * 8);
    }
    else if(GPIO_Pin > GPIO_PIN3)
    {
        GPIOx->AFR1 &= ~((uint32_t)(0xff) << ((GPIO_Pin - 4) * 8));
        GPIOx->AFR1 |= (uint32_t)GPIO_AFReg << ((GPIO_Pin - 4) * 8);
    }
    else
    {
        GPIOx->AFR0 &= ~((uint32_t)(0xff) << (GPIO_Pin * 8));
        GPIOx->AFR0 |= (uint32_t)GPIO_AFReg << (GPIO_Pin * 8);
    }
}

/**
  *@brief: GPIO_PullUpConfig
  *@note: GPIO Configuration Pullup,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: None
  */
void GPIO_PullUpConfig(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->PD &= ~((uint32_t)(0x1) << GPIO_Pin);
    GPIOx->PU |= (uint32_t)0x1 << GPIO_Pin;
}

/**
  *@brief: GPIO_PullDownConfig
  *@note: GPIO Configuration PullDown,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: None
  */
void GPIO_PullDownConfig(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->PD |= (uint32_t)0x1 << GPIO_Pin;
    GPIOx->PU &= ~((uint32_t)(0x1) << GPIO_Pin);
}

/**
  *@brief: GPIO_SetDriverPower
  *@note: GPIO Configuration drive strength,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@param: Speed selection, can be configured for Strong_Drive or  Weak_Drive
  *@retval: None
  */
void GPIO_SetDriverPower(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin, GPIO_DR_Mode_Enum PowerMode)
{
    if(PowerMode == Strong_Drive)
    {

        GPIOx->DR &= ~((uint32_t)(0x1) << GPIO_Pin);
    }
    else if(PowerMode == Weak_Drive)
    {

        GPIOx->DR |= (uint32_t)0x1 << GPIO_Pin;
    }
}

/**
  *@brief: GPIO_SetSpeed
  *@note: GPIO Configuration Speed,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@param: Speed selection, can be configured for GPIO_Speed_High or  GPIO_Speed_Low
  *@retval: None
  */
void GPIO_SetSpeed(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin, GPIO_Speed_Enum Speed)
{
    if(Speed == GPIO_Speed_High)
    {

        GPIOx->SR &= ~((uint32_t)(0x1) << GPIO_Pin);
    }
    else if(Speed == GPIO_Speed_Low)
    {

        GPIOx->SR |= (uint32_t)0x1 << GPIO_Pin;
    }
}

/**
  *@brief: GPIO_SetOD
  *@note: GPIO Configuration Open drain,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@param: NewState selection, can be configured for ENABLE or  DISABLE
  *@retval: None
  */
void GPIO_SetOD(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin,FunctionalState NewState)
{
    if(NewState == DISABLE)
    {
        GPIOx->OD &= ~((uint32_t)(0x1) << GPIO_Pin);
    }
    else 
  {
        GPIOx->OD |= (uint32_t)0x1 << GPIO_Pin;
    }
}

/**
  *@brief: GPIO_SetBits
  *@note: GPIO Configuration OUTPUT HIGH,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: None
  */
void GPIO_SetBits(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->DOUTSET = (uint32_t)0x1 << GPIO_Pin;
}

/**
  *@brief: GPIO_ResetBits
  *@note: GPIO Configuration OUTPUT LOW,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: None
  */
void GPIO_ResetBits(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->DOUTRST = (uint32_t)0x1 << GPIO_Pin;
}

/**
  *@brief: GPIO_Toggle
  *@note: GPIO Level Output Flip,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: None
  */
void GPIO_Toggle(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->DOUTFLP = (uint32_t)0x1 << GPIO_Pin;
}

/**
  *@brief: GPIO_Write
  *@note: GPIO Level Output Flip,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: PortVal  GPIO port selection, GPIOx All GPIO Output Level Status
  *@retval: None
  */
void GPIO_Write(GPIO_Type* GPIOx, uint8_t PortVal)
{
    GPIOx->DOUT = PortVal;
}

/**
  *@brief: GPIO_ReadOutputDataBit
  *@note: When the GPIO is configured in output mode, the GPIO level state is read,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: 0 or 1.
  */
uint16_t GPIO_ReadOutputDataBit(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    GPIOx->IE |= 0x1 << GPIO_Pin;
    __NOP();

    return ((GPIOx->DIN >> GPIO_Pin) & 0x1);
}

/**
  *@brief: GPIO_ReadInputDataBit
  *@note: GPIO Read Input Level Status, the GPIO level state is read,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@param: GPIO_Pin  GPIO port selection, can be configured for GPIO_PIN0 ~ GPIO_PIN15
  *@retval: 0 or 1.
  */
uint16_t GPIO_ReadInputDataBit(GPIO_Type* GPIOx, GPIO_PIN_Enum GPIO_Pin)
{
    return ((GPIOx->DIN >> GPIO_Pin) & 0x1);
}

/**
  *@brief: GPIO_ReadInputData
  *@note:GPIOx reads all IO level states, the GPIO level state is read,You can only configure one GPIO per call, not more than one GPIO at the same time.
  *@param: GPIOx GPIO selection, can be configured as GPIOA ~ GPIOD
  *@retval:GPIOx->IDR.
  */
uint16_t GPIO_ReadInputData(GPIO_Type* GPIOx)
{
    return GPIOx->DIN;
}
