/**
  ******************************************************************************
  * @file           : ms60f3xxx_i2c.c
  * @author         : sinomcu.com-SA
  * @brief          : i2c module driver.
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx.h"
#include "ms60f3xxx_scu.h"
#include "ms60f3xxx_i2c.h"
#include "ms60f3xxx_gpio.h"

/**
  * @brief  Initializes the IIC peripheral according to the specified parameters
  *         in the I2C_INIT_T.
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  i2c_init: pointer to an I2C_INIT_T structure that contains
  *         the configuration information for the specified I2C peripheral.
  * @retval None
  */
void I2C_Init(IIC_Type *pi2c, I2C_INIT_T *i2c_init )
{
    uint32_t temp;

    if(i2c_init->mode == SLAVER_MODE)
    {
        pi2c->CR = 0x12;
        pi2c->AR = 0xFC00 | i2c_init->address;
        pi2c->CR1 = 0x0F0500;
    }
    else
    {
        pi2c->CR = 0x10;
        temp = SCU_Get_frequecny() / i2c_init->frequency - 1;

        if(temp < 16)
            temp = 16;

        pi2c->TIMING &= 0x0000ffff;
        pi2c->TIMING |= (temp << 16);
        pi2c->TIMING2 = (100 - i2c_init->duty) * temp / 100;
        pi2c->CR1 = 0x0F0500;
    }
}

/**
  * @brief  I2C_Enable_IT:Set I2C interrupt enable.
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  I2C_Interrupt: where I2C_Interrupt can be I2C_Interrupt_Flag_ENUM to select the I2C Interrupt Flag.
  * @retval None
  */
void I2C_Enable_IT(IIC_Type *pi2c, I2C_Interrupt_Flag_ENUM I2C_Interrupt)
{

    pi2c->IER |= I2C_Interrupt;
}

/**
  * @brief  I2C_Disable_IT:Set I2C interrupt disable.
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  I2C_Interrupt: where I2C_Interrupt can be I2C_Interrupt_Flag_ENUM to select the I2C Interrupt Flag.
  * @retval None
  */
void I2C_Disable_IT(IIC_Type *pi2c, I2C_Interrupt_Flag_ENUM I2C_Interrupt)
{
    pi2c->IDR |= I2C_Interrupt;
}

/**
  * @brief  delay_us:Provides a delay function
  * @param  unsigned long uS:the number of delay
  * @retval None
  */
void delay_us(unsigned long uS)
{
    unsigned long ucA;

    for(ucA = 0; ucA < uS; ucA++)
    {
        __NOP();
			 __NOP();
			 __NOP();
			 __NOP();
			 __NOP();
    }
}

/**
  * @brief  I2CStop:Used to configure the end of IIC sending.
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @retval None
  */
void I2CStop(IIC_Type *pi2c)
{
    uint16_t wait = 500;
    pi2c->CR  = I2C_EN | I2C_STOP;

    while(pi2c->SR != 0x1B)
    {
        wait--;

        if(wait == 0)
        {
            pi2c->CR = 0;
            break;
        }
    }
}

/**
  * @brief  I2CStart:Used to configure the IIC start signal.
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @retval None
  */
void I2CStart(IIC_Type *pi2c)
{
    uint16_t  wait = 10000;
    pi2c->CR1 |= 1;
    pi2c->CR	= I2C_EN | I2C_START;

    while(!(pi2c->CR & I2C_SI))
    {
        wait--;

        if(wait == 0)
        {
            break;
        }
    }

    wait = 100;

    while(!(pi2c->SR == 0x01 || pi2c->SR == 0x02))
    {
        wait--;

        if(wait == 0)
        {
            I2CStop(pi2c);
            return;
        }
    }
}
/**
  * @brief  Used to configure IIC sending Data
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  byte: IIC sending byte
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */

uint8_t I2C_Slave_Send_Data(IIC_Type *pi2c, uint8_t byte, I2C_ACK_ENUM ack)
{
    pi2c->TDR = byte;

    if(ack)
        pi2c->CR = I2C_EN | ack;
    else
        pi2c->CR = I2C_EN;

    return 0;

}

/**
  * @brief  Used to configure IIC sending address
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  address: Slave address
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */
uint8_t I2C_Send_address(IIC_Type *pi2c, uint8_t address, I2C_ACK_ENUM ack)
{
    uint16_t wait = 30000;
    pi2c->TDR = address;
    pi2c->CR &= 0xFE;

    if(ack)
    {
        pi2c->CR = I2C_EN | I2C_ACK;
    }
    else
    {
        pi2c->CR  = I2C_EN;
    }

    while(!(pi2c->CR & I2C_SI))
    {
        wait--;

        if(wait == 0)
        {
            break;
        }
    }

    wait = 5000;

    if(ack)
    {
        while(!((pi2c->SR == 0x03) || (pi2c->SR == 0x08)))
        {
            wait--;

            if(wait == 0)
            {
                I2CStop(pi2c);
                return 0;
            }
        }
    }
    else
    {
        while(!(pi2c->SR == 0x04) || pi2c->SR == 0x09)
        {
            wait--;

            if(wait == 0)
            {
                I2CStop(pi2c);
                return 0;
            }
        }
    }

    return 1;
}
/**
  * @brief  Used to configure IIC to send data
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  byte: Data to be sent
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */
uint8_t I2C_Send_Byte(IIC_Type *pi2c, uint8_t byte, I2C_ACK_ENUM ack)
{

    uint16_t wait = 50000;
    pi2c->TDR = byte;
    pi2c->CR &= 0xFE;

    if(ack)
    {
        pi2c->CR = I2C_EN | I2C_ACK;
    }
    else
    {
        pi2c->CR  = I2C_EN;
    }

    while(!(pi2c->CR & I2C_SI))
    {
        wait--;

        if(wait == 0)
        {
            break;;
        }
    }

    wait = 1000;

    if(ack)
    {
        while(!( pi2c->SR == 0x05 ))
        {
            wait--;

            if(wait == 0)
            {
                I2CStop(pi2c);
                return 0;
            }
        }
    }
    else
    {
        while(!(I2C0->SR == 0x06 ))
        {
            wait--;

            if(wait == 0)
            {
                I2CStop(pi2c);
                return 0;
            }
        }
    }

    return 1;
}

/**
  * @brief  Used to configure IIC to receive data
  * @param  pi2c: where pi2c can be I2C0 or I2C1 to select the I2C peripheral.
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */
uint16_t I2C_Receive_Byte(IIC_Type *pi2c, I2C_ACK_ENUM ack)
{
    uint8_t byte;
    uint16_t wait = 50000;
    pi2c->CR1 = 2;

    if(ack)
    {
        pi2c->CR  = I2C_EN | I2C_ACK;
    }
    else
    {
        pi2c->CR  = I2C_EN;
    }

    while(! (pi2c->CR & I2C_SI))
    {
        wait--;

        if(wait == 0)
        {
            break;
        }
    }

    wait =3000;

    if(!ack)
    {
        while(!( pi2c->SR == 0x0B ))
        {
            wait--;

            if(wait == 0)
            {
                byte = pi2c->RDR ;
                I2CStop(pi2c);
                return 0xFFFF;
            }
        }
    }
    else
    {
        while(!( pi2c->SR == 0x0A ))
        {
            wait--;

            if(wait == 0)
            {
                byte = pi2c->RDR ;
                I2CStop(pi2c);
                return 0xFFFF;
            }
        }
    }

    byte = pi2c->RDR ;
    return byte;
}

/**
  * @brief  Used to Send data
  * @param  pi2c: where pi2c can be I2C1 or I2C2 to select the I2C peripheral.
  * @param  address: Slave address
  * @param  buff: Send data address
  * @param  len: Send data len
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */
uint16_t I2C_Send_Buff(IIC_Type *pi2c, uint8_t address, uint8_t *buff, uint8_t len, I2C_ACK_ENUM ack)
{
    uint8_t i;


    I2CStart(pi2c);

    if( I2C_Send_address(pi2c, address, ack) == 0)
        return 0;


    for(i = 0; i < len; i++)
    {
        if(I2C_Send_Byte(pi2c, buff[i], ack) == 0)
            return 0;
    }

    I2CStop(pi2c);
    return 1;
}

/**
  * @brief  Used to Receive data
  * @param  pi2c: where pi2c can be I2C1 or I2C2 to select the I2C peripheral.
  * @param  address: Slave address
  * @param  buff: Receive data address
  * @param  len: Receive data len
  * @param  ack: Choose whether to ACK confirmation
  * @retval None
  */
uint16_t I2C_Receive_Buff(IIC_Type *pi2c, uint8_t address, uint8_t *buff, uint8_t len, I2C_ACK_ENUM ack)
{
    uint8_t i;
    uint16_t result;
    pi2c->CR1 &= ~4;
    I2CStart(pi2c);

    if( I2C_Send_address(pi2c, address + 1, ack) == 0)
        return 0;

    for(i = 0; i < len; i++)
    {
        if(i < (len - 1) )
        {
            result = I2C_Receive_Byte(pi2c, ack);
        }
        else
        {
            result = I2C_Receive_Byte(pi2c, I2C_NOACK);

        }

        if(result == 0xFFFF)
        {
            buff[i] = result;
            return 0;
        }
        else
            buff[i] = result;
    }

    I2CStop(pi2c);
    return 1;
}

