/**
  ******************************************************************************
  * @file           : ms60f3xxx_iwdg.c
  * @author         : sinomcu.com-SA
  * @brief          : iwdg module driver.
  * @gcc            : toolchain version 5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with 
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_iwdg.h"
#include "ms60f3xxx.h"
#include "ms60f3xxx_scu.h"

/**
  * @brief  Enables or disables write access to IWDG registers.
  * @param  IWDG_WriteAccess: new state of write access to IWDG registers.
  *          This parameter can be one of the following values:
  *            @arg IWDG_WriteAccess_Enable: Enable write access to IWDG registers
  *            @arg IWDG_WriteAccess_Disable: Disable write access to IWDG registers
  * @retval None
  */
__INLINE static void IWDG_WriteAccessCmd(uint32_t IWDG_WriteAccess)
{
    /* Check the parameters */
    IWDG->KR = IWDG_WriteAccess;
}

/**
  * @brief  Deinitializes the IWDG peripheral registers to their default reset values.
  * @param  None
  * @retval None
  */
void IWDG_DeInit(void)
{
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);
}

/**
  * @brief  Reloads WDG counter with value defined in the reload register
  * @param  None
  * @retval None
  */
void IWDG_ReloadCounter(void)
{
    IWDG_WriteAccessCmd(0xAAAA);
}

/**
  * @brief  Enables WDG and load the counter value.
  * @param  Counter: specifies the watchdog counter value,MAX 4095
  * This parameter must be a number between 0x1 and 0xFff
  * @retval None
  */
void IWDG_Init(IWDG_DIV_Enum div, uint16_t Counter)
{
    uint32_t IWDG_counter = 0;
    /* Check the parameters */
    IWDG_WriteAccessCmd(IWDG_START_Enable);
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

    while(IWDG_counter++ < 0x50000)
    {
        if(!IWDG->SR)break;
    }
    IWDG->PR = div;
    IWDG->RLR = Counter;
    IWDG_ReloadCounter();
}

/**
  * @brief  Enables IWDG and load the counter value.
  * @param  Counter: specifies the watchdog counter value.
  *          This parameter must be a number between 0x1 and 0xffff
  * @retval None
  */
void IWDG_Disable(void)
{
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);
}

/**
  * @brief  Sets the WDG Prescaler.
  * @param  WWDG_Prescaler: specifies the WDG Prescaler.
  *          This parameter can be one of the following values:
  *            @arg WDG_Prescaler:
  * @retval None
  */
void IWDG_SetPrescaler(uint32_t WDG_Prescaler)
{
    /*for futher used as set system clock source*/
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);
}

/**
  * @brief  Sets the WDG counter value.
  * @param  Counter: specifies the watchdog counter value.
  *          This parameter must be a number between 0x0001 and 0xFFFF (to prevent
  *          generating an immediate reset).
  * @retval None
  */
void IWDG_SetCounter(uint16_t Counter)
{
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
    IWDG->RLR = Counter;
}

/**
 * @brief  WDG counter Start register
 * @param  None
 * @retval None
 */
void IWDG_Start(void)
{
    IWDG_WriteAccessCmd(IWDG_START_Enable);
}
