/**
  ******************************************************************************
  * @file           : ms60f3xxx_plu.c
  * @author         : sinomcu.com-SA
  * @brief          : PLU module driver.
  * @gcc            : toolchain version   5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  *	basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */  
#include "ms60f3xxx.h"
#include "ms60f3xxx_scu.h"
#include "ms60f3xxx_gpio.h"
#include "ms60f3xxx_plu.h"

/**
  * @brief PLU Mux Select function
  * @param  plu_unit Pointer to a plu instance.
  * @param MUXA_UNION_T   mux_a:mrx_b
  * @param MUXB_UNION_T   mux_b:mrx_b
  * @return None.
  */
void PLU_MuxSelect(PLU_UNIT_T *plu_unit, MUXA_UNION_T mux_a, MUXB_UNION_T mux_b)
{
    uint8_t value;
    value = mux_a.mrx_a;
    value <<= 4;
    value += mux_b.mrx_b;
    plu_unit->MXR = value;

}

/**
  * @brief PLU Function mode set
  * @param plu_unit Pointer to a plu instance.
  * @param function mode to be set.
  * @return None.
  */
void PLU_Function(PLU_UNIT_T *plu_unit, uint8_t function)
{
    plu_unit->FNR = function;
}

/**
  * @brief PLU enable
  * @param enable staus set.
  * @return None.
  */
void PLU_Enable(uint8_t enable)
{
    PLU->CR |= enable;

}

/**
  * @brief PLU config 
  * @param plu_unit Pointer to a plu instance.
  * @param plu_cfr_type Pointer to a plu config register instance.
  * @return None.
  */
void PLU_CFR(PLU_UNIT_T *plu_unit, PLU_CFR_T *plu_cfr_type)
{
    uint32_t value;
    value = plu_cfr_type->plu_clk_inv + plu_cfr_type->plu_d + plu_cfr_type->plu_gpio + plu_cfr_type->plu_out + plu_cfr_type->plu_trigger;
    value += plu_cfr_type->plu_filer_enable + plu_cfr_type->plu_filter_div + plu_cfr_type->plu_filter_num;
    plu_unit->CFR = value;
}

/**
  * @brief PLU Set Data  
  * @param plu_unit  Pointer to a plu instance ;
  * @param adata ;
  * @param bdata ;
  * @return None.
  */
void PLU_SetData(PLU_UNIT_T *plu_unit, PLU_ADATA_Enum adata, PLU_BDATA_Enum bdata)
{
   
    uint8_t value;
    value = adata + bdata;
    plu_unit->data=value;
}

/**
  * @brief PLU interrupt enable 
  * @param n,n 0..3;
  * @param edge,edge mode:NO_EDGE,
  *											FALLING_EDGE,
  *											RISING_EDGE,
  *											BOTH_EDGE;
  * @return None.
  */
void PLU_Enable_IT(uint8_t n, PLU_EDGE_Enum edge)
{
    uint32_t value;
    value = edge;
    value = value << (2 * n);
    PLU->ISR = 0;
    PLU->IER = value;
}

/**
  * @brief PLU interrupt disable 
  * @param n,n 0..3;
  * @param edge,edge mode:NO_EDGE,
  *											FALLING_EDGE,
  *											RISING_EDGE,
  *											BOTH_EDGE;
  * @return None.
  */
void PLU_Disable_IT(uint8_t n, PLU_EDGE_Enum edge)
{
    uint32_t value;
    value = edge;
    value = value << (2 * n);
    PLU->ISR = 0;
    PLU->IDR = value;
}

/**
  * @brief PLU Int deal
  * @param n,n 0..3;
  * @return None.
  */
void PLU_IntDeal(uint8_t n)
{
    uint32_t value;
    value = (3 << (n * 2));

    if(value & PLU->ISR)
    {
        PLU->ISR &= ~value;
    }

}

/**
  * @brief PLU out
  * @param n,n 0..3;
  * @return RESET ,SET.
  */
FlagStatus PLU_Out(uint8_t n)
{
    uint16_t result;
    result = (1 << n);

    if(result & PLU->OUTR)
    {
        return SET;
    }

    return RESET;
}

