/**
  ******************************************************************************
  * @file           : ms60f3xxx_pwm.c
  * @author         : sinomcu.com-SA
  * @brief          : PWM module driver.
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx.h"
#include "ms60f3xxx_pwm.h"

/**
 * @brief  Initialize the PWM source clock.
 * @param  Clock_Config: PWM clock configuration.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus PWM_ClocksInit(PWM_ClOCK_T *Clock_Config)
{
    uint32_t ul_pre   = 0;
    uint32_t ul_div   = 0;
    uint32_t ul_clock = 0;
    ErrorStatus status = SUCCESS;

    if (Clock_Config->ul_clka_diva != 0) /* Clock A */
    {
        ul_pre = Clock_Config->ul_clka_prea;// >> PWM_CLK_PREA_Pos;
        ul_div = Clock_Config->ul_clka_diva;// >> PWM_CLK_DIVA_Pos;

        if((ul_pre < PWM_CLOCK_PRE_MAX) || (ul_div < PWM_CLOCK_DIV_MAX))
            status = SUCCESS;
        else
            return ERROR;

        ul_clock |= ( Clock_Config->ul_clka_prea << PWM_CLK_PREA_Pos ) | ( Clock_Config->ul_clka_diva << PWM_CLK_DIVA_Pos );
    }

    if (Clock_Config->ul_clkb_divb != 0) /* Clock B */
    {
        ul_pre = Clock_Config->ul_clkb_preb;
        ul_div = Clock_Config->ul_clkb_divb;

        if((ul_pre < PWM_CLOCK_PRE_MAX) || (ul_div < PWM_CLOCK_DIV_MAX))
            status = SUCCESS;
        else
            return ERROR;

        ul_clock |= ( Clock_Config->ul_clkb_preb << PWM_CLK_PREB_Pos )| ( Clock_Config->ul_clkb_divb << PWM_CLK_DIVB_Pos );
    }

    PWM->CLK = ul_clock;

    return status;
}

/**
 * @brief  Change the period of the PWM channel.
 * @param  Channel:   Configurations of the specified PWM channel
 * @param  ul_period: New period value.
 * @retval ErrorStatus
 */
ErrorStatus PWM_ChannelUpdatePeriod(PWM_Channel_Enum Channel, uint32_t ul_period)
{
    if(ul_period > 0xFFFF)
        return ERROR;
    else
    {
        if (PWM->CH_NUM[Channel].CDTY > ul_period)
        {
            return ERROR;
        }
        else
        {
            PWM->CH_NUM[Channel].CPRDUPD = ul_period;
            return SUCCESS;
        }
    }
}

/**
 * @brief  Change the period of the PWM channel.
 * @param  Channel:  Configurations of the specified PWM channel
 * @param  ul_pulse: New duty cycle value.
 * @retval ErrorStatus
 */
ErrorStatus PWM_ChannelUpdateDuty(PWM_Channel_Enum Channel, uint32_t ul_pulse)
{
    ErrorStatus status = SUCCESS;
    uint32_t ul_data = PWM->CH_NUM[Channel].CPRD;

    if(ul_pulse > 0xFFFF)
        return ERROR;
    else
    {
        if ( ul_pulse > ul_data )
        {
            status = ERROR;
        }
        else
        {
            PWM->CH_NUM[Channel].CDTYUPD = ul_pulse;
        }

        return status;
    }
}
/**
 * @brief  Return channel counter value.
 * @param  Channel: Configurations of the specified PWM channel
 * @retval channel Counter value.
 */
uint32_t PWM_ChannelGetCounter(PWM_Channel_Enum Channel)
{
    return PWM->CH_NUM[Channel].CCNT;
}

/**
 * @brief  Enable the PWM channel.
 * @note   The PWM channel should be initialized before it is enabled.
 * @param  Channel: Configurations of the specified PWM channel.
 * @retval None
 */
void PWM_ChannelEnable(PWM_Channel_Enum Channel)
{
    PWM->ENA = (1 << Channel);
}

/**
 * @brief  Disable the PWM channel.
 * @note   A disabled PWM channel can be re-initialized.
 * @param  Channel: Configurations of the specified PWM channel.
 * @retval None
 */
void PWM_ChannelDisable(PWM_Channel_Enum Channel)
{
    PWM->DIS = (1 << Channel);
}

/**
 * @brief  Check which PWM channel is enabled.
 * @retval Bitmask of enabled PWM channel(s).
 * @retval None
 */
uint32_t PWM_ChannelGetStatus(void)
{
    return PWM->SR;
}

/**
 * @brief  Get channel counter event and fault protection trigger interrupt status.
 * @retval Channel counter event and fault protection trigger interrupt status.
 * @retval None
 */
uint32_t PWM_ChannelGet_IT1_Status(void)
{
    return PWM->ISR1;
}

/**
 * @brief  Get channel counter event and fault protection trigger interrupt mask.
 * @retval Channel counter event and fault protection trigger interrupt mask.
 * @retval None
 */
uint32_t PWM_ChannelGet_IT1_Mask(void)
{
    return PWM->IMR1;
}

/**
 * @brief  Enable the interrupt of a channel counter event.
 * @param  Channel: Channel number to enable counter event interrupt.
 * @retval None
 */
void PWM_Channel_Enable_IT(PWM_Channel_Enum Channel)
{
    PWM->IER1 = (1 << Channel);
}

/**
 * @brief  Disable the interrupt of a channel counter event.
 * @param  Channel: Bitmask of channel number to disable counter event interrupt.
 * @retval None
 */
void PWM_Channel_Disable_IT(PWM_Channel_Enum Channel)
{
    PWM->IDR1 = (1 << Channel);
}

/**
 * @brief  Enable the interrupt of a channel fault protection.
 * @param  Channel: Channel number to enable fault protection interrupt.
 * @retval None
 */
void PWM_ChannelFault_Enable_IT(PWM_Channel_Enum Channel)
{
    PWM->IER1 = 1 << (uint32_t)(Channel + 16);
}

/**
 * @brief  Disable the interrupt of a channel fault protection.
 * @param  Channel: Bitmask of channel number to disable fault protection interrupt.
 * @retval None
 */
void PWM_ChannelFault_Disable_IT(PWM_Channel_Enum Channel)
{
    PWM->IDR1 = 1 << (uint32_t)(Channel + 16);
}

/**
 * @brief  Change output selection of the PWM channel.
 * @param  p_output: pointer to a PWM_OutputSelection_T structure.
 * @param  Channel:  Configurations of the specified PWM channel.
 * @retval None
 */
void PWM_ChannelUpdateOutput(PWM_OutputSelect_T *p_output, PWM_Channel_Enum Channel)
{
    uint32_t OverrideValue = 0;

    uint32_t OverridePwmH = p_output->PWMH_OutputSelect;
    uint32_t OverridePwmL = p_output->PWML_OutputSelect;
    uint32_t PwmH_Level    = p_output->PWMH_OovLevel;
    uint32_t PwmL_Level    = p_output->PWML_OovLevel;
    /* Change override output level */
    OverrideValue  = PWM->OOV;
    OverrideValue &= ~((PWM_OOV_OOVH0 | PWM_OOV_OOVL0) << Channel);
    OverrideValue |= (((PwmL_Level << 16) | PwmH_Level) << Channel);
    PWM->OOV  		 = OverrideValue;

    /* Apply new output selection */
    if (p_output->UpdateReg)
    {
        PWM->OSSUPD = ((OverridePwmL << Channel) << 16) | (OverridePwmH << Channel);
        PWM->OSCUPD = ((!OverridePwmL << Channel) << 16) | (!OverridePwmH << Channel);
    }
    else
    {
        PWM->OSS = ((OverridePwmL << Channel) << 16) | (OverridePwmH << Channel);
        PWM->OSC = ((!OverridePwmL << Channel) << 16) | (!OverridePwmH << Channel);
    }
}
/**
 * @brief  Dead-Time Generator Enable
 * @param  Channel: PWM channel number to enable.
 * @retval None
 */
void PWM_ChannelDeadTimeEnable(PWM_Channel_Enum Channel)
{
    PWM->CH_NUM[Channel].CMR |= PWM_CMR_DTE;
}

/**
 * @brief  Dead-Time Generator Disable.
 * @param  Channel: PWM channel number to disable.
 * @retval None
 */
void PWM_ChannelDeadTimeDisable(PWM_Channel_Enum Channel)
{
    PWM->CH_NUM[Channel].CMR &= (~PWM_CMR_DTE);
}

/**
 * @brief  Update Now dead-time value for PWM outputs.
 * @param  Channel: Configurations of the specified PWM channel.
 * @param  ui_DeadTimePwmH New dead-time value for PWMH output.
 * @param  ui_DeadTimePwmL New dead-time value for PWML output.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus PWM_ChannelDeadTime(PWM_Channel_Enum Channel, uint16_t ui_DeadTimePwmH, uint16_t ui_DeadTimePwmL)
{
    if((ui_DeadTimePwmH > 0x0FFF) || (ui_DeadTimePwmL > 0x0FFF))
        return ERROR;
    else
    {
        /* Write channel dead time update register */
        PWM->CH_NUM[Channel].DT = PWM_DT_DTL(ui_DeadTimePwmL) | PWM_DT_DTH(ui_DeadTimePwmH);
        return SUCCESS;
    }
}
/**
 * @brief  Change dead-time value for PWM outputs.
 * @param  Channel: Configurations of the specified PWM channel.
 * @param  ui_DeadTimePwmH New dead-time value for PWMH output.
 * @param  ui_DeadTimePwmL New dead-time value for PWML output.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus PWM_ChannelUpdateDeadTime(PWM_Channel_Enum Channel, uint16_t ui_DeadTimePwmH, uint16_t ui_DeadTimePwmL)
{
    if((ui_DeadTimePwmH > 0x0FFF) || (ui_DeadTimePwmL > 0x0FFF))
        return ERROR;
    else
    {
        /* Write channel dead time update register */
        PWM->CH_NUM[Channel].DTUPD = PWM_DTUPD_DTLUPD(ui_DeadTimePwmL) | PWM_DTUPD_DTHUPD(ui_DeadTimePwmH);
        return SUCCESS;
    }
}

/**
 * @brief  Initialize the behavior of a fault input.
 * @param  p_fault Configurations of a fault input.
 * @retval None
 */
void PWM_FaultInit(PWM_Fault_T *p_fault)
{
    uint32_t fault_id  = p_fault->Fault_id;
    uint32_t fault_reg = PWM->FMR;

    uint32_t FaultEnableReg = 0;
    uint32_t Channel        = p_fault->Channel;

    /** Polarity of fault input */
    if (p_fault->Polarity == PWM_HIGH)
        fault_reg |= fault_id;
    else
        fault_reg &= ~fault_id;

    if (p_fault->Clear)
        fault_reg |= (fault_id << 8);
    else
        fault_reg &= ~(fault_id << 8);

    if (p_fault->Filtered)
        fault_reg |= (fault_id << 16);
    else
        fault_reg &= ~(fault_id << 16);

    PWM->FMR = fault_reg;

    /* Fault Protection Value Register */
    if (p_fault->Fault_Output_PWMH == PWM_HIGHZ)
        PWM->FPV2 |= (0x01 << Channel);
    else
    {
        PWM->FPV2 &= ~(0x01 << Channel);

        if (p_fault->Fault_Output_PWMH == PWM_HIGH)
            PWM->FPV1 |= (0x01 << Channel);
        else
            PWM->FPV1 &= (~(0x01 << Channel));
    }

    if (p_fault->Fault_Output_PWML == PWM_HIGHZ)
        PWM->FPV2 |= ((0x01 << Channel) << 16);
    else
    {
        PWM->FPV2 &= ~((0x01 << Channel) << 16);

        if (p_fault->Fault_Output_PWML == PWM_HIGH)
            PWM->FPV1 |= ((0x01 << Channel) << 16);
        else
            PWM->FPV1 &= (~((0x01 << Channel) << 16));
    }

    /* Fault Protection Enable Register */
    FaultEnableReg  = 0;
    Channel        *= 8;
    FaultEnableReg  = PWM->FPE;
    FaultEnableReg &= ~(0xFF << Channel);
    FaultEnableReg |= ((fault_id) << Channel);
    PWM->FPE        = FaultEnableReg;
}

/**
 * @brief  Get fault status.
 * @retval Bitmask of IDs of all active faulty.
 */
uint32_t PWM_FaultGetStatus(void)
{
    return ((PWM->FSR >> 8) & 0xFF);
}

/**
 * @brief  Get the level of a fault input.
 * @param  id:    Faulty ID.
 * @retval Level of the fault input.
 */
PWM_Level_Enum PWM_FaultGetInputLevel(PWM_FaultId_Enum id)
{
    uint32_t fault_status_reg = ((PWM->FSR >> 8) & 0xFF);
    fault_status_reg >>= id;

    return ((fault_status_reg & 1) ? PWM_HIGH : PWM_LOW);
}

/**
 * @brief  Clear a fault input.
 * @param  id: Faulty ID.
 * @retval None
 */
void PWM_FaultClearStatus(PWM_FaultId_Enum id)
{
    PWM->FCR = id;
}

/**
 * @brief  Initialize PWM comparison unit.
 * @param  p_cmp: Configurations of PWM comparison tagged by unit.
 * @retval None
 */
void PWM_CMP_Init(PWM_CMP_T *p_cmp)
{
    uint32_t unit = p_cmp->uc_unit;

    PWM->CMP[unit].CMPV = p_cmp->ul_value | (p_cmp->Is_Decrementing << 24);
    PWM->CMP[unit].CMPM = p_cmp->ul_trigger |	p_cmp->ul_period | p_cmp->ul_update_period;

    if (p_cmp->PulseOnLine0)
        PWM->ELMR[0] |= (1 << unit);
    else
        PWM->ELMR[0] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine1)
        PWM->ELMR[1] |= (1 << unit);
    else
        PWM->ELMR[1] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine2)
        PWM->ELMR[2] |= (1 << unit);
    else
        PWM->ELMR[2] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine3)
        PWM->ELMR[3] |= (1 << unit);
    else
        PWM->ELMR[3] &= ~((uint32_t) (1 << unit));

    if (p_cmp->Enable)
        PWM->CMP[unit].CMPM |= PWM_CMPM_CEN;
    else
        PWM->CMP[unit].CMPM &= ~PWM_CMPM_CEN;
}

/**
 * @brief  Change the setting of PWM comparison.
 * @param  p_cmp: Configurations of PWM comparison tagged by unit.
 * @retval None
 */
void PWM_CMP_ChangeSetting(PWM_CMP_T *p_cmp)
{
    uint32_t unit = p_cmp->uc_unit;

    PWM->CMP[unit].CMPVUPD = PWM_CMPV_CV(p_cmp->ul_value) | (p_cmp->Is_Decrementing << 24);

    PWM->CMP[unit].CMPMUPD = PWM_CMPM_CTR(p_cmp->ul_trigger) | PWM_CMPM_CPR(p_cmp->ul_period) |	PWM_CMPM_CUPR(p_cmp->ul_update_period);

    if (p_cmp->PulseOnLine0)
        PWM->ELMR[0] |= (1 << unit);
    else
        PWM->ELMR[0] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine1)
        PWM->ELMR[1] |= (1 << unit);
    else
        PWM->ELMR[1] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine2)
        PWM->ELMR[2] |= (1 << unit);
    else
        PWM->ELMR[2] &= ~((uint32_t) (1 << unit));

    if (p_cmp->PulseOnLine3)
        PWM->ELMR[3] |= (1 << unit);
    else
        PWM->ELMR[3] &= ~((uint32_t) (1 << unit));

    if (p_cmp->Enable)
        PWM->CMP[unit].CMPMUPD |= PWM_CMPM_CEN;
    else
        PWM->CMP[unit].CMPMUPD &= ~PWM_CMPM_CEN;
}

/**
 * @brief  Report the value of the comparison period counter.
 * @param  Unit: PWM comparison unit number.
 * @retval Value of the comparison period counter.
 * @retval None
 */
uint32_t PWM_CMP_GetPeriodCounter(PWM_CMP_Unit_Enum Unit)
{
    return (PWM_CMPM_CPRCNT(PWM->CMP[Unit].CMPM)	>> PWM_CMPM_CPRCNT_Pos) & 0x0000000F;
}

/**
 * @brief  Report the value of the comparison update period counter.
 * @param  Unit: PWM comparison unit number.
 * @retval Value of the comparison update period counter.
 * @retval None
 */
uint32_t PWM_CMP_GetUpdateCounter(PWM_CMP_Unit_Enum Unit)
{
    return (PWM_CMPM_CUPRCNT(PWM->CMP[Unit].CMPM) >> PWM_CMPM_CUPRCNT_Pos) & 0x0000000F;
}

/**
 * @brief  Enable the interrupt of comparison.
 * @param  ul_sources: Bitmask of comparison unit.
 *  									 PWM_IER2_WRDY  (0x1u << 0)
 * 										 PWM_IER2_UNRE  (0x1u << 3)
 * 									   PWM_IER2_CMPM0 (0x1u << 8)
 * 										 PWM_IER2_CMPM1 (0x1u << 9)
 * 										 PWM_IER2_CMPM2 (0x1u << 10)
 * 										 PWM_IER2_CMPM3 (0x1u << 11)
 * 										 PWM_IER2_CMPU0 (0x1u << 16)
 * 										 PWM_IER2_CMPU1 (0x1u << 17)
 * 										 PWM_IER2_CMPU2 (0x1u << 18)
 * 										 PWM_IER2_CMPU3 (0x1u << 19)
 * @param  IT_Type:    PWM_CMP_MATCH enables the match interrupt of the unit.
                       PWM_CMP_UPDATE enables the update interrupt of the comparison unit.
 * @retval None
 */
void PWM_CMP_Enable_IT(uint32_t ul_sources, PWM_CMP_IT_Enum IT_Type)
{
    if (IT_Type == PWM_CMP_MATCH)
    {
        PWM->IER2 = ((1 << ul_sources) << 8);
    }
    else if (IT_Type == PWM_CMP_UPDATE)
    {
        PWM->IER2 = ((1 << ul_sources) << 16);
    }
    else
    {
        /* Do Nothing */
    }
}

/**
 * @brief  Disable the interrupt of comparison.
 * @param  ul_sources: Bitmask of comparison unit.
 * 	  								 PWM_IDR2_CMPM0 (0x1u << 8)
 * 										 PWM_IDR2_CMPM1 (0x1u << 9)
 * 										 PWM_IDR2_CMPM2 (0x1u << 10)
 * 										 PWM_IDR2_CMPM3 (0x1u << 11)
 * 										 PWM_IDR2_CMPU0 (0x1u << 16)
 * 										 PWM_IDR2_CMPU1 (0x1u << 17)
 * 										 PWM_IDR2_CMPU2 (0x1u << 18)
 * 										 PWM_IDR2_CMPU3 (0x1u << 19)
 * @param  IT_Type:    PWM_CMP_MATCH disables the match interrupt of the unit.
                       PWM_CMP_UPDATE disables the update interrupt of the comparison unit.
 * @retval None
 */
void PWM_CMP_Disable_IT(uint32_t ul_sources, PWM_CMP_IT_Enum IT_Type)
{
    if (IT_Type == PWM_CMP_MATCH)
    {
        PWM->IDR2 = ((1 << ul_sources) << 8);
    }
    else if (IT_Type == PWM_CMP_UPDATE)
    {
        PWM->IDR2 = ((1 << ul_sources) << 16);
    }
    else
    {
        /* Do Nothing */
    }
}

/**
 * @brief  Initialize synchronous channels update mode and period.
 * @param  mode:            Synchronous channels update mode.
 * @param  uc_UpdatePeriod: Time between each update of the synchronous channels.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus  PWM_SyncInit(PWM_SyncUpdateMode_Enum mode, uint8_t uc_UpdatePeriod)
{
    uint32_t sync_mode = PWM->SCM;

    sync_mode &= ~PWM_SCM_UPDM_Msk;
    sync_mode |= mode;

    PWM->SCM = sync_mode;

    if(uc_UpdatePeriod <= 0x0F)
        PWM->SCUP = PWM_SCUP_UPR(uc_UpdatePeriod);
    else
        return ERROR;

    return SUCCESS ;
}

/**
 * @brief  Unlock the update of synchronous channels.
 * @note   After unlock, synchronous channels will be updated at the next PWM period.
 * @retval None
 */
void PWM_SyncUnlockUpdate(void)
{
    PWM->SCUC = PWM_SCUC_UPDULOCK;
}

/**
 * @brief  Change the wanted time between each update of the synchronous channels.
 * @param  uc_UpdatePeriod: Time between each update of the synchronous channels.
 * @retval None
 */
ErrorStatus PWM_SyncChangePeriod(uint8_t uc_UpdatePeriod)
{
    if(uc_UpdatePeriod <= 0xF)
    {
        PWM->SCUPUPD = PWM_SCUPUPD_UPRUPD(uc_UpdatePeriod);
        return SUCCESS;
    }
    else
        return ERROR;
}

/**
 * @brief  Get the value of the synchronization update period counter.
 * @retval Value of the synchronization update Period Counter.
 * @retval None
 */
uint32_t PWM_SyncGetPeriodCounter(void)
{
    return PWM_SCUP_UPRCNT(PWM->SCUP);
}

/**
 * @brief  Enable the interrupt of synchronous channel.
 * @param  ul_sources: Bitmask of PWM synchronous channels interrupt sources.
 * 										 PWM_IER2_WRDY  (0x1u << 0)
 * 				             PWM_IER2_UNRE  (0x1u << 3)
 * @retval  None
 */
void PWM_Sync_Enable_IT(uint32_t ul_sources)
{
    PWM->IER2 = ul_sources;
}

/**
 * @brief  Disable the interrupt of synchronous channels.
* @param   ul_sources: Bitmask of PWM synchronous channels interrupt sources.
 *           		       PWM_IDR2_WRDY  (0x1u << 0)
 * 				             PWM_IDR2_UNRE  (0x1u << 3)
 * @retval None
 */
void PWM_Sync_Disable_IT(uint32_t ul_sources)
{
    PWM->IDR2 = ul_sources;
}

/**
 * @brief  Enable PWM write protect.
 * @param  E_Group:       Bitmask of PWM register group.
 * @param  E_ProtectMode: Boolean of software protect. True for software protect and false for hardware protect.
 * @retval  None
 */
void PWM_ProtectEnable(PWM_ProtectReg_Enum E_Group, PWM_ProtectMode_Enum E_ProtectMode)
{
    if (E_ProtectMode == PWM_Enable_SW_Protect)
    {
        PWM->WPCR = PWM_WPCR_WPKEY_PASSWD | (E_Group << 2) | PWM_WPCR_WPCMD_ENABLE_SW_PROT;
    }
    else
    {
        PWM->WPCR = PWM_WPCR_WPKEY_PASSWD | (E_Group << 2) | PWM_WPCR_WPCMD_ENABLE_HW_PROT;
    }
}

/**
 * @brief  Disable PWM write protect.
 * @param  note Only a hardware reset of PWM controller can disable hardware write protect.
 * @param  E_Group:     Bitmask of PWM register group.
 * @retval None.
 */
void PWM_ProtectDisable(PWM_ProtectReg_Enum E_Group)
{
    PWM->WPCR = PWM_WPCR_WPKEY_PASSWD | (E_Group << 2) | PWM_WPCR_WPCMD_DISABLE_SW_PROT;
}

/**
 * @brief  Get PWM write protect status.
 * @note   Only a hardware reset of PWM controller can disable hardware write protect.
 * @param  p_protect: Pointer to a structure stored PWM protect status.
 * @retval ENABLE or DISABLE.
 * @retval None
 */
FunctionalState PWM_ProtectGetStatus(PWM_Protect_T *p_protect)
{
    uint32_t wpsr = PWM->WPSR;

    p_protect->ul_hw_status = (wpsr >> 8) & 0x3F;
    /** Bitmask of PWM register group for write protect software status */
    p_protect->ul_sw_status = (wpsr & 0x3F);

    if ((PWM_WPSR_WPVS & wpsr) == PWM_WPSR_WPVS)
    {
        p_protect->ul_offset = (wpsr & PWM_WPSR_WPVSRC_Msk) >> PWM_WPSR_WPVSRC_Pos;
        return ENABLE;
    }
    else
        return DISABLE;
}

/**
 * @brief  Get interrupt status of synchronous channels and comparison.
 * @retval Interrupt status of synchronous channels and comparison.
 * @retval None
 */
uint32_t PWM_CMP_Get_IT2_Status(void)
{
    return PWM->ISR2;
}

/**
 * @brief  Get interrupt mask of synchronous channels and comparison.
 * @retval Interrupt mask of synchronous channels and comparison.
 * @retval None
 */
uint32_t PWM_CMP_Get_IT2_Mask(void)
{
    return PWM->IMR2;
}

/**
 * @brief  Initialize PWM stepper motor mode.
 * @param  Pair: 			PWM channels used by stepper motor.
 * @param  DIR: 			Set true to use down counter. Set false to use up counter.
 * @param  EnableGray:Set true to enable gray count generation. Set false to disable it.
 * @retval None
 */
void PWM_StepperMotorInit(PWM_StepperMotor_Enum Pair, PWM_StepperMotor_DIR_Enum DIR, FunctionalState EnableGray)
{
    uint32_t motor = PWM->SMMR;
    motor &= ~((PWM_SMMR_GCEN0 | PWM_SMMR_DOWN0) << Pair);
    motor |= ((EnableGray | (DIR << 16)) << Pair);
    PWM->SMMR = motor;
}

/**
 * @brief  Change leading edge value and mode.
 * @param  p_channel: 							Configurations of the specified PWM channel.
 * @param  ul_LeadingEdgeDelay: 		Leading-Edge Blanking Delay for TRGINx.
 * @param  LeadingEdgeBlankingMode: New additional edge mode.
 * @retval None
 */
void PWM_ChannelUpdateLeadingEdge(PWM_LeadingEdge_T *p_channel, uint32_t ul_LeadingEdgeDelay,
                                  PWM_LEBR_Enum LeadingEdgeBlankingMode)
{
    p_channel->ul_LeadingEdgeDelay = ul_LeadingEdgeDelay;
    p_channel->LeadingEdgeBlankingMode = LeadingEdgeBlankingMode;

    /* Write channel leading edge update register */
    if (p_channel->channel == 1)
    {
        PWM->LEBR1 = PWM_LEBR1_LEBDELAY(ul_LeadingEdgeDelay) | LeadingEdgeBlankingMode;
    }
    else if (p_channel->channel == 2)
    {
        PWM->LEBR2 = PWM_LEBR2_LEBDELAY(ul_LeadingEdgeDelay) | LeadingEdgeBlankingMode;
    }
}

/**
 * @brief  set external trigger mode.
 * @param  p_channel: Configurations of the specified PWM channel.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus PWM_SetExtTrgMode(PWM_ETRG_T *PWM_ETRG_Config)
{
    uint32_t ul_mode = 0;

    if(PWM_ETRG_Config->ul_MaxCnt <= 0x0FFFFF )
    {
        ul_mode =   PWM_ETRG_Config->ul_EtrgRfen
                    | PWM_ETRG_Config->EtrgTrgEdge
                    | PWM_ETRG_Config->EtrgTrgFilt
                    | PWM_ETRG_Config->EtrgTrgMode
                    | PWM_ETRG_Config->EtrgTrgSrc
                    | PWM_ETRG_Config->ul_MaxCnt;

        if (PWM_ETRG_Config->EtrgCh == PWM_ETRG_CHANNEL1)
        {
            PWM->ETRG1 = ul_mode;
        }
        else if (PWM_ETRG_Config->EtrgCh == PWM_ETRG_CHANNEL2)
        {
            PWM->ETRG2 = ul_mode;
        }

        return SUCCESS ;
    }
    else
        return ERROR;
}

/**
 * @brief  Initialize one PWM channel.
 * @param  p_channel: Configurations of the specified PWM channel.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus PWM_ChannelInit(PWM_Channel_T *p_channel)
{
 
    uint32_t tmp_reg  = 0;
    uint8_t  ch_num   = p_channel->Channel;

    if((p_channel->ul_pwml_deadtime > 0xFFF) || p_channel->ul_pwmh_deadtime > 0xFFF)
        return ERROR;
    if(( p_channel->ul_period > 0xFFFF ) || (p_channel->ul_pulse > 0xFFFF))
			  return ERROR;
		
    /* Channel Mode/Clock Register */
    tmp_reg =   (p_channel->Prescaler & 0xF) |
                (p_channel->Alignment) |
                (p_channel->Polarity) |
                (p_channel->CounterEventSelect)  |
                (p_channel->CMR_UpdateSelect)        |
                (p_channel->PPM_Mode);
    if(p_channel->DeadTimeGenerator == ENABLE )
			tmp_reg |= PWM_CMR_DTE;
		if(p_channel->PWMH_OutputInverted == ENABLE)
			tmp_reg |= PWM_CMR_DTHI;
		if(p_channel->PWML_OutputInverted == ENABLE)
			tmp_reg |= PWM_CMR_DPOLI;
		if(p_channel->PolarityInverted == ENABLE)
			tmp_reg |= PWM_CMR_DPOLI;
		
    PWM->CH_NUM[ch_num].CMR = tmp_reg;
		
		/* Channel Period Register and Channel Duty Cycle Register */
		PWM->CH_NUM[ch_num].CPRD = p_channel->ul_period;
		PWM->CH_NUM[ch_num].CDTY = p_channel->ul_pulse;
		
    /* Channel Dead Time Register */
    if (p_channel->DeadTimeGenerator )
    {
        PWM->CH_NUM[ch_num].DT = PWM_DT_DTL(p_channel->ul_pwml_deadtime) | PWM_DT_DTH(p_channel->ul_pwmh_deadtime);
    }
    return SUCCESS ;
}

/**
 * @brief  Synchronous channel enable.
 * @param  p_channel: Configurations of the specified PWM channel.
 * @retval None
 */
void PWM_ChannelSyncEnable(PWM_Channel_Enum Channel)
{
    /* Sync Channels Mode Register */
    PWM->SCM |= (0x01 << Channel);
}

/**
 * @brief  Synchronous channel disable.
 * @param  p_channel: Configurations of the specified PWM channel.
 * @retval None
 */
void PWM_ChannelSyncDisable(PWM_Channel_Enum Channel)
{
    /* Sync Channels Mode Register */
    PWM->SCM &= ~(0x01 << Channel);
}
