/**
  ******************************************************************************
  * @file           : ms60f3xxx_scu.c
  * @author         : sinomcu.com-SA
  * @brief          : Main program body
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  *	basic chip information support.
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_scu.h"
#include "ms60f3xxx.h"
#include "ms60f3xxx_gpio.h"
#include "ms60f3xxx_exti.h"
#include "ms60f3xxx_flash.h"

#define HXTAL_FREQNCY_VALUE  8000000
uint32_t SystemCoreClock = 0;

/**
  *@brief: SCU_Unlock
  *@note:  Enable SCU register write operation.
  *@param:  None
  *@retval: None
  */
void SCU_Unlock(void)
{
    SCU->WPR  = SCU_UNLOCK_KEY;
}

/**
  *@brief: SCU_Lock
  *@note:  Disable SCU register write operation.
  *@param:  None
  *@retval: None
  */
void SCU_Lock(void)
{
    SCU->WPR  = 0;
}

/**
  *@brief: SCU_Get_frequecny
  *@note:  Gets the current configuration frequency of the system.
  *@param:  None
  *@retval: return the system frequecy
  */
uint32_t SCU_Get_frequecny(void)
{

    uint32_t frequency = 16000000;
    uint8_t clock_source;
    uint8_t sel16M;
    uint8_t xtal_type;
    uint8_t pll_type;
    clock_source = ((SCU->CLK_SEL_CR >> 2) & 0x03);
    sel16M = ((SCU->CLK_SEL_CR >> 1) & 0x01);

    switch(clock_source)
    {
        case 0:
            frequency = (32768 >> (SCU->CLK_DIV_CSR & 0x07)) ;
            break;

        case 1:
            if(!sel16M)
                frequency = (16000000 >> (SCU->CLK_DIV_CSR & 0x07));
            else
            {
                xtal_type = (SCU->PLL_CR & 0x30);

                switch(xtal_type)
                {
                    case HXTAL_12M:
                        frequency = (12000000 >> (SCU->CLK_DIV_CSR & 0x07));
                        break;

                    case HXTAL_8M:
                        frequency = (8000000 >> (SCU->CLK_DIV_CSR & 0x07));
                        break;

                    case HXTAL_4M:
                        frequency = (4000000 >> (SCU->CLK_DIV_CSR & 0x07));
                        break;

                    case HXTAL_16M:
                        frequency = (16000000 >> (SCU->CLK_DIV_CSR & 0x07));
                        break;
                }
            }

            break;

        case 2:
            pll_type =  (SCU->PLL_CR & 0x0F);

            switch( pll_type)
            {
                case PLL_72M:
                    frequency = (72000000 >> (SCU->CLK_DIV_CSR & 0x07));
                    break;

                case PLL_64M:
                    frequency = (64000000 >> (SCU->CLK_DIV_CSR & 0x07));
                    break;

                case PLL_48M:
                    frequency = (48000000 >> (SCU->CLK_DIV_CSR & 0x07));
                    break;

                case PLL_32M:
                    frequency = (32000000 >> (SCU->CLK_DIV_CSR & 0x07));
                    break;

                case PLL_36M:
                    frequency = (36000000 >> (SCU->CLK_DIV_CSR & 0x07));
                    break;
            }

            break;
    }

    return frequency;
}


/**
  *@brief: SCU_SetSysClock
  *@note:  set system clock parameter incuding LRC,HRC HXTAL,LXTAL,PLL.
  *@param:  CLOCK_SEL_TYPE clocktype
  * typedef enum
  *{
  * CLK_32K_USE_HRC_LRC=0,
  * CLK_32K_USE_HRC_LXTAL=1,
  * CLK_32K_USE_HXTAL_LRC=2,
  * CLK_32K_USE_HXTAL_LXTAL=3,
  * CLK_16M_USE_HRC_LRC=4,
  * CLK_16M_USE_HRC_LXTAL=5,
  * CLK_16M_USE_HXTAL_LRC=6,
  * CLK_16M_USE_HXTAL_LXTAL=7,
  * CLK_PLL_USE_HRC_LRC=8,
  * CLK_PLL_USE_HRC_LXTAL=9,
  * CLK_PLL_USE_HXTAL_LRC=10,
  * CLK_PLL_USE_HXTAL_LXTAL=11,
  *}CLOCK_SEL_TYPE;
  *Select one parameter as clock control register value
  *HXTAL_FREQ_TYP  hxtal_freq_type refrence from
  *typedef enum
  *{
  * HXTAL_12M =0,
  * HXTAL_8M  =0x10,
  * HXTAL_4M	 =0x20,
  * HXTAL_16M =0x30,
  * HRC_16M   =0x30,
  *}HXTAL_FREQ_TYP ;  this parameter is HXTAL frequecy
  *XTAL_SEL_TYPE xtal_type reference from
  *typedef enum
  *{
  *   NO_XTAL   =0,   //HRC,LRC
  *	 LXTAL_TYPE=1,   //HRC,LXTAL
  *	 HXTAL_TYPE=4,   //HXTAL,LRC
  *	 HXTAL_LXTAL_TYPE=5//HXTAL,LXTAL
  *}XTAL_SEL_TYPE;
  *select one as pameter ,
  *PLL_FREQ_TYPE plltype
  *Reference from
  *typedef enum
  *{
  * PLL_72M =5,
  *	PLL_64M =7,
  *	PLL_48M =1,
  *	PLL_36M =13,
  *	PLL_32M =15,
  *}PLL_FREQ_TYPE;
  *select one as parametger
  *CLOCK_DIV_TYPE clockdiv
  *Reference from
  *typedef enum
  *{
  *  SCU_SYS_Div1    = 0,
  *  SCU_SYS_Div2    = 1,
  *  SCU_SYS_Div4    = 2,
  *  SCU_SYS_Div8    = 3,
  *  SCU_SYS_Div16   = 4,
  *  SCU_SYS_Div32   = 5,
  *  SCU_SYS_Div64   = 6,
  *  SCU_SYS_Div128  = 7
  *} CLOCK_DIV_TYPE;
  *select one as parameter
  *@retval: None
  */
void SCU_SetSysClock(CLOCK_SEL_Enum clocktype, HXTAL_FREQ_Enum hxtal_freq_type, XTAL_SEL_Enum xtal_type, PLL_FREQ_Enum pll_freq, CLOCK_DIV_Enum clockdiv)
{
    volatile uint32_t counter=0;
    uint32_t temp;
    SCU_Unlock();
    SCU->CLK_SEL_CR = 4;

    if(clocktype >= 8)
    {
        SCU_PeriphClockCmd(SCU_ADC_CLKEN, ENABLE);
        ADC->TPS_CR = 8;
        SCU_PeriphClockCmd(SCU_ADC_CLKEN, DISABLE);
        MEM_MAP(FLS_CFG_EN) = FLASH_UNLOCK_KEY;

        if((pll_freq == PLL_72M) || (pll_freq == PLL_64M))
        {
            FLASH->ACR = 6;
        }
        else
        {
            FLASH->ACR = 2;
        }

        MEM_MAP(FLS_CFG_EN) = FLASH_LOCK_KEY;
    }

    temp = SCU->CLK_CSR;
    SCU->CLK_DIV_CSR = clockdiv;

    while(SCU->CLK_DIV_CSR & DIV_LOADING);

    switch(xtal_type)
    {
        case NO_XTAL:
        {
            SCU->CLK_EN_CR |= HRC_EN_ENUM;
        }
        break;

        case HXTAL_TYPE:
        {

            if(hxtal_freq_type == HXTAL_4M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_4M;
                SCU->CLK_CSR = temp;
            }
            else if(hxtal_freq_type == HXTAL_8M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_8M;
                SCU->CLK_CSR = temp;
            }
            else if(hxtal_freq_type == HXTAL_12M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_12M;
                SCU->CLK_CSR = temp;
            }
            else
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_16M;
                SCU->CLK_CSR = temp;
            }

            SCU->CLK_EN_CR &= ~HXTAL_EN_ENUM;


            for(counter = 0; counter < 1000; counter++) 
						;
            SCU->CLK_EN_CR |= HXTAL_EN_ENUM;
            counter = 0;

            while(!(SCU->CLK_CSR & HXTAL_OSCILATED))
            {
                if(counter++ > 0xfffff)
                    break;
            }

            SCU->CLK_EN_CR &= ~HXTAL_EN_ENUM;
            SCU->CLK_EN_CR |= HXTAL_EN_ENUM;
        }
        break;

        case LXTAL_TYPE:
        {
            SCU->CLK_EN_CR |= LXTAL_EN_ENUM;
            counter = 0;

            while(!(SCU->CLK_CSR & LXTAL_OSCILATED))
            {
                if(counter++ > 0xffff)
                    break;
            }
        }
        break;

        case HXTAL_LXTAL_TYPE:
        {
            if(hxtal_freq_type == HXTAL_4M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_4M;
                SCU->CLK_CSR = temp;
            }
            else if(hxtal_freq_type == HXTAL_8M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_8M;
                SCU->CLK_CSR = temp;
            }
            else if(hxtal_freq_type == HXTAL_12M)
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_12M;
                SCU->CLK_CSR = temp;
            }
            else
            {
                temp &= (~0x1FF8);
                temp |= HXTAL_RDY_DIV_12M;
                SCU->CLK_CSR = temp;
            }

            SCU->CLK_EN_CR &= ~HXTAL_EN_ENUM;
            SCU->CLK_EN_CR |= HXTAL_EN_ENUM;
            SCU->CLK_EN_CR &= ~LXTAL_EN_ENUM;
            SCU->CLK_EN_CR |= LXTAL_EN_ENUM;
        }

        counter = 0;

        while(!(SCU->CLK_CSR & HXTAL_OSCILATED))
        {
            if(counter++ > 0xfffff)
                break;
        }

        counter = 0;

        while(!(SCU->CLK_CSR & LXTAL_OSCILATED))
        {
            if(counter++ > 0xfffff)
                break;
        }

        break;

        default:
            break;
    }

    if((clocktype == CLK_PLL_USE_HRC_LRC) || (clocktype == CLK_PLL_USE_HRC_LXTAL))
    {
        SCU->PLL_CR &= (~PLL_MASK_REF_DIVIDER_DIV_M);
        SCU->PLL_CR |= PLL_EN_ENUM;
        SCU->PLL_CR &= ~0x0F;
        SCU->PLL_CR |= pll_freq;
        SCU->PLL_CR |= HRC_16M;
        SCU->PLL_CR |= PLL_LOCK_RESET;
        SCU->CLK_EN_CR &= ~PLL_EN_ENUM;

        for(counter = 0; counter < 1000; counter++) 
			;
        SCU->CLK_EN_CR |= PLL_EN_ENUM;

       for(counter = 0; counter < 1000; counter++) 
			;
        counter = 0;

        while(!(SCU->CLK_CSR & PLL_OSCILATED))
        {
            if(counter > 0xffffff)
            {
                counter = 0;
                break;
            }
            else
                counter++;
        }
    }
    else if((clocktype == CLK_PLL_USE_HXTAL_LRC) || ( clocktype == CLK_PLL_USE_HXTAL_LXTAL))
    {
        SCU->PLL_CR &= (~PLL_MASK_REF_DIVIDER_DIV_M);
        SCU->PLL_CR |= PLL_REF;
        SCU->PLL_CR &= ~0x0F;
        SCU->PLL_CR |= pll_freq;
        SCU->PLL_CR |= hxtal_freq_type;
        SCU->PLL_CR |= PLL_LOCK_RESET;
        SCU->CLK_EN_CR &= ~PLL_EN_ENUM;

        for(counter = 0; counter < 1000; counter++) 
			;

        SCU->CLK_EN_CR |= PLL_EN_ENUM;

        for(counter = 0; counter < 1000; counter++) 
			;

        counter = 0;

        while(!(SCU->CLK_CSR & PLL_OSCILATED))
        {
            if(counter > 0xfffff)
                break;
            else
                counter++;
        }
    }

    __NOP();
    SCU->CLK_SEL_CR = clocktype;
    SystemCoreClock = SCU_Get_frequecny();

}

/**
  * @brief: SCU_MCO
  * @note:  set clock out
  * @param:  LOCK_OUT_SELECT_TYPE mcosel
  * refernce from
  * typedef enum
  *{
  *	CLK_LRC =0,
  *	CLK_LXTAL =(1>>2),
  *	CLK_HXTAL =(2>>2),
  *	CLK_HRC =(3>>2),
  *	CLK_PLL =(4>>2),
  *	CLK_SYS =(5>>2),
  *	CLK_BUS =(6>>2),
  *	CLK_ADC=(7>>2),
  *} CLOCK_OUT_SELECT_TYPE;
  *  select one as parameter
  *  CLOCK_OUT_DIV_TYPE mcopre
  *  Reference from
  *typedef enum
  *{
  *  SCU_OUT_Div1    = 0,
  *  SCU_OUT_Div2    = 1,
  *  SCU_OUT_Div4    = 2,
  *  SCU_OUT_Div8    = 3,
  *} CLOCK_OUT_DIV_TYPE;
  *select one as parameter
  *CLOCK_OUT_ENABLE_ENUM out_enable
  *Reference from
  *typedef enum
  *{
  *	CLOCK_OUT_DISABLE=0,
  *  CLOCK_OUT_ENABLE=0x20,
  *}CLOCK_OUT_ENABLE_ENUM;
  *select one as parameter
  *@retval:None
  */
  
 
void SCU_MCO(CLOCK_OUT_SELECT_Enum mcosel, CLOCK_OUT_DIV_Enum mcopre, CLOCK_OUT_ENABLE_Enum out_enable)
{
    uint8_t out_register = 0;
    SCU_Unlock();
    SCU_PeriphClockCmd(SCU_GPIOB_CLKEN, ENABLE);

    if(!out_enable)
    {
        out_register = out_enable;
    }
    else
    {
        out_register = mcopre;
        out_register |= mcosel;
        out_register |= out_enable;
        GPIO_Init(GPIOB, GPIO_PIN5, GPIO_MODE_AF);
        GPIO_PinRemapConfig(GPIOB, GPIO_PIN5, GPIO_ALT_PLUOUT_EXTCLK);
    }

    SCU->COUT_CR = out_register;
}

/**
  *@brief: SCU_PeriphClockCmd
  *@note:  Peripheral clock configuration.
  *@param:  Periph
  *Reference from
  *SCU_ALL_CLKEN
  *SCU_ADC_CORE_CLKEN
  *SCU_SYS_CLKEN
  *SCU_GPIOD_CLKEN
  *SCU_GPIOC_CLKEN
  *SCU_GPIOB_CLKEN
  *SCU_GPIOA_CLKEN
  *SCU_PGA_CLKEN
  *SCU_DAC3_CLKEN
  *SCU_DAC2_CLKEN
  *SCU_DAC1_CLKEN
  *SCU_DAC0_CLKEN
  *SCU_ADC_CLKEN
  *SCU_COMP_CLKEN
  *SCU_IIC1_CLKEN
  *SCU_IIC0_CLKEN
  *SCU_CORDIC_CLKEN
  *SCU_PWM_CLKEN
  *SCU_SPI1_CLKEN
  *SCU_SPI0_CLKEN
  *SCU_USART_CLKEN
  *SCU_UART2_CLKEN
  *SCU_UART1_CLKEN
  *SCU_UART0_CLKEN
  *SCU_PLU_CLKEN
  *SCU_CRC_CLKEN
  *SCU_WWDT_CLKEN
  *SCU_LPTIMER_CLKEN
  *SCU_TIM1_CLKEN
  *SCU_TIM0_CLKEN
  *@param:  Periph
  *Reference from enum {DISABLE = 0, ENABLE = !DISABLE}
  *@retval: None
  */
void SCU_PeriphClockCmd(uint32_t Periph, FunctionalState NewState)
{
    SCU_Unlock();

    if (NewState != DISABLE)
    {
        SCU->PCLK_CR |= Periph;
    }
    else
    {
        SCU->PCLK_CR &= ~Periph;
    }
}

/**
  *@brief: SCU_Set_Wake
  *@note:  set wake mode.
  *@param:  input wake up mode ,reference from
  *typedef enum
  *{
  *	EXTI0_WAKEUP     =1,
  *	EXTI1_WAKEUP     =2,
  *    EXTI2_WAKEUP     =4,
  *	EXTI3_WAKEUP     =8,
  *	PVD_WAKEUP       =0x10,
  *	LPTIMER_WAKEUP   =0x20
  *}WAKEUP_MODE_TYPE;
  *select one or multiple.
  *@retval: None
  */
void SCU_Set_Wake(uint8_t mode)
{
    SCU->STBY_WKUP_CR = mode;
}

/**
  *@brief: SCU_Get_IF
  *@note:  get clock int if register value.
  *@param: None
  *@retval: SCU->CLKSC_IMR
  */
uint8_t SCU_Get_IF(void)
{
    return SCU->CLKSC_IMR;
}

/**
  *@brief: SCU_Disable_IT
  *@note:  set clock change interrupt disable register.
  *@param: uint8_t id ,reference from
  *typedef enum
  *{
  *  LXTAL_CHANGE_ID=1,
  *	 HXTAL_CHANGE_ID=2,
  *	 PLL_LOCK_CHANGE_ID=4
  *}CLOCK_INT_DISABLE;
  * select one or multiple
  *@retval: None
  */

void SCU_Disable_IT(uint8_t ID)
{
    SCU->CLKSC_IDR = ID;
}

/**
  *@brief: SCU_Enable_IT
  *@note:  set clock change interrupter enable register.
  *@param:uint8_t ie ,reference from
  *typedef enum
  *{
  *   LXTAL_CHANGE_IE=1,
  *	  HXTAL_CHANGE_IE=2,
  *	  PLL_LOCK_CHANGE_IE=4
  *}CLOCK_INT_ENABLE;
  * select one or multiple
  *@retval: None
  */

void SCU_Enable_IT(uint8_t IE)
{
    SCU->CLKSC_IER = IE;
    NVIC_EnableIRQ(EXTI5_15_IRQn);
}

/**
  *@brief: SCU_ResetPeriphClock
  *@note:  Reset the peripheral clock.
  *@param: Periph
  *Reference from
  *SCU_ALL_RST
  *SCU_GPIOD_RST
  *SCU_GPIOC_RST
  *SCU_GPIOB_RST
  *SCU_GPIOA_RST
  *SCU_DAC3_RST
  *SCU_DAC2_RST
  *SCU_DAC1_RST
  *SCU_DAC0_RST
  *SCU_ADC_RST
  *SCU_COMP_RST
  *SCU_IIC1_RST
  *SCU_IIC0_RST
  *SCU_CORDIC_RST
  *SCU_PWM_RST
  *SCU_SPI1_RST
  *SCU_SPI0_RST
  *SCU_USART_RST
  *SCU_UART2_RST
  *SCU_UART1_RST
  *SCU_UART0_RST
  *SCU_PLU_RST
  *SCU_CRC_RST
  *SCU_WWDT_RST
  *SCU_LPTIMER_RST
  *SCU_TIM1_RST
  *SCU_TIM0_RST
  *@retval: None
  */
void SCU_ResetPeriphClock(uint32_t Periph)
{
    SCU_Unlock();
    SCU->PRST_CR |= Periph;
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    SCU->PRST_CR &= ~Periph;
    SCU_Lock();
}

/**
  *@brief: SCU_SetBor
  *@note:  set BOR rising edge and BOR falling edge value.
  *@param: BOR_RISING_VALUE_TYPE rising_value
  *typedef enum
  *{
  *  BOR_RISING_2V   = 0,
  *  BOR_RISING_2V25 =0x10,
  *  BOR_RISING_2V5  = 0x20,
  *  BOR_RISING_2V75 = 0x30,
  *  BOR_RISING_3V  = 0x40,
  *  BOR_RISING_3V3  = 0x50,
  *  BOR_RISING_3V6 = 0x60,
  *  BOR_RISING_4V = 0x70,
  *}BOR_RISING_VALUE_TYPE;
  *   Select one as rising_value parameter
  *BOR_FALLING_VALUE_TYPE falling_value
  *  typedef enum
  *{
  *  BOR_FALLING_2V   = 0,
  *  BOR_FALLING_2V25 =2,
  *  BOR_FALLING_2V5  = 4,
  *  BOR_FALLING_2V75 = 6,
  *  BOR_FALLING_3V  = 8,
  *  BOR_FALLING_3V3  = 10,
  *  BOR_FALLING_3V6 = 12,
  *  BOR_FALLING_4V = 14,
  *}BOR_FALLING_VALUE_TYPE;
  *select one as falling_value parameter
  *FunctionalState NewState
  *NewState ENABLE(BOR)/DISABLE(BOR)
  *@retval: None
  */
void SCU_SetBor(BOR_RISING_VALUE_Enum rising_value, BOR_FALLING_VALUE_Enum falling_value, FunctionalState NewState)
{

    uint16_t temp = 0;

    if (NewState != DISABLE)
    {
        temp = NewState + rising_value + falling_value;
        SCU->PSM_CSR &= (~BOR_MODUEL_MASK);
        SCU->PSM_CSR |= temp;
    }
    else
    {
        SCU->PSM_CSR &= (~BOR_ENABLE_MASK);
    }

}

/**
  *@brief: SCU_SetPVD
  *@note:  Set PVD VALUE.
  *@param: PVD_VALUE_TYPE PVDLevel
  * typedef enum
  *{
  *    PVD_2V=0,
  *    PVD_2V25=(1>>8),
  *    PVD_2V5= (2>>8),
  *    PVD_2V75=(3>>8),
  *	  PVD_3V=  (4>>8),
  *	  PVD_3V3= (5>>8),
  *	  PVD_3V6= (6>>8),
  *	  PVD_4V=  (7>>8),
  *}PVD_VALUE_TYPE	;
  *select one as parameter
  * PVD_ENABLE_TYPE NewState
  * typedef enum
  * {
  *	 PVD_DISABLE=0  ,
  *   PVD_ENABLE =0x80,
  * }PVD_ENABLE_TYPE;
  *select one as parameter
  *	NewState PVD_ENABLE(BOR)/PVD_DISABLE(BOR)
  *@retval: None
  */
void SCU_SetPVD(PVD_VALUE_Enum PVDLevel, PVD_ENABLE_Enum NewState)
{
    uint16_t temp = 0;
    uint8_t i;

    if (NewState != PVD_DISABLE)
    {
        temp = PVDLevel + NewState;
        SCU->PSM_CSR &= (~PVD_MODUEL_MASK);
        SCU->PSM_CSR |= temp;
    }
    else
    {
        SCU->PSM_CSR &= (~PVD_ENABLE_MASK);
    }

    for(i = 0; i < 200; i++)
		;
}

/**
  *@brief: SCU_Sleep_Mode
  *@note:  set chip sleep mod ,inculding sleep ,stop,standby.
  *@param:LOW_POWER_MODE_Enum mode reference from
  *  typedef enum
  *{
  *	SLEEP_MODE=0,
  *  STOP_MODE=1,
  *	STANDBY_MODE=2
  *}LOW_POWER_MODE_TYPE;
  *Select one parameter as mode
  *SLEEP_MODE_TYPE stop_method  in stop mode ,reference from
  *typedef enum
  *{
  *	ALL_OFF        =0x00,
  *	PLL_ON        =0x01,
  *  HRC_ON        =0x02,
  *  HTXAL_ON      =0x04,
  *  LXTAL_ON      =0x08,
  *  LXTAL_STB_ON  =0x10,
  *  MLDO_ON       =0x20,
  *}SLEEP_MODE_Enum;
  *select  one or more combinations
  *SCU_WAKMode_Enum wake_methord
  *0 is event wake ,others is intterupt  wake
  *@retval: None
  */
void SCU_Sleep_Mode(LOW_POWER_MODE_Enum mode, STOP_MODE_Enum stop_method, SCU_WAKMode_Enum wake_methord)
{
    SCU_Unlock();

    switch(mode)
    {
        case SLEEP_MODE:
			      SCU->LPM_PWR_CR =0;
            SCB->SCR = 0;
            break;

        case  STOP_MODE:
			
				   
            SCU->LPM_CR = 1;
            SCU->LPM_PWR_CR = stop_method;
            SCB->SCR = 0;
            break;

        case STANDBY_MODE:
            
            SCU->LPM_CR = 1;
            SCB->SCR = 4;
            break;
    }

    if(wake_methord)
    {
        __WFI();
        __NOP();
    }
    else
    {
        __WFE();
        __WFE();
        __NOP();
    }
}

/**
  *@brief: SCU_Get_Version
  *@param:None
  *@retval: Version
  */
uint16_t SCU_Get_Version(void)
{
    return SCU->VERSION ;
}

/**
  *@brief:uint16_t SCU_Get_Wake_IF(void) get wakeup flag
  *@param:None
  *@retval: wake up flag
  */
uint16_t SCU_Get_Wake_IF(void)
{
    uint16_t temp;
    temp = SCU->STBY_WKUP_SR;
    return temp;
}

/**
  *@brief:uint16_t SCU_Get_Reset_Source(void) get reset source
  *@param:None
  *@retval:wreset source Reference from
  *typedef enum
  *{
  *	POR_RESET      =1,
  *	BOR_RESET      =2,
  * KEY_RESET      =4,
  *	IWDG_RESET     =8,
  *	WWDG_RESET     =0x10,
  *	SOFT_RESET     =0x20,
  *	LOCKUP_RESET   =0x40,
  *	CPU_RESET      =0x80
  *}SCU_RESET_TYPE;
  */
uint16_t SCU_Get_Reset_Source(void)
{
    uint16_t temp;
    temp = SCU->RST_SR; /*after read it,SCU->RST_SR will be clear zero*/
    return temp;
}

/**
  *@brief:void SCU_Soft_Reset(void) SCU software reset
  *@param:None
  *@retval:None
  */
void SCU_Soft_Reset(void)
{
    SCU->RST_CR = SCU_SOFT_RESET_KEY;

    while(1);
}

/**
  *@brief:void SCU_Set_Boot(BOOT_SELECT_TYPE boot_type) boot from flash or sram
  *@param:BOOT_SELECT_TYPE boot_type
  *typedef enum
  *{
  * BOOT_FROM_FLASH =0,
  * BOOT_FROM_SRAM  =1
  *}BOOT_SELECT_TYPE;
  *select one as parameter
  *@retval:None
  */
void SCU_Set_Boot(BOOT_SELECT_Enum boot_type)
{
    SCU->RAMADDR = boot_type;
}

/**
  *@brief:void SCU_PVD_Enable_IT(uint8_t IE) set PVD interrupt enable
  *@param:uint8_t ie ,reference from
  *typedef enum
  *{
  *  PVD_F_IE =1,
  *	PVD_R_IE =2
  *}PVD_IE_TYPE;//interrupt enable
  * select one or more
  *@retval:None
  */
void SCU_PVD_Enable_IT(PVD_IE_Enum IE)
{
    SCU->PVD_IER = IE;
}

/**
  *@brief:void SCU_PVD_Disable_IT(uint8_t ID) set PVD interrupt disable
  *@param:uint8_t ie ,reference from
  *typedef enum
  *{
  *  PVD_F_ID =1,
  *	PVD_R_ID =2
  *}PVD_IE_TYPE;//interrupt disable
  * select one or more
  *@retval:None
  */
void SCU_PVD_Disable_IT(PVD_IE_Enum ID)
{
    SCU->PVD_IDR = ID;
}

/**
  *@brief:FunctionalState SCU_Get_PVD_IF(void)
  *@param:None
  *@retval:None
  */
FunctionalState SCU_Get_PVD_IF(void)
{
    uint16_t temp;
    temp = SCU->PSM_CSR & 0x800;

    if(!temp)
        return DISABLE;
    else
        return ENABLE;
}

/**
  *@brief:void SCU_EANBLE_HRC(FunctionalState state)  ENABLE OR DISABLE HRC
  *@param:FunctionalState state  DISABLE or ENABLE
  *@retval:None
  */
void SCU_EANBLE_HRC(FunctionalState state )
{
    if(state != DISABLE)
    {
        SCU->CLK_EN_CR |= HRC_EN_ENUM;
    }
    else
    {
        SCU->CLK_EN_CR &= (~HRC_EN_ENUM);
    }
}

/**
  *@brief:void SCU_EANBLE_LXTAL(FunctionalState state) ENABLE OR DISABLE LXTAL
  *@param:FunctionalState state   DISABLE or ENABLE
  *@retval:None
  */
void SCU_EANBLE_LXTAL(FunctionalState state )
{
    if(state != DISABLE)
    {
        SCU->CLK_EN_CR |= LXTAL_EN_ENUM;
    }
    else
    {
        SCU->CLK_EN_CR &= (~LXTAL_EN_ENUM);
    }
}

/**
  *@brief:void SCU_EANBLE_HXTAL(FunctionalState state) ENABLE OR DISABLE HXTAL
  *@param:FunctionalState state    DISABLE or ENABLE
  *@retval:None
  */
void SCU_EANBLE_HXTAL(FunctionalState state )
{
    if(state != DISABLE)
    {
        SCU->CLK_EN_CR |= HXTAL_EN_ENUM;
    }
    else
    {
        SCU->CLK_EN_CR &= (~HXTAL_EN_ENUM);
    }
}

/**
  *@brief:void SCU_EANBLE_PLL(FunctionalState state) ENABLE OR DISABLE PLL
  *@param:FunctionalState state   DISABLE or ENABLE
  *@retval:None
  */
void SCU_EANBLE_PLL(FunctionalState state )
{
    if(state != DISABLE)
    {
        SCU->CLK_EN_CR |= PLL_EN_ENUM;
    }
    else
    {
        SCU->CLK_EN_CR &= (~PLL_EN_ENUM);
    }
}

