/**
  ******************************************************************************
  * @file           : ms60f3xxx_spi.c
  * @author         : sinomcu.com-SA
  * @brief          : SPI module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx.h"
#include "ms60f3xxx_spi.h"
#include "ms60f3xxx_scu.h"

/* 64 bytes data buffer for SPI transfer and receive. */
uint8_t g_SPI_receive_counter = 64;
/* SPI Communicate buffer size. */
uint8_t g_SPI_Sbuffer[COMM_BUFFER_SIZE];
uint8_t g_SPI_Rbuffer[COMM_BUFFER_SIZE];

/**
 @brief: Spi interrupt enable configuration.
 @note: This function is only used to enable configuration of the SPI interrupt function. It is not available for disabled configuration.
 @param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
 @param: ul_sources: SPI configuration parameters control address.
 @retval: None
**/
void SPI_Enable_IT(SPI_Type *SPIx, uint32_t ul_sources)
{
    SPIx->SPI_IER = ul_sources;
}

/**
 @brief: Spi Reset.
 @note: None.
 @param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
 @retval: None
**/
void SPI_Reset(SPI_Type *SPIx)
{
    SPIx->SPI_CR |=  SPI_CR_SPIDIS | SPI_CR_SWRST;
}

/**
 @brief: Spi Enable.
 @param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
 @retval: None
**/
void SPI_Enable(SPI_Type *SPIx)
{
    SPIx->SPI_CR |= SPI_CR_SPIEN;
}

/**
 @brief: Spi Disable.
 @param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
 @retval: None
**/
void SPI_Disable(SPI_Type *SPIx)
{
    SPIx->SPI_CR |= SPI_CR_SPIDIS;
}

/**
 @brief: Spi interrupt disable configuration.
 @note: This function is only used to disabled configuration of the SPI interrupt function. It is not available for enabled configuration.
 @param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
 @param: ul_sources: SPI configuration parameters control address.
 @retval: None
**/
void SPI_Disable_IT(SPI_Type *SPIx, uint32_t ul_sources)
{
    SPIx->SPI_IDR = ul_sources;
}

/**
@brief: SPI master-slave mode initial configuration.
@note: Only one mode can be configured per call to this function, not multiple modes at the same time.
@param: * SPIx: SPI base address, which can be configured as SPI0 or SPI1.
@param: * spi_init_def: SPI configuration parameters include SPI_Mode_Enum,SPI_CPOL_NCPHA_Mode_Enum,SPI_Transmit_Mode_Enum,BaudRate.
							This parameter can be one of the SPI_Mode_Enum values:
				@arg Mode_Master: Master Mode
				@arg Mode_Slave: Slave Mode
							This parameter can be one of the SPI_CPOL_NCPHA_Mode_Enum values:
				@arg CPOL_NCPHA_Mode_0,CPOL_NCPHA_Mode_1,CPOL_NCPHA_Mode_2,CPOL_NCPHA_Mode_3
							This parameter can be one of the SPI_Transmit_Mode_Enum values:
				@arg DMA_Disable,DMA_Enable
							This parameter of BaudRate value should according to System clock to set:
				@arg BaudRate = SystemCoreClock/SCBR,SCBR Value range [1,255]
@retval: None
**/
void SPI_Init(SPI_Type *SPIx, SPI_Init_T *SPI_InitConfig)
{
    SPI_Reset(SPIx);

    if(SPI_InitConfig->Mode == SPI_Mode_Master)
    {
        SPIx->SPI_MR |= SPI_MR_MODFDIS | SPI_InitConfig->Mode;
        SPIx->SPI_CSR = SPI_InitConfig->CPOL_NCPHA_Mode | SPI_InitConfig->Data_Bit;
        SPIx->SPI_CSR |= SPI_CSR_SCBR(SCU_Get_frequecny() / SPI_InitConfig->BaudRate);
        SPI_Enable(SPIx);
    }
    else
    {
        SPIx->SPI_MR |= SPI_InitConfig->Mode;
        SPIx->SPI_CSR = SPI_InitConfig->CPOL_NCPHA_Mode | SPI_InitConfig->Data_Bit;
        SPI_Enable(SPIx);
    }

    if(SPI_InitConfig->Transmit_Mode == SPI_DMA_Enable)
    {
        SPIx->SPI_RPR = (uint32_t)g_SPI_Rbuffer;
        SPIx->SPI_RCR = COMM_BUFFER_SIZE;
        SPIx->SPI_PTCR = SPI_PTCR_RXTEN;
        SPI_Enable_IT(SPIx, SPI_IER_RXBUFF);
    }
    else
    {
        /* Configure SPI interrupts for slave only. */
        SPI_Enable_IT(SPIx, SPI_IER_RDRF);

        if(SPIx == SPI0)
        {
            NVIC_EnableIRQ(SPI0_IRQn);
        }
        else
        {
            NVIC_EnableIRQ(SPI1_IRQn);
        }
    }


}

/**
@brief: SPI data transmission, Buff mode
@note: Buff mode, for data sending only.
@param: *SPIx: SPI base address, which can be configured as SPI0 or SPI1.
@param: *addr: Address of the data sending source.
@param: len:Data transmission length.
@retval: None
**/
void SPI_BuffTransmission(SPI_Type *SPIx, uint8_t *addr, uint32_t len)
{
    int i = 0;

    for(i = 0; i < len; i++)
    {
        SPIx->SPI_TDR = addr[i];

        while(!(SPIx->SPI_SR & SPI_SR_TXEMPTY));
    }
}

/**
@brief: SPI data receive.
@note: return data of receive.
@param: *SPIx: SPI base address, which can be configured as SPI0 or SPI1.
@retval: None
**/
uint8_t SPI_reveice(SPI_Type *SPIx)
{
    return SPIx->SPI_RDR;
}

/**
@brief: SPI data transmission, DMA mode.
@note: DMA mode, for data sending only.
@param: *SPIx: SPI base address, which can be configured as SPI0 or SPI1.
@param: *addr: Address of the data sending source.
@param: len:Data transmission length.
@retval: None
**/
void SPI_DmaTransmission(SPI_Type *SPIx, uint8_t *addr, uint32_t len)
{
    while(SPIx->SPI_TCR != 0) {}

    SPIx->SPI_PTCR = SPI_PTCR_TXTDIS;
    SPIx->SPI_TPR = (uint32_t)addr;
    SPIx->SPI_TCR = len;
    SPIx->SPI_PTCR = SPI_PTCR_TXTEN;
}

/**
@brief: SPI transmit and receive, DMA mode.
@note: DMA mode, for data sending and receiving.
@param: *SPIx: SPI base address, which can be configured as SPI0 or SPI1.
@param: *data:Address of the data sending source.
@param: len:Data transmission length.
@param: *data:Address for storing received data.
@param: len:Data receiving length.
@retval: None
**/
void SPI_DmaTransfer(SPI_Type *SPIx, uint8_t *addr, uint32_t len, uint8_t *addr_rev, uint32_t relen)
{
    while(SPIx->SPI_TCR != 0) {}

    SPIx->SPI_PTCR = SPI_PTCR_TXTDIS;
    SPIx->SPI_TPR = (uint32_t)addr;
    SPIx->SPI_TCR = len;
    SPIx->SPI_PTCR = SPI_PTCR_TXTEN;
    SPIx->SPI_PTCR = SPI_PTCR_RXTDIS;
    SPIx->SPI_RPR = (uint32_t)addr_rev;
    SPIx->SPI_RCR = relen;
    SPIx->SPI_PTCR = SPI_PTCR_RXTEN;
}
