/**
  ******************************************************************************
  * @file           : ms60f3xxx_tim.c
  * @author         : sinomcu.com-SA
  * @brief          : TIM module driver.
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_tim.h"

/**
  * @brief  Configure TIM for timer, capture.
  * @param  TIMx: TIMx base address, which can be configured as TIM0 or TIM1.
  * @param  TIM_CMR_CaptureConfig: pointer to a TIM_CMR_Capture_T structure that contains
  *  	                            the configuration information for the specified TIMx peripheral.
  * @retval SUCCESS or ERROR.
  */
ErrorStatus TIM_CaptureInit(TIM_Type *TIMx, TIM_CMR_Capture_T *TIM_CMR_CaptureConfig)
{
    ErrorStatus status = SUCCESS;
    uint16_t ui_channel = TIM_CMR_CaptureConfig->CHANNEL;

    if( ( TIM_CMR_CaptureConfig->WAVE & TIM_CMR_WAVE_MODE ) == 0 )
    {
        TIMx->CHANNEL[ui_channel].CCR = TIM_CCR_CLKDIS;/*  Disable TIM clock. */
        TIMx->CHANNEL[ui_channel].IDR = 0xFFFFFFFF;   /*  Disable interrupts. */
        TIMx->CHANNEL[ui_channel].SR;                 /*  Clear status register. */

        TIMx->CHANNEL[ui_channel].CMR  =  /*  Set mode. */
            TIM_CMR_CaptureConfig->WAVE
            | TIM_CMR_CaptureConfig->TIMCLKS
            | TIM_CMR_CaptureConfig->CLKI
            | TIM_CMR_CaptureConfig->BURST
            | TIM_CMR_CaptureConfig->LDBSTOP
            | TIM_CMR_CaptureConfig->LDBDIS
            | TIM_CMR_CaptureConfig->ETRGEDG
            | TIM_CMR_CaptureConfig->ABETRG
            | TIM_CMR_CaptureConfig->CPCTRG
            | TIM_CMR_CaptureConfig->LDRA
            | TIM_CMR_CaptureConfig->LDRB
            | TIM_CMR_CaptureConfig->SBSMPL;
    }
    else
        status = ERROR;

    return status;
}

/**
  * @brief  Configure TIM for timer, waveform generation.
  * @param  TIMx:               TIMx base address, which can be configured as TIM0 or TIM1.
  * @param  TIM_CMR_WaveConfig: pointer to a TIM_CMR_Wave_T structure that contains
  *			                       the configuration information for the specified TIMx peripheral.
  * @retval SUCCESS or ERROR.
  */
ErrorStatus TIM_WaveInit(TIM_Type *TIMx, TIM_CMR_Wave_T *TIM_CMR_WaveConfig)
{
    ErrorStatus status = SUCCESS;

    uint16_t ui_channel = TIM_CMR_WaveConfig->CHANNEL;

    if(TIM_CMR_WaveConfig->WAVE & TIM_CMR_WAVE_MODE )
    {
        TIMx->CHANNEL[ui_channel].CCR = TIM_CCR_CLKDIS;/*  Disable TIM clock. */
        TIMx->CHANNEL[ui_channel].IDR = 0xFFFFFFFF;    /*  Disable interrupts. */
        TIMx->CHANNEL[ui_channel].SR;                  /*  Clear status register. */

        TIMx->CHANNEL[ui_channel].CMR =                /*  Set mode. */
            TIM_CMR_WaveConfig->WAVE
            | TIM_CMR_WaveConfig->TIMCLKS
            | TIM_CMR_WaveConfig->CLKI
            | TIM_CMR_WaveConfig->BURST
            | TIM_CMR_WaveConfig->CPCSTOP
            | TIM_CMR_WaveConfig->CPCDIS
            | TIM_CMR_WaveConfig->EEVTEDG
            | TIM_CMR_WaveConfig->EEVT
            | TIM_CMR_WaveConfig->ENETRG
            | TIM_CMR_WaveConfig->WAVSEL
            | TIM_CMR_WaveConfig->ACPA
            | TIM_CMR_WaveConfig->ACPC
            | TIM_CMR_WaveConfig->AEEVT
            | TIM_CMR_WaveConfig->ASWTRG
            | TIM_CMR_WaveConfig->BCPB
            | TIM_CMR_WaveConfig->BCPC
            | TIM_CMR_WaveConfig->BEEVT
            | TIM_CMR_WaveConfig->BSWTRG ;
    }
    else
        status = ERROR;

    return status;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM0_0_SetTriggerInput(TIM_TRGIN_TIM0_0_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM0_0_TIOB = Trginput;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM0_1_SetTriggerInput(TIM_TRGIN_TIM0_1_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM0_1_TIOB = Trginput;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM0_2_SetTriggerInput(TIM_TRGIN_TIM0_2_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM0_2_TIOB = Trginput;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM1_0_SetTriggerInput(TIM_TRGIN_TIM1_0_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM1_0_TIOB = Trginput;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM1_1_SetTriggerInput(TIM_TRGIN_TIM1_1_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM1_1_TIOB = Trginput;
}

/**
  * @brief  TIOB is chosen as the external event signal.
  * @param  Trginput: choose external trigger source.
  * @retval None
  */
void TIM1_2_SetTriggerInput(TIM_TRGIN_TIM1_2_TIOB_Enum Trginput)
{
	  TIM_TRGIN->TIOB_b.TIM1_2_TIOB = Trginput;
}

/**
  * @brief  Asserts a SYNC signal to generate a software trigger on all channels.
  * @param  TIMx: TIMx Module hardware register base address pointer.
  * @retval None
  */
void TIM_Sync_Trigger(TIM_Type *TIMx)
{
    TIMx->BCR = TIM_BCR_SYNC;
}

/**
  * @brief  Configure the TIM Block mode.
  * @param  TIMx: Module hardware register base address pointer
  * @param  TIM_BMR_QdecConfig: pointer to a TIM_BMR_Qdec_T structure that contains
  * 			                       the configuration information for "TIM Block Mode Register".
  * @retval SUCCESS or ERROR.
  */
ErrorStatus TIM_SetBlockMode(TIM_Type *TIMx, TIM_BMR_T *TIM_BMR_QdecConfig)
{
    ErrorStatus status = SUCCESS;

    if( (TIM_BMR_QdecConfig->MAXFILT > 0 && TIM_BMR_QdecConfig->MAXFILT <= 63)
            || ( TIM_BMR_QdecConfig->MAXCMP <= 15))
    {
        TIMx->BMR =   TIM_BMR_QdecConfig->TIM0XC0S
                      | TIM_BMR_QdecConfig->TIM1XC1S
                      | TIM_BMR_QdecConfig->TIM2XC2S
                      | TIM_BMR_QdecConfig->QDEN
                      | TIM_BMR_QdecConfig->POSEN
                      | TIM_BMR_QdecConfig->SPEEDEN
                      | TIM_BMR_QdecConfig->QDTRANS
                      | TIM_BMR_QdecConfig->EDGPHA
                      | TIM_BMR_QdecConfig->INVA
                      | TIM_BMR_QdecConfig->INVB
                      | TIM_BMR_QdecConfig->INVIDX
                      | TIM_BMR_QdecConfig->SWAP
                      | TIM_BMR_QdecConfig->IDXPHB
                      | TIM_BMR_QdecConfig->AUTOC
                      | TIM_BMR_QdecConfig->MAXFILT
                      | TIM_BMR_QdecConfig->MAXCMP;
    }
    else
        status = ERROR;

    return status;
}

/**
  * @brief  Start the TIM clock on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @retval None.
  */
void TIM_Start(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIMx->CHANNEL[TIM_CHANNEL].CCR = TIM_CCR_CLKEN | TIM_CCR_SWTRG;
}

/**
  * @brief  Enable TIM clock on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @retval None.
  */
void TIM_ClockEnable(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIMx->CHANNEL[TIM_CHANNEL].CCR = TIM_CCR_CLKEN;
}

/**
  * @brief  Enable TIM swtrg on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @retval None.
  */
void TIM_Swtrg(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIMx->CHANNEL[TIM_CHANNEL].CCR = TIM_CCR_SWTRG;
}

/**
  * @brief  Stop the TIM clock on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @retval None.
  */
void TIM_Stop(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIMx->CHANNEL[TIM_CHANNEL].CCR = TIM_CCR_CLKDIS;
}

/**
  * @brief  Read the counter value on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to read
  * @retval The counter value.
  */
uint32_t TIM_Read_CV(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    return TIMx->CHANNEL[TIM_CHANNEL].CV;
}

/**
  * @brief  Read TIM Register A (RA) on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to read
  * @retval The TIM Register A (RA) value.
  */
uint32_t TIM_Read_RA(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    return TIMx->CHANNEL[TIM_CHANNEL].RA;
}

/**
  * @brief  Read TIM Register B (RB) on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to read
  * @retval The TIM Register B (RB) value.
  */
uint32_t TIM_Read_RB(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    return TIMx->CHANNEL[TIM_CHANNEL].RB;
}

/**
  * @brief  Read TIM Register C (RC) on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to read
  * @retval The Register C (RC) value.
  */
uint32_t TIM_Read_RC(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    return TIMx->CHANNEL[TIM_CHANNEL].RC;
}

/**
  * @brief  Write to TIM Register A (RA) on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to write
  * @param  ul_value:    Value to write
  * @retval None.
  */
void TIM_Write_RA(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL, uint32_t ul_value)
{
    TIMx->CHANNEL[TIM_CHANNEL].RA = ul_value;
}

/**
  * @brief  Write to TIM Register B (RB) on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to write
  * @param  ul_value:    Value to write
  * @retval None.
  */
void TIM_Write_RB(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL, uint32_t ul_value)
{
    TIMx->CHANNEL[TIM_CHANNEL].RB = ul_value;
}

/**
  * @brief  Write to TIM Register C (RC) on the selected channel.
  * @param  TIMx:        Module hardware register base address pointer.
  * @param  TIM_CHANNEL: Channel to write.
  * @param  ul_value:    Value to write.
  * @retval None.
  */
void TIM_Write_RC(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL, uint32_t ul_value)
{
    TIMx->CHANNEL[TIM_CHANNEL].RC = ul_value;
}

/**
  * @brief  Enable the TIM interrupts on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @param  ul_sources:  ul_sources a bitmask of interrupt sources
  *				 Where the input parameter ul_sources can be one or more of the following:
  * 											TIM_IER_COVFS		Enables the Counter Overflow Interrupt
  *											TIM_IER_LOVRS  Enables the Load Overrun Interrupt
  *											TIM_IER_CPAS 	Enables the RA Compare Interrupt
  *											TIM_IER_CPBS		Enables the RB Compare Interrupt
  *											TIM_IER_CPCS		Enables the RC Compare Interrupt
  *											TIM_IER_LDRAS	Enables the RA Load Interrupt
  *											TIM_IER_LDRBS	Enables the RB Load Interrupt
  *											TIM_IER_ETRGS	Enables the External Trigger Interrupt
  * @retval: None.
  */
void TIM_Enable_IT(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL, uint32_t ul_sources)
{
    TIM_Channel *tim_channel;
    tim_channel         = TIMx->CHANNEL + TIM_CHANNEL;
    tim_channel->IER    = ul_sources;
}

/**
  * @brief Disable the TIM interrupts on the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to configure
  * @param  ul_sources: ul_sources a bitmask of interrupt sources
				Where the input parameter ul_sources can be one or more of the following:
				TIM_IER_COVFS	Disables the Counter Overflow Interrupt
				TIM_IER_LOVRS  Disables the Load Overrun Interrupt
				TIM_IER_CPAS 	Disables the RA Compare Interrupt
				TIM_IER_CPBS		Disables the RB Compare Interrupt
				TIM_IER_CPCS		Disables the RC Compare Interrupt
				TIM_IER_LDRAS	Disables the RA Load Interrupt
				TIM_IER_LDRBS	Disables the RB Load Interrupt
				TIM_IER_ETRGS	Disables the External Trigger Interrupt
  * @retval None.
  */
void TIM_Disable_IT(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL, uint32_t ul_sources)
{
    TIM_Channel *tim_channel;
    tim_channel = TIMx->CHANNEL + TIM_CHANNEL;
    tim_channel->IDR = ul_sources;
}

/**
  * @brief  Read the TIM interrupt mask for the specified channel.
  * @param  TIMx:        Module hardware register base address pointer
  * @param  TIM_CHANNEL: Channel to read
  * @retval The TIM interrupt mask value.
  */
uint32_t TIM_Get_IT_Mask(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIM_Channel *tim_channel;
    tim_channel = TIMx->CHANNEL + TIM_CHANNEL;
    return tim_channel->IMR;
}

/**
  * @brief  Get the current status for the specified TIM channel.
  * @param  TIMx:    Module hardware register base address pointer
  * @param  CHANNEL: Channel number
  * @retval The current TIM status..
  */
uint32_t TIM_Get_Status(TIM_Type *TIMx, TIM_CHANNEL_Enum TIM_CHANNEL)
{
    TIM_Channel *tim_channel;
    tim_channel = TIMx->CHANNEL + TIM_CHANNEL;
    return tim_channel->SR;
}

/**
  * @brief  Enable TIM QDEC interrupts.
  * @param  TIMx:       Module hardware register base address pointer
  * @param  ul_sources: A bitmask of QDEC interrupts to be enabled
  *				 Where the input parameter "ul_sources" can be one or more of the following:
  *				             TIM_QIER_IDX			Enable the rising edge detected on IDX input interrupt
  *				             TIM_QIER_DIRCHG	Enable the change in rotation direction detected interrupt
  *				             TIM_QIER_QERR		Enable the quadrature error detected on PHA/PHB interrupt
  * @retval None.
  */
void TIM_Qdec_Enable_IT(TIM_Type *TIMx, uint32_t ul_sources)
{
    TIMx->QIER = ul_sources;
}

/**
  * @brief  Disable TIM QDEC interrupts.
  * @param  TIMx:       Module hardware register base address pointer
  * @param  ul_sources: A bitmask of QDEC interrupts to be disabled
  *				 Where the input parameter "ul_sources" can be one or more of the following:
  *				             TIM_QIDR_IDX		 Disable the rising edge detected on IDX input interrupt
  *				             TIM_QIDR_DIRCHG Disable the change in rotation direction detected interrupt
  *				             TIM_QIDR_QERR	 Disable the quadrature error detected on PHA/PHB interrupt
  * @retval None.
  */
void TIM_Qdec_Disable_IT(TIM_Type *TIMx, uint32_t ul_sources)
{
    TIMx->QIDR = ul_sources;
}

/**
  * @brief  Read TIM QDEC interrupt mask.
  * @param  TIMx: Module hardware register base address pointer.
  * @retval The QDEC interrupt mask value.
  */
uint32_t TIM_GetQdec_IT_Mask(TIM_Type *TIMx)
{
    return TIMx->QIMR;
}

/**
  * @brief  Get current TIM QDEC interrupt status.
  * @param  TIMx: Module hardware register base address pointer.
  * @retval The QDEC interrupt mask value.
  */
uint32_t TIM_GetQdec_IT_Status(TIM_Type *TIMx)
{
    return TIMx->QISR;
}
/**
  * @brief  Set TIM_FMR fault mode.
  * @param  TIMx: 				 Module hardware register base address pointer
  * @param  ul_faultmode: Faultmode to set
  * @retval None.
  */
void TIM_SetFaultMode(TIM_Type *TIMx, uint32_t ul_faultmode)
{
    TIMx->FMR = ul_faultmode;
}

/**
  * @brief  Enable or disable write protection of TIM registers.
  * @param  TIMx: 			Module hardware register base address pointer
  * @param  ul_enable: 1--enable, 0--disable
  * @retval None.
  */
void TIM_SetWriteProtect(TIM_Type *TIMx, uint32_t ul_enable)
{
    if (ul_enable)
    {
        TIMx->WPMR = TIM_WPMR_WPKEY_PASSWD | TIM_WPMR_WPEN;
    }
    else
    {
        TIMx->WPMR = TIM_WPMR_WPKEY_PASSWD;
    }
}


