/**
  ******************************************************************************
  * @file           : ms60f3xxx_uart.c
  * @author         : sinomcu.com-SA
  * @brief          : UART module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx_uart.h"
#include "ms60f3xxx_scu.h"
#include <stdio.h>
/* UART Communicate buffer size. */
uint8_t g_UART_Sbuffer[64];
uint8_t g_UART_Rbuffer[64];
/**
  *@brief enable UART transmitter.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_TxEnable(UART_Type *p_uart)
{
    /* Enable transmitter */
    p_uart->CR = UART_CR_TXEN;
}

/**
  *@brief Disable UART transmitter.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_TxDisable(UART_Type *p_uart)
{
    /* Disable transmitter */
    p_uart->CR = UART_CR_TXDIS;
}

/**
  *@brief Reset UART transmitter.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_TxReset(UART_Type *p_uart)
{
    /* Reset transmitter */
    p_uart->CR = UART_CR_RSTTX | UART_CR_TXDIS;
}

/**
  *@brief Enable UART receiver.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_RxEnable(UART_Type *p_uart)
{
    /* Enable receiver */
    p_uart->CR = UART_CR_RXEN;
}

/**
  *@brief Disable UART receiver.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_RxDisable(UART_Type *p_uart)
{
    /* Disable receiver */
    p_uart->CR = UART_CR_RXDIS;
}

/**
  *@brief Reset UART receiver.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_RxReset(UART_Type *p_uart)
{
    /* Reset receiver */
    p_uart->CR = UART_CR_RSTRX | UART_CR_RXDIS;
}

/**
  *@brief Enable UART receiver and transmitter.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_Enable(UART_Type *p_uart)
{
    /* Enable receiver and transmitter */
    p_uart->CR = UART_CR_RXEN | UART_CR_TXEN;
}

/**
  *@brief Disable UART receiver and transmitter.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_Disable(UART_Type *p_uart)
{
    /* Disable receiver and transmitter */
    p_uart->CR = UART_CR_RXDIS | UART_CR_TXDIS;
}

/**
  *@brief Reset UART receiver and transmitter.
  *@aram *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_Reset(UART_Type *p_uart)
{
    /* Reset and disable receiver & transmitter */
    p_uart->CR = UART_CR_RSTRX | UART_CR_RSTTX
                 | UART_CR_RXDIS | UART_CR_TXDIS | UART_CR_RSTSTA;
}

/**
  *@brief Enable UART interrupts.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  *@param ul_sources Interrupts to be enabled.
  */
void UART_Enable_IT(UART_Type *p_uart, uint32_t ul_sources)
{
    p_uart->IER = ul_sources;
}

/**
  *@brief Disable UART interrupts.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  *@param ul_sources Interrupts to be disabled.
  */
void UART_Disable_IT(UART_Type *p_uart, uint32_t ul_sources)
{
    p_uart->IDR = ul_sources;
}

/**
  *@brief Read UART interrupt mask.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  *@return The interrupt mask value.
  */
uint32_t UART_GetInterruptMask(UART_Type *p_uart)
{
    return p_uart->IMR;
}

/**
  *@brief Get current status.
  *@param p_uart Pointer to a UART instance.
  *@return The current UART status.
  */
uint32_t UART_GetStatus(UART_Type *p_uart)
{
    return p_uart->SR;
}

/**
  *@brief Reset status bits.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  */
void UART_ResetStatus(UART_Type *p_uart)
{
    p_uart->CR |= UART_CR_RSTSTA;
}

/**
  *@brief Check if Transmit Hold Register is empty.
  *@Check if the last data written in UART_THR has been loaded in TSR and the
  *@last data loaded in TSR has been transmitted.
  *@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  *@retval 1 Transmitter is empty.
  *@retval 0 Transmitter is not empty.
  */
uint32_t UART_IsTxEmpty(UART_Type *p_uart)
{
    return (p_uart->SR & UART_SR_TXEMPTY) > 0;
}

/**
  *@brief Write to UART Transmit Holding Register
  *@Before writing user should check if tx is ready (or empty).
*@param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  *@param data Data to be sent.
  *@retval 0 Success.
  *@retval 1 I/O Failure, UART is not ready.
  */
ErrorStatus UART_Write(UART_Type *p_uart, const uint8_t uc_data)
{
    /* Check if the transmitter is ready */
    if (!(p_uart->SR & UART_SR_TXRDY))
        return ERROR;

    /* Send character */
    p_uart->THR = uc_data;
    return SUCCESS;
}

/**
  * @brief Read from UART Receive Holding Register.
  * Before reading user should check if rx is ready.
  * @param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  * @retval 0 Success.
  * @retval 1 I/O Failure, UART is not ready.
  */
ErrorStatus UART_Read(UART_Type *p_uart, uint8_t *puc_data)
{
    /* Check if the receiver is ready */
    if ((p_uart->SR & UART_SR_RXRDY) == 0)
        return ERROR;

    /* Read character */
    *puc_data = (uint8_t) p_uart->RHR;
    return SUCCESS;
}
/**
 * @brief Configure the receive timeout register.
 * @param *p_uart Pointer to a UART instance: UART0 UART1UART2.
 * @param timeout The value of receive timeout.
 */
void UART_SetRxTimeout(UART_Type *p_usart, uint32_t timeout)
{
    p_usart->RTOR = timeout;
}

/**
 * @brief  DMA received.
 * @param  *p_uart Pointer to a UART instance: UART0 UART1UART2.
 * @param  data:   data to be received.
 * @param  len:    len to be received.
 * @retval None
 */
void UART_DMA_Receive(UART_Type *p_uart, uint8_t *data, uint32_t len)
{
    p_uart->PTCR = UART_PTCR_RXTDIS;
    p_uart->RPR  = (uint32_t)data;
    p_uart->RCR	 = len;
    p_uart->PTCR = UART_PTSR_RXTEN;
}

/**
  * @brief UART_DMASend function.
  * @param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  * @param *data:Data pointer to be sent.
  * @param len: Length of data to be sent.
  */
void UART_DMASend(UART_Type *p_uart, uint8_t *data, uint32_t len)
{
    while(!(p_uart->SR & UART_SR_ENDTX))
    {
        __NOP();
    }

    p_uart->PTCR = UART_PTCR_TXTDIS;
    p_uart->TPR = (uint32_t)data;
    p_uart->TCR = len;
    p_uart->PTCR = UART_PTCR_TXTEN;
}

/**
  * @brief UART_Send.
  * @param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  * @param data:Data to be sent.
  */
void	UART_Send(UART_Type *p_uart, uint8_t data)
{
    while(!(p_uart->SR & UART_SR_TXEMPTY ))
    {
        __NOP();
    };

    p_uart->THR = data;
}

/**
  * @brief before using UART call it
  * @param *p_uart Pointer to a UART instance: UART0 UART1UART2.
  * @param UART_Init_T   baudrate, baud Rate = SelectedClock/(8*(2-OVER)(CD+FP/8)
  *                         CD=SelectedClock/(8*(2-OVER)
  *                         FP=(SelectedClock-8*(2-OVER)*baudrate*CD)/uart_init_def->UART_BaudRate/2-OVER
  *                         p_uart->BRGR=cd+((uint32_t)fp<<16);
  * @param UART_Init_T   UART_WordLength:DATA_5_BIT,DATA_6_BIT,DATA_7_BIT,DATA_8_BIT,,
  * @param UART_Init_T   UART_Stop_BIT_TypeDef
  *                         BIT_1,            1 stop bit
  *                         BIT_1_5,          1.5 stop bit (SYNC = 0) or reserved (SYNC = 1)
  *                         BIT_2,            2 stop bit
  * @param UART_Init_T  UART_Parity_TypeDef
  *                        EVEN,              Even parity
  *                        ODD  ,             Odd parity
  *                        SPACE,             Parity forced to 0 (Space)
  *                        MARK,              Parity forced to 1 (Mark)
  *                        NO ,               No parity
  *                        MULTIDROP,         Multidrop mode
  * @param UART_Init_T  USART_MODE_TypeDef
  *                        NORMAL_MODE
  * @retval NONE.
  */
void UART_Init(UART_Type* p_uart, UART_Init_T *p_uart_init)
{
    uint32_t cd_fp;
    uint16_t cd;                                                                                              /*baud rate interger*/
    uint8_t fp;                                                                                               /*bate rate Fractional*/
    uint8_t over;
    UART_Reset(p_uart);                                                                                       /*reset uart*/
    cd_fp = 0;
    cd_fp = p_uart_init->Parity + p_uart_init->WordLength + p_uart_init->StopBits; /*set parity,word len,stop bit*/

    p_uart->MR = cd_fp;

    if( p_uart_init->OverSampling == UART_OVERSAMPLING_16 )
        over = 16;
    else
    {
        over = 8;
        p_uart->MR |=  (0x1u << 19);
    }

    /*caculate uart baudrate*/
    cd_fp = (8 * SCU_Get_frequecny() + (over * p_uart_init->BaudRate) / 2) / (over * p_uart_init->BaudRate);
    cd = (cd_fp >> 3);                                                                                      /*cd =cd_fp/8*/
    fp = cd_fp & 0x07;                                                                                      /*fp=cd_fp % 8*/

    p_uart->BRGR = cd + ((uint32_t)fp << 16);
    UART_Enable(p_uart);
    p_uart->RTOR = p_uart_init->TimeOut;

}

/**
  * @brief fputcUsed to configure printf.
  * @param ch:Data to be sent.
  * DON'T DELETE THIS FILE!
  */
int fputc( int ch, FILE *f)
{
    UART_Send(DEBUG_UART, ch);
    return ch;
}
