/**
  ******************************************************************************
  * @file           : ms60f3xxx_usart.c
  * @author         : sinomcu.com-SA
  * @brief          : USART module driver.
  * @gcc            : toolchain version   5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */
#include "ms60f3xxx.h"
#include "ms60f3xxx_usart.h"
#include "ms60f3xxx_scu.h"
#include <stdio.h>
/**
 * @brief  Enable USART transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_EnableTx(USART_Type *p_uart)
{
    /* Enable transmitter */
    p_uart->CR = USART_CR_TXEN;
}

/**
 * @brief  Disable USART transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_DisableTx(USART_Type *p_uart)
{
    /* Disable transmitter */
    p_uart->CR = USART_CR_TXDIS;
}

/**
 * @brief  Reset USART transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_ResetTx(USART_Type *p_uart)
{
    /* Reset transmitter */
    p_uart->CR = USART_CR_RSTTX | USART_CR_TXDIS;
}

/**
 * @brief  Enable USART receiver.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_EnableRx(USART_Type *p_uart)
{
    /* Enable receiver */
    p_uart->CR = USART_CR_RXEN;
}

/**
 * @brief  Disable USART receiver.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_DisableRx(USART_Type *p_uart)
{
    /* Disable receiver */
    p_uart->CR = USART_CR_RXDIS;
}

/**
 * @brief  Reset USART receiver.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_ResetRx(USART_Type *p_uart)
{
    /* Reset receiver */
    p_uart->CR = USART_CR_RSTRX | USART_CR_RXDIS;
}

/**
 * @brief  Enable USART receiver and transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_Enable(USART_Type *p_uart)
{
    /* Enable receiver and transmitter */
    p_uart->CR = USART_CR_RXEN | USART_CR_TXEN;
}

/**
 * @brief  Disable USART receiver and transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_Disable(USART_Type *p_uart)
{
    /* Disable receiver and transmitter */
    p_uart->CR = USART_CR_RXDIS | USART_CR_TXDIS;
}

/**
 * @brief  Reset USART receiver and transmitter.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_Reset(USART_Type *p_uart)
{
    /* Reset and disable receiver & transmitter */
    p_uart->CR = USART_CR_RSTRX | USART_CR_RSTTX
                 | USART_CR_RXDIS | USART_CR_TXDIS;
}

/**
 * @brief  Enable USART interrupts.
 * @param  p_uart:     Pointer to a USART instance.
 * @param  ul_sources: Interrupts to be enabled.
 * @retval None.
 */
void USART_Enable_IT(USART_Type *p_uart, uint32_t ul_sources)
{
    p_uart->IER = ul_sources;
}

/**
 * @brief  Disable USART interrupts.
 * @param  p_uart:     Pointer to a USART instance.
 * @param  ul_sources: Interrupts to be disabled.
 * @retval None.
 */
void USART_Disable_IT(USART_Type *p_uart, uint32_t ul_sources)
{
    p_uart->IDR = ul_sources;
}

/**
 * @brief  Read USART interrupt mask.
 * @param  p_uart:     Pointer to a USART instance.
 * @return The interrupt mask value.
 */
uint32_t USART_GetInterruptMask(USART_Type *p_uart)
{
    return p_uart->IMR;
}

/**
 * @brief  Get current status.
 * @param  p_uart: Pointer to a USART instance.
 * @return The current USART status.
 */
uint32_t USART_GetStatus(USART_Type *p_uart)
{
    return p_uart->SR;
}

/**
 * @brief  Reset status bits.
 * @param  p_uart: Pointer to a USART instance.
 * @retval None.
 */
void USART_ResetStatus(USART_Type *p_uart)
{
    p_uart->CR |= USART_CR_RSTSTA;
}

/**
 * @brief  Check if Transmit Hold Register is empty.
 * @note   Check if the last data written in USART_THR has been loaded in TSR and the
 * 				 last data loaded in TSR has been transmitted.
 * @param  p_uart: Pointer to a USART instance.
 * @retval 1 Transmitter is empty.
 * @retval 0 Transmitter is not empty.
 */
uint32_t USART_IsTxEmpty(USART_Type *p_uart)
{
    return (p_uart->SR & USART_SR_TXEMPTY) > 0;
}

/**
 * @brief  Write to USART Transmit Holding Register
 * @note   Before writing user should check if tx is ready (or empty).
 * @param  p_uart:  Pointer to a USART instance.
 * @param  uc_data: Data to be sent.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_Write(USART_Type *p_uart, const uint8_t uc_data)
{
    /* Check if the transmitter is ready */
    if (!(p_uart->SR & USART_SR_TXRDY))
        return ERROR;

    /* Send character */
    p_uart->THR = uc_data;
		while(!(p_uart->SR & USART_SR_TXEMPTY))
			;
    return SUCCESS;
}

/**
 * @brief  Read from USART Receive Holding Register.
 * @note   Before reading user should check if rx is ready.
 * @param  p_uart:    Pointer to a USART instance.
 * @param  p_uc_data: Data to be read.
 * @retval SUCCESS.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_Read(USART_Type *p_uart, uint8_t *p_uc_data)
{
    /* Check if the receiver is ready */
    if ((p_uart->SR & USART_SR_RXRDY) == 0)
        return ERROR;

    /* Read character */
    *p_uc_data = (uint8_t) p_uart->RHR;
    return SUCCESS;
}
/**
 * @brief  USART send data.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data  to be sent.
 * @retval None.
 */
void	USART_Send(USART_Type *p_uart, uint8_t data)
{
    while(!(p_uart->SR & USART_SR_TXEMPTY ))
    {
			 __NOP();
    }

    p_uart->THR = data;
}

/**
 * @brief  SPI send and receive data.
 * @note   Before reading user should check if USART_SR_TXRDY is ready.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data to be sent.
 * @retval SPI receive data..
 */
uint8_t USART_SPI_SendData(USART_Type *p_uart, uint8_t data)
{

    USART_Send(p_uart, data);

    while(!(p_uart->SR & USART_SR_TXRDY ));

    return p_uart->RHR;
}
/**
 * @brief  SPI send datas.
 * @note   Before reading user should check if USART_SR_TXRDY is ready.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data to be sent.
 * @param  len:    data len.
 * @retval None.
 */
void USART_SPI_SendBuffer(USART_Type *p_uart, uint8_t *data, uint8_t len)
{
    uint8_t i;

    for(i = 0; i < len; i++)
        USART_SPI_SendData(p_uart, data[i]);
}

/**
 * @brief  RS485 send datas.
 * @note   Before reading user should check if rx is ready.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data to be sent.
 * @param  len:    len to be sent.
 * @retval None.
 */
void RS485_SendBuffer(USART_Type *p_uart, uint8_t *data, uint32_t len)
{
    uint32_t counter;
    uint8_t *pdata = data;

    for(counter = 0; counter < len; counter++)
        USART_Send(p_uart, pdata[counter]);
}

/**
 * @brief  RS485 set The Transmitter timeguard is enabled and the timeguard delay
 *  			 this parameter is only for RS485 communication
 * @param  p_uart:      Pointer to a USART instance.
 * @param  time_guard:  time_guard 0: The Transmitter Timeguard is disabled..
 * @param  time_guard:  time_guard 1-255: The Transmitter timeguard is enabled and the timeguard delay is TG x Bit Period
 * @retval None.
 */
void RS485_SetTimeGuard(USART_Type *p_uart, uint8_t time_guard)
{
    p_uart->TTGR = time_guard;
}

/**
 * @brief  Configure the receive timeout register.
 * @param  p_usart: Pointer to a USART instance.
 * @param  timeout: The value of receive timeout.
 * @retval None.
 */
void USART_SetRxTimeout(USART_Type *p_usart, uint32_t timeout)
{
    p_usart->RTOR = timeout;
}

/**
 * @brief  Start transmission of a break.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None.
 */
void USART_StartTxBreak(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_STTBRK;
}

/**
 * @brief  Stop transmission of a break.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None.
 */
void USART_StopTxBreak(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_STPBRK;
}

/**
 * @brief  In Multidrop mode only, the next character written to the USART_THR
 * 			   is sent with the address bit set.
 * @param  p_usart: Pointer to a USART instance.
 * @param  ul_addr: The address to be sent out.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_SendAddress(USART_Type *p_usart, uint32_t ul_addr)
{
    if ((p_usart->MR & USART_MR_PAR_MULTIDROP) != USART_MR_PAR_MULTIDROP)
        return ERROR;

    p_usart->CR = USART_CR_SENDA;

    if (USART_Write(p_usart, ul_addr))
        return ERROR;
    else
        return SUCCESS;
}

/**
 * @brief  Calculate a clock divider(CD) and a fractional part (FP) for the
 *  			 USART asynchronous modes to generate a baudrate as close as possible to the baudrate set point.
 * @note   Baud rate calculation: Baudrate = ul_bus_clk/(Over * (CD + FP/8)) * (Over being 16 or 8).
 *  			 The maximal oversampling is selected if it allows to generate a baudrate close to the set point.
 * @param  p_usart Pointer to a USART instance.
 * @param  baudrate Baud rate set point.
 * @param  ul_bus_clk USART module input clock frequency.
 * @retval SUCCESS Baud rate is successfully initialized.
 */
#define HIGH_FRQ_SAMPLE_DIV 16
#define LOW_FRQ_SAMPLE_DIV 8

ErrorStatus USART_SetAsyncBaudrate(USART_Type *p_usart,
                                   uint32_t baudrate, uint32_t ul_bus_clk)
{
    uint32_t over;
    uint32_t cd_fp;
    uint32_t cd;
    uint32_t fp;

    /* Calculate the receiver sampling divide of baudrate clock. */
    #if 1

    if (ul_bus_clk >= HIGH_FRQ_SAMPLE_DIV * baudrate)
    {
        over = HIGH_FRQ_SAMPLE_DIV;
    }
    else
    {
        over = LOW_FRQ_SAMPLE_DIV;
    }

    #endif
    /* Calculate clock divider according to the fraction calculated formula. */
    cd_fp = (8 * ul_bus_clk + (over * baudrate) / 2) / (over * baudrate);
    cd = cd_fp >> 3;
    fp = cd_fp & 0x07;

    /* Configure the OVER bit in MR register. */
    if (over == 8)
    {
        p_usart->MR |= USART_MR_OVER;
    }

    /* Configure the baudrate generate register. */
    p_usart->BRGR = cd | (fp << 16);

    return SUCCESS;
}
/**
 * @brief  Calculate a clock divider for the USART synchronous master modes
 * 			   to generate a baudrate as close as possible to the baudrate set point.
 * @note   Synchronous baudrate calculation: baudrate = ul_bus_clk / cd
 * @param  p_usart:    Pointer to a USART instance.
 * @param  baudrate:   Baud rate set point.
 * @param  ul_bus_clk: USART module input clock frequency.
 * @retval SUCCESS Baud rate is successfully initialized.
 */
static ErrorStatus USART_SetSyncMasterBaudrate(USART_Type *p_usart, uint32_t baudrate, uint32_t ul_bus_clk)
{
    uint32_t cd;
    /* Calculate clock divider according to the formula in synchronous mode. */
    cd = (ul_bus_clk + baudrate / 2) / baudrate;
    #if 0

    if (cd < MIN_CD_VALUE || cd > MAX_CD_VALUE)
    {
        return ERRROR;
    }

    #endif
    /* Configure the baudrate generate register. */
    p_usart->BRGR = cd << USART_BRGR_CD_Pos;

    p_usart->MR = USART_MR_SYNC;
    return SUCCESS;
}

/**
 * @brief  Select the SCK pin as the source of baud rate for the USART synchronous slave modes.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None.
 */
static void USART_SetSyncSlaveBaudrate(USART_Type *p_usart)
{
    p_usart->MR = USART_MR_SYNC;
}

/**
 * @brief  Calculate a clock divider (\e CD) for the USART SPI master mode to
 * 				 generate a baud rate as close as possible to the baud rate set point.
 * @note   Baud rate calculation:
 * @param  p_usart: Pointer to a USART instance.
 * @param  baudrate: Baud rate set point.
 * @param  ul_bus_clk: USART module input clock frequency.
 * @retval SUCCESS Baud rate is successfully initialized.
**/
static ErrorStatus USART_Set_SPI_MasterBaudrate(USART_Type *p_usart, uint32_t baudrate, uint32_t ul_bus_clk)
{
    uint32_t cd;

    /* Calculate the clock divider according to the formula in SPI mode. */
    cd = (ul_bus_clk + baudrate / 2) / baudrate;
    #if 0

    if (cd < MIN_CD_VALUE_SPI || cd > MAX_CD_VALUE)
    {
        return ERROR;
    }

    #endif
    p_usart->BRGR = cd << USART_BRGR_CD_Pos;
    return SUCCESS;
}

/**
 * @brief  Disable write protect of USART registers.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None.
 */
void USART_DisableWriteProtect(USART_Type *p_usart)
{
    p_usart->WPMR = USART_WPMR_WPKEY_PASSWD;
}

/**
 * @brief  Drive the pin RTS to 1.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None.
 */
void USART_RTS_High(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_RTSDIS;
}
/**
 * @brief  Configure USART to work in RS232 mode.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitRS232(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;

    /* Check whether the input values are legal. */
    if (!p_usart_opt || USART_SetAsyncBaudrate(p_usart,
            p_usart_opt->baudrate, ul_bus_clk))
    {
        return ERROR;
    }

    /* Configure the USART option. */
    ul_reg_val |= p_usart_opt->USART_WordLength | p_usart_opt->USART_Parity |
                  p_usart_opt->USART_Mode | p_usart_opt->USART_StopBits;

    /* Configure the USART mode as normal mode. */
    ul_reg_val |= USART_MR_USART_MODE_NORMAL;

    p_usart->MR |= ul_reg_val;

    return SUCCESS;
}

/**
 * @brief  Configure USART to work in hardware handshaking mode.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitHwHandshaking(USART_Type *p_usart,
                                    const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk)
{
    /* Initialize the USART as standard RS232. */
    if (USART_InitRS232(p_usart, p_usart_opt, ul_bus_clk))
    {
        return ERROR;
    }

    /* Set hardware handshaking mode. */
    p_usart->MR = (p_usart->MR & ~USART_MR_USART_MODE_Msk) |
                  USART_MR_USART_MODE_HW_HANDSHAKING;

    return SUCCESS;
}

/**
 * @brief  Configure USART to work in SYNC mode and act as a master.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitSyncMaster(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;

    /* Check whether the input values are legal. */
    if (!p_usart_opt || USART_SetSyncMasterBaudrate(p_usart,
            p_usart_opt->baudrate, ul_bus_clk))
    {
        return ERROR;
    }

    /* Configure the USART option. */
    ul_reg_val |= p_usart_opt->USART_WordLength | p_usart_opt->USART_Parity |
                  p_usart_opt->USART_Mode | p_usart_opt->USART_StopBits;
    /* Set normal mode and output clock as synchronous master. */
    ul_reg_val |= USART_MR_USART_MODE_NORMAL | USART_MR_CLKO;
    p_usart->MR |= ul_reg_val;
    return SUCCESS;
}

/**
 * @brief  Configure USART to work in SYNC mode and act as a slave.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitSyncSlave(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;
    USART_SetSyncSlaveBaudrate(p_usart);

    /* Check whether the input values are legal. */
    if (!p_usart_opt)
    {
        return ERROR;
    }

    /* Configure the USART option. */
    ul_reg_val |= p_usart_opt->USART_WordLength | p_usart_opt->USART_Parity |
                  p_usart_opt->USART_Mode | p_usart_opt->USART_StopBits;

    /* Set normal mode. */
    ul_reg_val |= USART_MR_USART_MODE_NORMAL;
    p_usart->MR |= ul_reg_val;

    return SUCCESS;
}
/**
 * @brief Configure USART to work in IrDA mode.
 * @note By default, the transmitter and receiver aren't enabled.
 * @param p_usart:     Pointer to a USART instance.
 * @param p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @param ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitIrda(USART_Type *p_usart, const USART_SimilarModeInit_T *p_usart_opt, uint32_t ul_bus_clk)
{
    /* Initialize the USART as standard RS232. */
    if (USART_InitRS232(p_usart, p_usart_opt, ul_bus_clk))
    {
        return ERROR;
    }

    /* Set IrDA filter. */
    p_usart->IF = p_usart_opt->irda_filter;

    /* Set IrDA mode. */
    p_usart->MR = (p_usart->MR & ~USART_MR_USART_MODE_Msk) |
                  USART_MR_USART_MODE_IRDA;

    return SUCCESS;
}

/**
 * @brief  Calculate a clock divider (\e CD) for the USART ISO7816 mode to
 * 			   generate an ISO7816 clock as close as possible to the clock set point.
 * @note   ISO7816 clock calculation: Clock = ul_bus_clk / cd
 * @param  p_usart:    Pointer to a USART instance.
 * @param  clock:      ISO7816 clock set point.
 * @param  ul_bus_clk: USART module input clock frequency.
 * @retval SUCCESS ISO7816 clock is successfully initialized.
 */
static ErrorStatus USART_SetIso7816_Clock(USART_Type *p_usart,
        uint32_t clock, uint32_t ul_bus_clk)
{
    uint32_t cd;
    cd = (ul_bus_clk + clock / 2) / clock;

    p_usart->MR = (USART_MR_SYNC | USART_MR_OVER | USART_MR_CLKO );

    /* Configure the baudrate generate register. */
    p_usart->BRGR = cd ;
    return SUCCESS;
}

/**
 * @brief  Configure USART to work in ISO7816 mode.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart Pointer to a USART instance.
 * @param  p_usart_opt Pointer to sam_usart_opt_t instance.
 * @param  ul_bus_clk USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitIso7816(USART_Type *p_usart, USART_ISO7816_OPT_T *p_usart_opt, uint32_t ul_bus_clk)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;

    /* Check whether the input values are legal. */
    if (!p_usart_opt || ((p_usart_opt->parity_type != USART_MR_PAR_EVEN) &&
                         (p_usart_opt->parity_type != USART_MR_PAR_ODD)))
    {
        return ERROR;
    }

    if (p_usart_opt->protocol_type == ISO7816_T_0)
    {    ul_reg_val |= USART_MR_USART_MODE_IS07816_T_0 | USART_MR_NBSTOP_2_BIT |
                      (p_usart_opt->max_iterations << USART_MR_MAX_ITERATION_Pos);

        if (p_usart_opt->bit_order) 
            ul_reg_val |= USART_MR_MSBF;
    }
    else if (p_usart_opt->protocol_type == ISO7816_T_1) {
        /*
         * Only LSBF is used in the T=1 protocol, and max_iterations field
         * is only used in T=0 mode.
         */
        if (p_usart_opt->bit_order || p_usart_opt->max_iterations)
            return ERROR;

        /* Set USART mode to ISO7816, T=1, and always uses 1 stop bit. */
        ul_reg_val |= USART_MR_USART_MODE_IS07816_T_1 | USART_MR_NBSTOP_1_BIT;
    }
    else 
        return ERROR;

    /* Set up the baudrate. */
    if (USART_SetIso7816_Clock(p_usart, p_usart_opt->iso7816_hz, ul_bus_clk)) 
        return ERROR;

    /* Set FIDI register: bit rate = iso7816_hz / fidi_ratio. */
    p_usart->FIDI = p_usart_opt->fidi_ratio;

    /* Set ISO7816 parity type in the MODE register. */
    ul_reg_val |= p_usart_opt->parity_type;

    if (p_usart_opt->inhibit_nack) {
        ul_reg_val |= USART_MR_INACK;
    }

    if (p_usart_opt->dis_suc_nack) {
        ul_reg_val |= USART_MR_DSNACK;
    }

    p_usart->MR |= ul_reg_val;
    return SUCCESS;
}
/**
 * @brief  Reset the ITERATION in USART_CSR when the ISO7816 mode is enabled.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_ResetIterations(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_RSTIT;
}

/**
 * @brief  Reset NACK in USART_CSR.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_ResetNack(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_RSTNACK;
}

/**
 * @brief  Configure USART to work in SPI mode and act as a master.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_SPI_MasterInit(USART_Type *p_usart, USART_SPI_T *p_usart_spi, uint32_t ul_bus_clk)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;

    /* Check whether the input values are legal. */
    if (!p_usart_spi ||
            USART_Set_SPI_MasterBaudrate(p_usart, p_usart_spi->baudrate,
                                         ul_bus_clk))
    {
        /* Configure the character length bit in MR register. */
        ul_reg_val |= p_usart_spi->char_length;

        /* Set SPI master mode and channel mode. */
        ul_reg_val |= (SPI_MASTER_MODE | USART_MR_CLKO |
                       p_usart_spi->channel_mode |	p_usart_spi->spi_mode);
        p_usart->MR |= ul_reg_val;
        USART_Enable(p_usart);
        return SUCCESS;
    }

    return ERROR;
}

/**
 * @brief  Configure USART to work in SPI mode and act as a slave.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  p_usart_opt: Pointer to sam_usart_opt_t instance.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_SPI_SlaveInit(USART_Type *p_usart, USART_SPI_T *p_usart_spi)
{
    static uint32_t ul_reg_val;

    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    ul_reg_val = 0;

    /* Check whether the input values are legal. */
    if (!p_usart_spi || p_usart_spi->spi_mode > SPI_MODE_3)
    {
        return ERROR;
    }

    /* Configure the character length bit in MR register. */
    ul_reg_val |= p_usart_spi->char_length;
    /* Set SPI slave mode and channel mode. */
    ul_reg_val |= USART_MR_USART_MODE_SPI_SLAVE | p_usart_spi->channel_mode;
    ul_reg_val |= p_usart_spi->spi_mode;
    p_usart->MR |= ul_reg_val;
    USART_Enable(p_usart);
    return SUCCESS;
}
/**
 * @brief  Configure USART to work in LIN mode and act as a LIN master.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  ul_baudrate: Baudrate to be used.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus  USART_InitLinMaster(USART_Type *p_usart, uint32_t ul_baudrate, uint32_t ul_bus_clk)
{
    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);

    /* Set up the baudrate. */
    if (USART_SetAsyncBaudrate(p_usart, ul_baudrate, ul_bus_clk) == ERROR)
    {
        return ERROR;
    }

    /* Set LIN master mode. */
    p_usart->MR = (p_usart->MR & ~USART_MR_USART_MODE_Msk) | LIN_MASTER_MODE;
    USART_EnableRx(p_usart);
    USART_EnableTx(p_usart);
    return SUCCESS;
}
/**
 * @brief  Configure USART to work in LIN mode and act as a LIN slave.
 * @note   By default, the transmitter and receiver aren't enabled.
 * @param  p_usart:     Pointer to a USART instance.
 * @param  ul_baudrate: Baudrate to be used.
 * @param  ul_bus_clk:  USART module input clock frequency.
 * @retval SUCCESS or ERROR.
 */
ErrorStatus USART_InitLinSlave(USART_Type *p_usart, uint32_t ul_baudrate, uint32_t ul_bus_clk)
{
    /* Reset the USART and shut down TX and RX. */
    USART_Reset(p_usart);
    USART_EnableRx(p_usart);
    USART_EnableTx(p_usart);
    /* Set LIN slave mode. */
    p_usart->MR = (p_usart->MR & ~USART_MR_USART_MODE_Msk) | LIN_SLAVE_MODE;

    /* Set up the baudrate. */
    if (USART_SetAsyncBaudrate(p_usart, ul_baudrate, ul_bus_clk))
    {
        return ERROR;
    }

    return SUCCESS;
}
/**
 * @brief  Abort the current LIN transmission.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinAbortTx(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_LINABT;
}
/**
 * @brief  Send a wakeup signal on the LIN bus.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinSendWakeupSignal(USART_Type *p_usart)
{
    p_usart->CR = USART_CR_LINWKUP;
}
/**
 * @brief Configure the LIN node action, which should be one of PUBLISH,SUBSCRIBE or IGNORE.

 * @param p_usart Pointer to a USART instance.
 * @param uc_action 0 for PUBLISH, 1 for SUBSCRIBE, 2 for IGNORE.
**/
void USART_LinSetNodeAction(USART_Type *p_usart, uint8_t uc_action)
{
    p_usart->LIN_MR = (p_usart->LIN_MR & ~USART_LINMR_NACT_Msk) | (uc_action << USART_LINMR_NACT_Pos);
}
/**
 * @brief  Disable the parity check during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinDisableParity(USART_Type *p_usart)
{
    p_usart->LIN_MR |= USART_LINMR_PARDIS;
}
/**
 * @brief  Enable the parity check during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinEnableParity(USART_Type *p_usart)
{
    p_usart->LIN_MR &= ~USART_LINMR_PARDIS;
}
/**
 * @brief  Disable the checksum during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinDisableChecksum(USART_Type *p_usart)
{
    p_usart->LIN_MR |= USART_LINMR_CHKDIS;
}
/**
 * @brief  Enable the checksum during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinEnableChecksum(USART_Type *p_usart)
{
    p_usart->LIN_MR &= ~USART_LINMR_CHKDIS;
}
/**
 * @brief  Configure the checksum type during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @param  uc_type: 0 for LIN 2.0 Enhanced checksum or 1 for LIN 1.3 Classic checksum.
 * @retval None
 */
void USART_LinSetChecksumType(USART_Type *p_usart, uint8_t uc_type)
{
    p_usart->LIN_MR = (p_usart->LIN_MR & ~USART_LINMR_CHKTYP) | (uc_type << 4);
}
/**
 * @brief Configure the data length mode during the LIN communication.
 * @param p_usart: Pointer to a USART instance.
 * @param uc_mode: Indicate the data length type: 0 if the data length is
 *                 defined by the DLC of LIN mode register or 1 if the data length is defined
 *                 by the bit 5 and 6 of the identifier.
 * @retval None
 */
void USART_LinSetDataLenMode(USART_Type *p_usart, uint8_t uc_mode)
{
    p_usart->LIN_MR = (p_usart->LIN_MR & ~USART_LINMR_DLM) | (uc_mode << 5);
}
/**
 * @brief  Disable the frame slot mode during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinDisableFrameSlot(USART_Type *p_usart)
{
    p_usart->LIN_MR |= USART_LINMR_FSDIS;
}

/**
 * @brief  Enable the frame slot mode during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @retval None
 */
void USART_LinEnableFrameSlot(USART_Type *p_usart)
{
    p_usart->LIN_MR &= ~USART_LINMR_FSDIS;
}
/**
 * @brief Configure the wakeup signal type during the LIN communication.
 * @param p_usart: Pointer to a USART instance.
 * @param uc_type: Indicate the checksum type: 0 if the wakeup signal is a
 *                 LIN 2.0 wakeup signal; 1 if the wakeup signal is a LIN 1.3 wakeup signal.
 * @retval None
 */
void USART_LinSetWakeupSignalType(USART_Type *p_usart, uint8_t uc_type)
{
    p_usart->LIN_MR = (p_usart->LIN_MR & ~USART_LINMR_WKUPTYP) | (uc_type << 7);
}
/**
 * @brief  Configure the response data length if the data length is defined by
 *         the DLC field during the LIN communication.
 * @param  p_usart: Pointer to a USART instance.
 * @param  uc_len: Indicate the response data length.
 * @retval None
 */
void USART_LinSetResponseDataLen(USART_Type *p_usart, uint8_t uc_len)
{
    p_usart->LIN_MR = (p_usart->LIN_MR & ~USART_LINMR_DLC_Msk) |
                      ((uc_len - 1) << USART_LINMR_DLC_Pos);
}
/**
 * @brief  Configure the LIN identifier when USART works in LIN master mode.
 * @param  p_usart: Pointer to a USART instance.
 * @param  uc_id:   The identifier to be transmitted.
 * @retval None
 */
void USART_LinSetTxIdentifier(USART_Type *p_usart, uint8_t uc_id)
{
    p_usart->LIN_IR = (p_usart->LIN_IR & ~USART_LINIR_IDCHR_Msk) | USART_LINIR_IDCHR(uc_id);
}
/**
 * @brief  Read the identifier when USART works in LIN mode.
 * @param  p_usart: Pointer to a USART instance.
 * @return The last identifier received in LIN slave mode or the last
 *         identifier transmitted in LIN master mode.
 */
uint8_t USART_LinReadIdentifier(USART_Type *p_usart)
{
    return (p_usart->LIN_IR & USART_LINIR_IDCHR_Msk);
}
/**
 * @brief  Get data length.
 * @param  p_usart: Pointer to a USART instance.
 * @return Data length.
 */
uint8_t USART_LinGetDataLength(USART_Type *usart)
{
    if (usart->LIN_MR & USART_LINMR_DLM)
    {
        uint8_t data_length = 1 << ((usart->LIN_IR >> (USART_LINIR_IDCHR_Pos + 4)) & 0x03);
        return data_length;
    }
    else
    {
        return ((usart->LIN_MR & USART_LINMR_DLC_Msk) >> USART_LINMR_DLC_Pos) + 1;
    }
}

/**
 * @brief  Get identifier send status.
 * @param  p_usart: Pointer to a USART instance.
 * @return
 *         0:  No LIN identifier has been sent since the last RSTSTA.
 *         1:  At least one LIN identifier has been sent since the last RSTSTA.
 */
uint8_t USART_LinIdentifierSendComplete(USART_Type *usart)
{
    return (usart->SR & USART_SR_LINTC) > 0;
}
/**
 * @brief  Get identifier received status.
 * @param  p_usart Pointer to a USART instance.
 * @return
 * 				 0:  No LIN identifier has been reveived since the last RSTSTA.
 *         1: At least one LIN identifier has been received since the last RSTSTA.
 */
uint8_t USART_LinIdentifierReceptionComplete(USART_Type *usart)
{
    return (usart->SR & USART_SR_LINID) > 0;
}

/**
 * @brief  DMA received.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data to be received.
 * @param  len:    len to be received.
 * @retval None
 */
void USART_DMA_Receive(USART_Type *p_uart, uint8_t *data, uint32_t len)
{
    p_uart->PTCR = USART_PTCR_RXTDIS;
    p_uart->RPR  = (uint32_t)data;
    p_uart->RCR	 = len;
    p_uart->PTCR = USART_PTSR_RXTEN;
}
/**
 * @brief  DMA send.
 * @param  p_uart: Pointer to a USART instance.
 * @param  data:   data to be sent.
 * @param  len:    len to be sent.
 * @retval None
 */
void USART_DMA_Send(USART_Type *p_uart, uint8_t *data, uint32_t len)
{
    p_uart->PTCR = USART_PTCR_TXTDIS;
    p_uart->TPR  = (uint32_t)data;
    p_uart->TCR  = len;
    p_uart->PTCR = USART_PTCR_TXTEN;
}

/**
 * @brief before using USART or USART485 or IRDAshould call uart_init
 * @param p_uart: Pointer to a USART instance.
 * @param USART_InitTypeDef:  baudrate, baud Rate = SelectedClock/(8*(2-OVER)(CD+FP/8)
 *                            CD=SelectedClock/(8*(2-OVER)
 *                            FP=(SelectedClock-8*(2-OVER)*baudrate*CD)/p_usart_init->USART_BaudRate/2-OVER
 *                            p_uart->BRGR=cd+((u32)fp<<16);
 * @param USART_InitTypeDef:  USART_WordLength:DATA_5_BIT,DATA_6_BIT,DATA_7_BIT,DATA_8_BIT,,
 * @param USART_InitTypeDef:  USART_Stop_BIT_TypeDef
 *                            BIT_1        //1 stop bit
 *                            BIT_1_5,     // 1.5 stop bit (SYNC = 0) or reserved (SYNC = 1)
 *                            BIT_2,       //2 stop bit
 * @param USART_InitTypeDef:  USART_Parity_TypeDef
 * 														EVEN,        // Even parity
 * 														ODD  ,       // Odd parity
 * 														SPACE,       // Parity forced to 0 (Space)
 * 														MARK,        // Parity forced to 1 (Mark)
 * 														NO ,         //No parity
 * 														MULTIDROP,   // Multidrop mode
 * @param USART_InitTypeDef:  USART_MODE_Enum
 *  													NORMAL_MODE
 * 														RS485_MODE
 * 														HW_HANDSHAKING_MODE
 * 														MODEM_MODE
 *  													IS07816_T_0_MODE
 * 														IS07816_T_1_MODE
 * 														IRDA_MODE
 * 														LIN_MASTER_MODE
 * 														LIN_SLAVE_MODE
 * 														SPI_MASTER_MODE
 * 														SPI_SLAVE_MODE
 * @retval None.
 */
void USART_Init(USART_Type* p_usart, USART_Init_T *p_usart_init)
{
    uint32_t cd_fp;
    uint16_t cd;
    uint8_t fp;
    uint8_t over;
    USART_Reset(p_usart);
	 
    p_usart->MR =    p_usart_init->Parity  | p_usart_init->WordLength
                   | p_usart_init->StopBits | p_usart_init->TransEncode| p_usart_init->Mode;

	  if( p_usart_init->OverSampling == USART_OVERSAMPLING_16 )
			over = 16;
		else {
			over = 8;
			p_usart->MR |= USART_MR_OVER;
		}
    cd_fp = (8 * SCU_Get_frequecny() + (over * p_usart_init->BaudRate) / 2) / (over * p_usart_init->BaudRate);
    cd = cd_fp >> 3;
    fp = cd_fp & 0x07;
    p_usart->BRGR = cd + ((u32)fp << 16);

    switch(p_usart_init->Mode)
    {
        case IRDA_MODE:
            p_usart->IF = p_usart_init->IrDA_Filter;
            break;

        case RS485_MODE:
            p_usart->TTGR = p_usart_init->TimeGuard;
            break;


        default:
            break;
    }
		if(p_usart_init->TransEncode)
		{USART->MAN=0xB0001000;}

    p_usart->RTOR = p_usart_init->TimeOut;
    USART_Enable(p_usart);
}
