/**
  ******************************************************************************
  * @file           : pwm.c
  * @author         : sinomcu.com-SA
  * @brief          : PWM complementary output Functions
  * @gcc            : toolchain version 5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with basic chip information support
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */  
#include "include.h"

#define PWM_FREQUENCY      10000
#define PWM_INIT_DUTY      50

#define PWMH_DEAD_TIMER_US   2
#define PWML_DEAD_TIMER_US   4

/**
 * @brief   PWM GPIO initialization
 * @param   None
 * @retval  None
 */	
void PWM_GPIO_Init(void)
{
    #if 0
    SCU_PeriphClockCmd(SCU_GPIOA_CLKEN, ENABLE);
    GPIO_Init(GPIOA, GPIO_PIN7, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN7, GPIO_ALT_PWM); /* PWMH0 */
    GPIO_Init(GPIOA, GPIO_PIN8, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN8, GPIO_ALT_PWM); /* PWML0 */
    #endif
    #if 0
    SCU_PeriphClockCmd(SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(GPIOB, GPIO_PIN4, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN4, GPIO_ALT_PWM); /* PWMH1 */
    GPIO_Init(GPIOB, GPIO_PIN5, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN5, GPIO_ALT_PWM); /* PWML1 */
    #endif
    #if 1
    SCU_PeriphClockCmd(SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(GPIOB, GPIO_PIN9, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN9, GPIO_ALT_PWM); /* PWMH2 */
    GPIO_Init(GPIOB, GPIO_PIN8, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN8, GPIO_ALT_PWM); /* PWML2 */
    #endif
}

/**
 * @brief   PWM Clock initialization
 * @param   p_ClockConfig: Configure Pwm clock.
 * @retval  None
 */	
void PWM_Clock_Config(PWM_ClOCK_T * p_ClockConfig)
{
    p_ClockConfig->ul_clka_prea = PWM_CLK_PREA_CLK_DIV8;
    p_ClockConfig->ul_clka_diva = PWM_CLK_DIVA_0_255(10);

    p_ClockConfig->ul_clkb_preb = PWM_CLK_PREB_CLK_DIV8;
    p_ClockConfig->ul_clkb_divb = PWM_CLK_DIVB_0_255(10);

    p_ClockConfig->ul_bus_clk   = SCU_Get_frequecny();
    PWM_ClocksInit(p_ClockConfig);
}

/**
 * @brief   PWM channel configuration
 * @param   Channel:Configurations of the specified PWM channel.
 * @retval  None
 */
void PWM_ChannelConfig(PWM_Channel_Enum Channel)
{
	  uint32_t PWM_Clock = 0;
	
	  PWM_ClOCK_T   PWM_ClockConfig  = { 0 };
    PWM_Channel_T PwmChannelConfig = { 0 };
		
    uint32_t ul_Pre[PWM_CLOCK_PRE_MAX] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024 };
		
		PWM_Clock_Config(&PWM_ClockConfig);
		
    PwmChannelConfig.Channel             = Channel;
		PwmChannelConfig.Polarity            = PWM_CMR_CPOL_LOW;
		#if 1
		PwmChannelConfig.Alignment           = PWM_ALIGN_LEFT;
		#else
		PwmChannelConfig.Alignment           = PWM_ALIGN_CENTER;
		#endif
		
		#if 1
		PwmChannelConfig.Prescaler           = PWM_CMR_CPRE_CLKA ;
		#elif 0
		PwmChannelConfig.Prescaler           = PWM_CMR_CPRE_CLKB ;
		#else
		PwmChannelConfig.Prescaler           = PWM_CMR_CPRE_CLK_BUS_DIV1024 ;
		#endif
		
		if( PwmChannelConfig.Prescaler == PWM_CMR_CPRE_CLKA )
			 PWM_Clock = SCU_Get_frequecny() / ul_Pre[PWM_ClockConfig.ul_clka_prea] / PWM_ClockConfig.ul_clka_diva;
		else if( PwmChannelConfig.Prescaler == PWM_CMR_CPRE_CLKB )
			 PWM_Clock = SCU_Get_frequecny() / ul_Pre[PWM_ClockConfig.ul_clkb_preb] / PWM_ClockConfig.ul_clkb_divb;
		else 
			 PWM_Clock = SCU_Get_frequecny() / ul_Pre[PwmChannelConfig.Prescaler];
		
		if( PwmChannelConfig.Alignment == PWM_ALIGN_LEFT )
			PwmChannelConfig.ul_period = PWM_Clock / PWM_FREQUENCY;
		else  /* PWM_ALIGN_CENTER */
			PwmChannelConfig.ul_period = PWM_Clock / PWM_FREQUENCY / 2 ;
		
		if(PwmChannelConfig.Polarity == PWM_CMR_CPOL_LOW)
		   PwmChannelConfig.ul_pulse         = PwmChannelConfig.ul_period - PwmChannelConfig.ul_period * PWM_INIT_DUTY / 100;
    else
			 PwmChannelConfig.ul_pulse         = PwmChannelConfig.ul_period * PWM_INIT_DUTY / 100;
		
    PwmChannelConfig.CounterEventSelect  = PWM_EVENT_PERIOD_HALF_END;

    PwmChannelConfig.DeadTimeGenerator   = DISABLE;
    PwmChannelConfig.PWMH_OutputInverted = DISABLE;
    PwmChannelConfig.PWML_OutputInverted = DISABLE;

    PwmChannelConfig.ul_pwmh_deadtime    = PWMH_DEAD_TIMER_US * PWM_Clock / (1000 * 1000 );
    PwmChannelConfig.ul_pwml_deadtime    = PWML_DEAD_TIMER_US * PWM_Clock / (1000 * 1000 );
		
    PwmChannelConfig.PPM_Mode            = PWM_CMR_PPM_DIS;

    PWM_ChannelInit(&PwmChannelConfig);
}

/**
 * @brief   PWM asynchronous mode initialization
 * @param   None
 * @retval  None
 */
void PWM_AsyncMode_Init(void)
{
    SCU_PeriphClockCmd( SCU_PWM_CLKEN, ENABLE );
    PWM_GPIO_Init();

    PWM_ChannelConfig(PWM_CHANNEL2);
	
    PWM_ChannelEnable(PWM_CHANNEL2);
}
