/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-SA
  * @brief          : Main program body
  * @gcc            : toolchain version   5 or 6
  * This file provides example code for Spi transmit data by dma.
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
	* basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */

#include "include.h"
#include "systick.h"
#include "board.h"

#define LSBMode 1
#define MSBMode 0
extern uint8_t g_SPI_Rbuffer[COMM_BUFFER_SIZE];
uint8_t data_send[64] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
void SPI_MasterInit(SPI_Type *p_spi);
void SPI_SlaveInit(SPI_Type *p_spi);
void GPIO_TestInit(void);

/**
  * @brief: main
  * @note: None.
  * @param: None.
  * @retval: None
  */
int	main()
{
    SCU_SetSysClock(CLK_PLL_USE_HRC_LRC, HRC_16M, NO_XTAL, PLL_72M, SCU_SYS_Div1);
    delay_init();
    GPIO_TestInit();
    SPI_MasterInit(SPI0);
    SPI_SlaveInit(SPI1);

    while(1)
    {
        if(GPIO_ReadInputDataBit(KEY_BUTTON_GPIO_PORT, KEY_BUTTON_PIN) == 0)
        {
            GPIO_SetBits(LED_GPIO_PORT, LED_PIN);
            SPI_DmaTransmission(SPI0, data_send, sizeof(data_send));
            delay_ms(1000);
        }
        if(g_SPI_Rbuffer[0] == 0x01 && g_SPI_Rbuffer[63] == 0x00)
        {
            GPIO_ResetBits(LED_GPIO_PORT, LED_PIN);
            g_SPI_Rbuffer[0]=0;
        }
    }
}

/**
  * @brief: SPI_MasterInit
  * @note: None.
  * @param: * p_spi: SPI base address, which can be configured as SPI0 or SPI1.
  * @retval: None
  */
void SPI_MasterInit(SPI_Type *p_spi)
{
    SPI_Init_T SPI_InitConfig = {0};
    SCU_PeriphClockCmd(SCU_GPIOB_CLKEN | SCU_SPI0_CLKEN, ENABLE);
    GPIO_Init(GPIOB, GPIO_PIN8, GPIO_MODE_AF);                   /*CS*/
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN8, GPIO_ALT_SPI0);
    GPIO_Init(GPIOB, GPIO_PIN4, GPIO_MODE_AF);                   /*SCK*/
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN4, GPIO_ALT_SPI0);
    GPIO_Init(GPIOB, GPIO_PIN9, GPIO_MODE_AF);                   /*MOSI*/
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN9, GPIO_ALT_SPI0);
    GPIO_Init(GPIOB, GPIO_PIN7, GPIO_MODE_AF);                   /*MISO*/
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN7, GPIO_ALT_SPI0);

    SPI_InitConfig.Mode = SPI_Mode_Master;
    SPI_InitConfig.CPOL_NCPHA_Mode = SPI_CPOL_NCPHA_Mode_0;
    SPI_InitConfig.Transmit_Mode = SPI_DMA_Enable;
    SPI_InitConfig.Data_Bit = SPI_CSR_BITS_8bit;
    SPI_InitConfig.BaudRate = 400000;                            /*400KHz*/
    SPI_Init(p_spi, &SPI_InitConfig);
}

/**
 * @brief   SPI_SlaveInit
 * @param   *p_spi: SPI base address, which can be configured as SPI0 or SPI1.
 * @retval  None
 */
void SPI_SlaveInit(SPI_Type *p_spi)
{
    SPI_Init_T SPI_InitConfig;
    SCU_PeriphClockCmd(SCU_GPIOA_CLKEN | SCU_SPI1_CLKEN, ENABLE);
    GPIO_Init(GPIOA, GPIO_PIN12, GPIO_MODE_AF);                   /*CS*/
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN12, GPIO_ALT_SPI1);
    GPIO_Init(GPIOA, GPIO_PIN13, GPIO_MODE_AF);                   /*SCK*/
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN13, GPIO_ALT_SPI1);
    GPIO_Init(GPIOA, GPIO_PIN9, GPIO_MODE_AF);                    /*MOSI*/
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN9, GPIO_ALT_SPI1);
    GPIO_Init(GPIOA, GPIO_PIN10, GPIO_MODE_AF);                   /*MISO*/
    GPIO_PinRemapConfig(GPIOA, GPIO_PIN10, GPIO_ALT_SPI1);

    SPI_InitConfig.Mode = SPI_Mode_Slave;
    SPI_InitConfig.Transmit_Mode = SPI_DMA_Enable;
    SPI_InitConfig.CPOL_NCPHA_Mode = SPI_CPOL_NCPHA_Mode_0;
    SPI_InitConfig.Data_Bit = SPI_CSR_BITS_8bit;
    SPI_Init(p_spi, &SPI_InitConfig);
}

/**
 * @brief       GPIO Init for led and key function
 * @param       None
 * @retval      None
 *
 */
void GPIO_TestInit(void)
{
    SCU_PeriphClockCmd(SCU_GPIOA_CLKEN | SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(LED_GPIO_PORT, LED_PIN, GPIO_MODE_OUTPUT_PP);
    GPIO_Init(KEY_BUTTON_GPIO_PORT, KEY_BUTTON_PIN, GPIO_MODE_INPUT_PU);
}

/**
  * @brief: SPI_data_LSBShift, data shift to LSB mode
  * @param: data:data for shifting,from MSB to LSB.
  * @retval: LSB data
  */
uint16_t SPI_data_LSBShift(uint16_t data)
{
    uint8_t i = 0;
    uint16_t  temp = 0;
    temp = data;
    data = 0;

    for(i = 0; i < 16; i++)
    {
        if((temp & 0x8000) == 1)
        {
            data |= (1 << i);
            temp = temp << 1;
        }
    }

    return data;
}

/**
  * @brief: SPI data transmission, Byte mode
  * @note: Byte mode, for data sending and receiving.
  * @param: *p_spi: SPI base address, which can be configured as SPI0 or SPI1.
  * @param: data:data for sending.
  * @param: data_mode:LSB or MSB mode,MSB default,0 MSB,1 LSB.
  * @retval: None
  */
uint16_t SPI_SendByte(SPI_Type *p_spi, uint16_t data, uint8_t data_shift_mode)
{
    uint16_t  temp = 0;
    uint16_t counter = 0;

    if(data_shift_mode == 1)
    {
        data = SPI_data_LSBShift(data);
    }

    while((!p_spi->SPI_SR & SPI_SR_TXEMPTY))
    {
        counter++;

        if(counter > 0xFFF0)
            return falsefp;
    }

    p_spi->SPI_TDR = data;

    while(!(p_spi->SPI_SR & SPI_SR_RDRF))
    {
        counter++;

        if(counter > 0xFFF0)
            return falsefp;
    }

    temp = p_spi->SPI_RDR;

    if(data_shift_mode == 1)
    {
        temp = SPI_data_LSBShift(temp);
    }

    return temp;
}
