/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-SA
  * @brief          : main program file body.
  * @gcc            : toolchain version 5 or 6
  *
  ******************************************************************************
  * @attention
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */

#include "include.h"
#include  "systick.h"
#include  "board.h"

uint8_t crc_idata[8] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
uint32_t result;

void CRC8_TestInit(void);
void CRC16_TestInit(void);
void CRC32_TestInit(void);
uint32_t CRC_Cal(CRC_WIDTH_Enum width, uint8_t *data, uint32_t BufferLength);
void UART_TestInit(void);

int	main()
{

    SCU_SetSysClock(CLK_PLL_USE_HRC_LRC, HRC_16M, NO_XTAL, PLL_72M, SCU_SYS_Div1);
    delay_init();
    UART_TestInit();
    CRC8_TestInit();
    result = CRC_Cal(CRC_8, crc_idata, sizeof(crc_idata));

    while(1)
    {
        delay_ms(1000);
        printf("crc calculate result:%x\n", result);
    }
}

/**
  * @brief       CRC8 Init
 * @param       None
 * @retval      None
 *
 */
void CRC8_TestInit(void)
{

    CRC_Init_T CRC_InitConfig;
    SCU_PeriphClockCmd(SCU_CRC_CLKEN, ENABLE);

    /*The polynomial model selected for this test item is named CRC-8*/
    CRC_InitConfig.width_type = CRC_8;
    CRC_InitConfig.CRC_XOR = 0;
    CRC_InitConfig.inout_type = IN_AND_OUT_NORMAL;
    CRC_InitConfig.InitValue = 0;
    CRC_InitConfig.POLY = 0x07;/*This value supports only any odd value*/
    CRC_InitConfig.enable_type = CRC_ENABLE;
    CRC_Init(&CRC_InitConfig);
}

/**
  * @brief       CRC16 Init
  * @note        When the 16-bit CRC is used for data verification, the number of
  *              data involved in the calculation must be an even number
  * @param       None
  * @retval      None
  */
void CRC16_TestInit(void)
{

    CRC_Init_T CRC_InitConfig;
    SCU_PeriphClockCmd(SCU_CRC_CLKEN, ENABLE);

    /*The polynomial model selected for this test item is named CRC-16-MODBUS*/
    CRC_InitConfig.width_type = CRC_16;
    CRC_InitConfig.CRC_XOR = 0x0000;
    CRC_InitConfig.inout_type = IN_REVERSE_OUT_REVERSE;
    CRC_InitConfig.InitValue = 0xFFFF;
    CRC_InitConfig.POLY = 0x8005;/*This value supports only any odd value*/
    CRC_InitConfig.enable_type = CRC_ENABLE;
    CRC_Init(&CRC_InitConfig);

}

/**
  * @brief       CRC32 Init
  * @note        When using the 32-bit CRC for data verification, note that the
  *              data involved in the calculation must be an entire word
  * @param       None
  * @retval      None
  */
void CRC32_TestInit(void)
{

    CRC_Init_T CRC_InitConfig;
    SCU_PeriphClockCmd(SCU_CRC_CLKEN, ENABLE);

    /*The polynomial model selected for this test item is named CRC-32*/
    CRC_InitConfig.width_type = CRC_32;
    CRC_InitConfig.CRC_XOR = 0xFFFFFFFF;
    CRC_InitConfig.inout_type = IN_REVERSE_OUT_REVERSE;
    CRC_InitConfig.InitValue = 0xFFFFFFFF;
    CRC_InitConfig.POLY = 0x04C11DB7;/*This value supports only any odd value*/
    CRC_InitConfig.enable_type = CRC_ENABLE;
    CRC_Init(&CRC_InitConfig);

}

/**
  * @brief       CRC_Cal  Data check calculation function
  * @param       CRC_WIDTH_Enum width:Data calculation bit width, optional CRC_8, CRC_16,CRC_32
  * @retval      Calculation results
  */
uint32_t CRC_Cal(CRC_WIDTH_Enum width, uint8_t *data, uint32_t BufferLength)
{
    return CRC_Calculate(width, data, BufferLength);
}


/**
  * @brief       UART_TestInit  Serial port configuration initialization
  * @param       None
  * @retval      None
  */
void UART_TestInit(void)
{

    UART_Init_T UART_InitConfig;
    SCU_PeriphClockCmd(UART_PORT_RX_GPIO_CLK | UART_PORT_TX_GPIO_CLK | UART_PORT_CLK, ENABLE);
    SCU_ResetPeriphClock(SCU_UART0_RST);

    GPIO_Init(UART_PORT_RX_GPIO_PORT, UART_PORT_RX_PIN, GPIO_MODE_AF);
    GPIO_PinRemapConfig(UART_PORT_RX_GPIO_PORT, UART_PORT_RX_PIN, GPIO_ALT_RX0);
    GPIO_Init(UART_PORT_TX_GPIO_PORT, UART_PORT_TX_PIN, GPIO_MODE_AF);
    GPIO_PinRemapConfig(UART_PORT_TX_GPIO_PORT, UART_PORT_TX_PIN, GPIO_ALT_TX0);
    UART_InitConfig.BaudRate = 115200;
    UART_InitConfig.Parity = UART_PARITY_NO;
    UART_InitConfig.StopBits = UART_STOPBITS_1;
    UART_InitConfig.WordLength = UART_DATA_8_BIT;
    UART_InitConfig.TimeOut = 10;
    UART_Init(UART_PORT, &UART_InitConfig);

}
