/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-SA
  * @brief          : main program file.
  * @gcc            : toolchain version 5 or 6
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * This firmware is for guidance only and is designed to provide customers with 
  * basic chip information support.
  *
  * Copyright (c) 2023 sinomcu.com All rights reserved.
  *
  ******************************************************************************
  */  
	
#include "include.h"
#include  "systick.h"
#include "board.h"

#define i2cSlaveAddr 0x50
#define sendbufSize 300
uint8_t sendbuf[sendbufSize];
uint16_t sendbufIdx ;
uint16_t sendbufLen ;
uint16_t memBufIdx = 0;
uint8_t memBuf[sendbufSize];

void I2C1MasterCfg(void);
void I2C0SlaveCfg(void);
void I2C0_state(void);
void GPIO_TestInit(void);

uint8_t result, I2C_Buffer[8] = {0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8};

int	main()
{
    int i;
    SCU_SetSysClock(CLK_PLL_USE_HRC_LRC, HRC_16M, NO_XTAL, PLL_72M, SCU_SYS_Div1);
    delay_init();
    I2C1MasterCfg();
    GPIO_TestInit();
    I2C0SlaveCfg();

    while(1)
    {
        if(GPIO_ReadInputDataBit(KEY_BUTTON_GPIO_PORT, KEY_BUTTON_PIN) == 0)
        {
            GPIO_SetBits(LED_GPIO_PORT, LED_PIN);
            I2CStart(I2C1);
            result = I2C_Send_address(I2C1, 0X50, I2C_ACK);

            if(result == falsefp)
                return  falsefp;

            for(i = 0; i < sizeof(I2C_Buffer); i++)
            {
                I2C_Send_Byte(I2C1, I2C_Buffer[i], I2C_ACK);
            }

            I2CStop(I2C1);
            delay_ms(1000);
            GPIO_ResetBits(LED_GPIO_PORT, LED_PIN);
        }
    }
}

/**
  * @brief  Configure I2C1 Master pin multiplexing
  * @param  None
  * @retval None
  */
void I2C1MasterCfg(void)
{
    I2C_INIT_T i2c;
    SCU_Unlock();
    SCU_PeriphClockCmd(SCU_IIC1_CLKEN | SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(GPIOB, GPIO_PIN8, GPIO_MODE_AF);
    GPIO_Init(GPIOB, GPIO_PIN9, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN8, GPIO_ALT_SDA1);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN9, GPIO_ALT_SCL1);
    GPIO_PullUpConfig(GPIOB, GPIO_PIN8);
    GPIO_PullUpConfig(GPIOB, GPIO_PIN9);
    GPIO_SetOD(GPIOB, GPIO_PIN8, ENABLE);
    GPIO_SetOD(GPIOB, GPIO_PIN9, ENABLE);

    i2c.mode = MASTER_MODE;
    i2c.frequency = 10000;
    i2c.duty = 50;
    I2C_Init(I2C1, &i2c);
}

/**
  * @brief  Configure I2C0 Master pin multiplexing
  * @param  None
  * @retval None
  */
void I2C0SlaveCfg(void)
{
    I2C_INIT_T i2c;
    SCU_PeriphClockCmd(SCU_IIC0_CLKEN | SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(GPIOB, GPIO_PIN2, GPIO_MODE_AF);
    GPIO_Init(GPIOB, GPIO_PIN1, GPIO_MODE_AF);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN2, GPIO_ALT_SDA0);
    GPIO_PinRemapConfig(GPIOB, GPIO_PIN1, GPIO_ALT_SCL0);
    GPIO_PullUpConfig(GPIOB, GPIO_PIN2);
    GPIO_PullUpConfig(GPIOB, GPIO_PIN1);
    GPIO_SetOD(GPIOB, GPIO_PIN1, ENABLE);
    GPIO_SetOD(GPIOB, GPIO_PIN2, ENABLE);

    i2c.mode = SLAVER_MODE;
    i2c.frequency = 10000;
    i2c.duty = 50;
    i2c.address = i2cSlaveAddr;
    I2C_Init(I2C0, &i2c);
    I2C_Enable_IT(I2C0, SI);
    NVIC_EnableIRQ(IIC0_IRQn);
}

/**
  * @brief  Configure I2C1 Slave state.
  * @param  None
  * @retval None
  */
void I2C0_state(void)
{
    uint8_t Receive = I2C0->RDR;

    switch (I2C0->SR)
    {
        case 0x15:/*The slave has received the address and returned ACK*/

            I2C_Slave_Send_Data(I2C0, memBuf[memBufIdx], I2C_ACK);
            break;

        case 0x17:/*Slave has sent data and received ACK*/
            I2C_Slave_Send_Data(I2C0, memBuf[memBufIdx], I2C_ACK);
            break;

        case 0x10:/*Slave has sent data and received ACK*/
        case 0x11:/*Slave has sent data and received NACK*/
            memBuf[memBufIdx++] = Receive;
            I2C0_SETFLAG(I2C_ACK);
            break;

        case 0x0C:/*The slave has received address+w and returned ACK*/
            memBufIdx = 0;
            I2C0_SETFLAG(I2C_ACK);
            break;

        case 0x1A:/*Slave has received STOP*/
            I2C0_SETFLAG(I2C_ACK);
            memBufIdx = 0;
            break;

        default:
            I2C0_SETFLAG(I2C_ACK);
            sendbufIdx = 0;
            break;
    }
}

/**
 * @brief       GPIO Init for led and key function
 * @param       None
 * @retval      None
 *
 */
void GPIO_TestInit(void)
{
    SCU_PeriphClockCmd(SCU_GPIOA_CLKEN | SCU_GPIOB_CLKEN, ENABLE);
    GPIO_Init(LED_GPIO_PORT, LED_PIN, GPIO_MODE_OUTPUT_PP);
    GPIO_Init(KEY_BUTTON_GPIO_PORT, KEY_BUTTON_PIN, GPIO_MODE_INPUT_PU);
}



